
/* 
 * This include file contains macros, declarations, etc. required
 * for the mpeg-2 package that should really be extensions/modifications
 * of other packages. They have been broken out into this separate file
 * so to facilitate integration of the mpeg-2 package into the official
 * Cornell distributions. 
 */

/* largebitfill & LargePeekBits
 * 
 * The following macros are additions to the macros in bitparser.h
 * to allow for peeking more than 16 bits at a time. 
 */

#define largebitfill(bp)       \
{                              \
    unsigned char _w;          \
    bitfill(bp);               \
    if (bp->bitCount < 24) {   \
       _w = *(bp->offsetPtr++);\
       bp->currentBits <<= 8;  \
       bp->currentBits |= _w;  \
       bp->bitCount += 8;      \
    }                          \
}
  
#define Bp_LargePeekBits(bp, num, retval)               \
{                                                       \
    largebitfill(bp);                                   \
    (retval) = (bp->currentBits >> (bp->bitCount-(num))) & bitmask[(int)num];\
}

/* ScGetBlock
 * 
 * This macro is used to get a pointer to a specific ScBlock of an
 * ScImage. It should be moved to dvmbasic.h
 */

#define ScGetBlock(b,i,j) &((b)->firstBlock[((j)*((b)->parentWidth))+(i)])

/* ByteField
 *
 * Declaration of ByteField command which creates a virtual ByteImage
 * from a parent ByteImage comprised of every other line of the image.
 * This declaration should be moved to dvmbasic.h. The definition of
 * this function is currently in mpeg2compat.c in the mpeg2 package and
 * should be moved to the basic package.
 */

ByteImage *ByteField(ByteImage *, int);

