#include "dvmmpeg2.h"

Mpeg2Cache *
Mpeg2CacheNew(int num_entries)
{
  Mpeg2Cache *new_cache;
  Mpeg2CacheEntry *entry;
  int i;

  if (num_entries < 1) {
    return NULL;
  }

  new_cache = (Mpeg2Cache *) malloc(sizeof(Mpeg2Cache));
  new_cache->num_entries = num_entries;
  new_cache->free_entries = NULL;
  new_cache->cur_entries = NULL;

  for (i=0; i<num_entries; i++) {
    entry = (Mpeg2CacheEntry *) malloc(sizeof(Mpeg2CacheEntry));
    entry->frame_num = 0;
    entry->y = NULL; 
    entry->u = NULL;
    entry->v = NULL;

    entry->next = new_cache->free_entries;
    new_cache->free_entries = entry;
  }

  return new_cache;
}

void
Mpeg2CachePutEntry(Mpeg2Cache *cache,
		   unsigned int framenum, 
		   ByteImage *y,
		   ByteImage *u,
		   ByteImage *v)
{
  Mpeg2CacheEntry *entry, *prev_entry;

  if (cache->free_entries == NULL) {
    prev_entry = NULL;
    entry = cache->cur_entries;
    while (entry->next != NULL) {
      prev_entry = entry;
      entry = entry->next;
    }
    if (prev_entry != NULL) {
      prev_entry->next = NULL;
    } else {
      cache->cur_entries = NULL;
    }
    /* Need to free ByteImage memory associated with
     * this entry
     */
    ByteFree(entry->y);
    ByteFree(entry->u);
    ByteFree(entry->v);
  } else {
    entry = cache->free_entries;
    cache->free_entries = entry->next;
    entry->next = NULL;
  }

  entry->frame_num = framenum;
  entry->y = y;
  entry->u = u;
  entry->v = v;

  entry->next = cache->cur_entries;
  cache->cur_entries = entry;
}

/******************************************
 * Mpeg2CacheGetEntry
 *
 * Returns 1 if cache entry with corresponding framenum
 * found, 0 if not found. Parameters y, u, v should be
 * pointer to ByteImage pointers which are filled with
 * the appropriate value if found. 
 *******************************************/

int 
Mpeg2CacheGetEntry(Mpeg2Cache *cache, 
		   unsigned int framenum, 
		   ByteImage **y, 
		   ByteImage **u, 
		   ByteImage **v)
{
  Mpeg2CacheEntry *entry, *prev_entry;

  prev_entry = NULL;
  entry = cache->cur_entries;
  while (entry != NULL) {
    if (entry->frame_num == framenum) {
      break;
    }
    prev_entry = entry;
    entry = entry->next;
  }
  if (entry == NULL) {
    return 0;
  }

  if (prev_entry != NULL) {
    prev_entry->next = entry->next;
  } else {
    cache->cur_entries = entry->next;
  }

  *y = entry->y;
  *u = entry->u;
  *v = entry->v;
  entry->y = entry->u = entry->v = NULL;
  entry->next = cache->free_entries;
  cache->free_entries = entry;
  return 1;
}

void
Mpeg2CacheFree(Mpeg2Cache *cache) 
{
  Mpeg2CacheEntry *entry, *next_entry;

  entry = cache->cur_entries;
  while (entry != NULL) {
    next_entry = entry->next;
    ByteFree(entry->y);
    ByteFree(entry->u);
    ByteFree(entry->v);
    free((char *) entry);
    entry = next_entry;
  }
  entry = cache->free_entries;
  while (entry != NULL) {
    next_entry = entry->next;
    free((char *) entry);
    entry = next_entry;
  }
  free((char *) cache);
}
