#include <stdio.h>
#include "mpeg2Int.h"


Mpeg2RefIdx *
Mpeg2RefIdxNew() 
{
  Mpeg2RefIdx *new_idx;

  new_idx = (Mpeg2RefIdx *) malloc(sizeof(Mpeg2RefIdx));
  new_idx->num_recs = 1000;
  new_idx->max_size = 0;
  new_idx->recs = (Mpeg2RefIdxRecord *) malloc(sizeof(Mpeg2RefIdxRecord)*
					       new_idx->num_recs);
  return new_idx;
}

void
Mpeg2RefIdxFree(Mpeg2RefIdx *idx) 
{
  if (idx->recs != NULL) {
    free((char *) idx->recs);
  }
  free((char *) idx);
}

Mpeg2RefIdxRecord *
Mpeg2RefIdxGetRecord(Mpeg2RefIdx *idx,
		     unsigned int frame_num)
{
  if (idx == NULL) {
    return NULL;
  }

  if (frame_num >= idx->num_recs) {
    return NULL;
  }

  return &(idx->recs[frame_num]);
}

int
Mpeg2ReadRefIndex(Mpeg2RefIdx *idx,
		  char *idx_filename)
{
  FILE *fp;
  unsigned int frame_num, offset, size, rec_idx;
  char frame_type;
  Mpeg2RefIdxRecord *rec;

  if (idx == NULL) {
    return -1;
  }
		    
  fp = fopen(idx_filename, "r");

  if (fp == NULL) {
    return -1;
  }

  while((fscanf(fp, " %u %u %u %c", &frame_num, &offset, &size, &frame_type)) == 4) {
    rec_idx = frame_num;
    if (rec_idx >= idx->num_recs) {
      idx->num_recs = rec_idx * 2;
      idx->recs = (Mpeg2RefIdxRecord *) realloc(idx->recs, idx->num_recs * sizeof(Mpeg2RefIdxRecord));
    }
    rec = &(idx->recs[rec_idx]);
    rec->offset = offset;
    rec->size = size;
    if (rec->size > idx->max_size) {
      idx->max_size = rec->size;
    }
    if (frame_type == 'I') {
      rec->type_flag = I_FRAME;
    } else {
      unsigned int ref1;

      fscanf(fp, " %u", &ref1);
      rec->ref1 = ref1;

      if (frame_type == 'P') {
	rec->type_flag = P_FRAME;
      } else {
	unsigned int ref2;

	fscanf(fp, " %u", &ref2);
	rec->ref2 = ref2;
	
	rec->type_flag = B_FRAME;
      }
    }
  }

  fclose(fp);
  return 1;
}



