
#include "tclDvmMpeg2.h"
#include "mpeg2Int.h"

/* Declare init routine defined in mpeg2_compat_cmd.c that 
 * does initialization for commands required for mpeg2 package
 * that belong in other packages when integrated with original
 * Cornell source.
 */

int Mpeg2CompatInit(Tcl_Interp *interp);

EXTERN EXPORT(int, Tcldvmmpeg2_Init) _ANSI_ARGS_((Tcl_Interp * interp));
EXTERN void CreateCommands _ANSI_ARGS_((Tcl_Interp *interp, Commands cmd[], int cmdsize));

static Commands mpeg2cmd[] = 
{ 
  {"mpeg2_seq_hdr_new", Mpeg2SeqHdrNewCmd, NULL, NULL},
  {"mpeg2_seq_hdr_free", Mpeg2SeqHdrFreeCmd, NULL, NULL},
  {"mpeg2_seq_hdr_find", Mpeg2SeqHdrFindCmd, NULL, NULL},
  {"mpeg2_seq_hdr_parse", Mpeg2SeqHdrParseCmd, NULL, NULL},
  {"mpeg2_seq_hdr_ext_find", Mpeg2SeqHdrExtFindCmd, NULL, NULL},
  {"mpeg2_seq_hdr_ext_parse", Mpeg2SeqHdrExtParseCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_aspect_ratio", Mpeg2SeqHdrGetAspectRatioCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_chroma_format", Mpeg2SeqHdrGetChromaFormatCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_frame_rate", Mpeg2SeqHdrGetFrameRateCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_width", Mpeg2SeqHdrGetWidthCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_height", Mpeg2SeqHdrGetHeightCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_mb_width", Mpeg2SeqHdrGetMBWidthCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_mb_height", Mpeg2SeqHdrGetMBHeightCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_profile", Mpeg2SeqHdrGetProfileCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_level", Mpeg2SeqHdrGetLevelCmd, NULL, NULL},
  {"mpeg2_seq_hdr_is_progressive", Mpeg2SeqHdrIsProgressiveCmd, NULL, NULL},
  {"mpeg2_seq_hdr_is_low_delay", Mpeg2SeqHdrIsLowDelayCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_bit_rate", Mpeg2SeqHdrGetBitRateCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_buffer_size", Mpeg2SeqHdrGetVBVBufferSizeCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_lum_iqt", Mpeg2SeqHdrGetLumIQuantMatrixCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_lum_niqt", Mpeg2SeqHdrGetLumNonIQuantMatrixCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_chr_iqt", Mpeg2SeqHdrGetChrIQuantMatrixCmd, NULL, NULL},
  {"mpeg2_seq_hdr_get_chr_niqt", Mpeg2SeqHdrGetChrNonIQuantMatrixCmd, NULL, NULL},
  {"mpeg2_any_start_code_find", Mpeg2AnyStartCodeFindCmd, NULL, NULL},
  {"mpeg2_get_curr_start_code", Mpeg2GetCurrStartCodeCmd, NULL, NULL},
  {"mpeg2_pic_hdr_new", Mpeg2PicHdrNewCmd, NULL, NULL},
  {"mpeg2_pic_hdr_free", Mpeg2PicHdrFreeCmd, NULL, NULL},
  {"mpeg2_pic_hdr_find", Mpeg2PicHdrFindCmd, NULL, NULL},
  {"mpeg2_pic_hdr_parse", Mpeg2PicHdrParseCmd, NULL, NULL},
  {"mpeg2_pic_hdr_ext_find", Mpeg2PicHdrExtFindCmd, NULL, NULL},
  {"mpeg2_pic_hdr_ext_parse", Mpeg2PicHdrExtParseCmd, NULL, NULL},
  {"mpeg2_get_curr_ext_type", Mpeg2GetCurrExtensionTypeCmd, NULL, NULL},
  {"mpeg2_skip_to_next_start_code", Mpeg2SkipToNextStartCodeCmd, NULL, NULL},
  {"mpeg2_pic_hdr_get_type", Mpeg2PicHdrGetPictureTypeCmd, NULL, NULL},
  {"mpeg2_pic_hdr_get_structure", Mpeg2PicHdrGetStructureCmd, NULL, NULL},
  {"mpeg2_pic_hdr_get_temporal_ref", Mpeg2PicHdrGetTemporalReferenceCmd, NULL, NULL},
  {"mpeg2_mbinfo_array_new", Mpeg2MBInfoArrayNewCmd, NULL, NULL},
  {"mpeg2_mbinfo_array_free", Mpeg2MBInfoArrayFreeCmd, NULL, NULL},
  {"mpeg2_pic_i_field_parse", Mpeg2ParseIFieldCmd, NULL, NULL},
  {"mpeg2_pic_i_frame_parse", Mpeg2ParseIFrameCmd, NULL, NULL},
  {"mpeg2_sc_i_field_to_byte", Mpeg2ScIFieldToByteCmd, NULL, NULL},
  {"mpeg2_sc_i_frame_to_byte", Mpeg2ScIFrameToByteCmd, NULL, NULL},
  {"mpeg2_pic_p_field_parse", Mpeg2ParsePFieldCmd, NULL, NULL},
  {"mpeg2_pic_p_frame_parse", Mpeg2ParsePFrameCmd, NULL, NULL},
  {"mpeg2_sc_p_field_to_byte", Mpeg2ScPFieldToByteCmd, NULL, NULL},
  {"mpeg2_sc_p_frame_to_byte", Mpeg2ScPFrameToByteCmd, NULL, NULL},
  {"mpeg2_mbinfo_info", Mpeg2MBInfoInfoCmd, NULL, NULL},
  {"mpeg2_pic_b_field_parse", Mpeg2ParseBFieldCmd, NULL, NULL},
  {"mpeg2_sc_b_field_to_byte", Mpeg2ScBFieldToByteCmd, NULL, NULL},
  {"mpeg2_sc_b_frame_to_byte", Mpeg2ScBFrameToByteCmd, NULL, NULL},
  {"mpeg2_pic_b_frame_parse", Mpeg2ParseBFrameCmd, NULL, NULL},
  {"mpeg2_cellmgr_new", Mpeg2CellMgrNewCmd, NULL, NULL},
  {"mpeg2_cellmgr_free", Mpeg2CellMgrFreeCmd, NULL, NULL},
  {"mpeg2_cellmgr_get_width", Mpeg2CellMgrGetWidthCmd, NULL, NULL},
  {"mpeg2_cellmgr_get_height", Mpeg2CellMgrGetHeightCmd, NULL, NULL},
  {"mpeg2_cellmgr_get_frame_as_planes", Mpeg2CellMgrGetFrameAsPlanesCmd, NULL, NULL},
  {"sc_is_skipped_mb", Mpeg2ScIsSkippedMBCmd, NULL, NULL},
  {"sc_is_intra_mb", Mpeg2ScIsIntraMBCmd, NULL, NULL},
  {"mbinfo_get_motion_vector_info", Mpeg2MBInfoGetMotionVectorInfoCmd, NULL, NULL},
  {"mbinfo_get_all", Mpeg2MBInfoGetAllCmd, NULL, NULL}
};

#ifdef __WIN32__
BOOL APIENTRY
DllEntryPoint(hInst, reason, reserved)
     HINSTANCE hInst;            /* Library instance handle. */
     DWORD reason;               /* Reason this function is being called. */
     LPVOID reserved;            /* Not used. */
{
  return TRUE;
}
#endif          /* __WIN32__ */

/*
 *----------------------------------------------------
 *
 * Tcldvmmpeg2_Init -- 
 *
 *    This procedure initializes the MPEG-2 package
 *
 *-----------------------------------------------------
 */

EXPORT(int, Tcldvmmpeg2_Init) (Tcl_Interp *interp)
{
  if (Tcl_PkgRequire(interp, "DvmBasic", "1.0", 1) == NULL) {
    sprintf(interp->result, "Error loading tclDvmBasic package");
    return TCL_ERROR;
  }
  if (Tcl_PkgRequire(interp, "DvmColor", "1.0", 1) == NULL) {
    sprintf(interp->result, "Error loading tclDvmColor package");
    return TCL_ERROR;
  }
    
  CreateCommands(interp, mpeg2cmd, sizeof(mpeg2cmd));

  /* The following call will not be needed after this code
   * is integrated with the Cornell source.
   */
  Mpeg2CompatInit(interp);

  Initialize_Fast_IDCT();
  return Tcl_PkgProvide(interp, "DvmMpeg2", "1.0");
}
