/*
  media-filesystem library
  tridge@samba.org, January 2001
  released under the Gnu GPL v2
*/

#include "mfs.h"

void read_all(int fd, void *buf, int size)
{
	while (size) {
		int n = read(fd, buf, size);
		if (n <= 0) {
			// fprintf(stderr,"ERROR: eof in read_all\n");
			exit(1);
		}
		buf += n;
		size -= n;
	}
}

void write_all(int fd, void *buf, int size)
{
	while (size) {
		int n = write(fd, buf, size);
		if (n <= 0) {
			// fprintf(stderr,"ERROR: eof in write_all\n");
			exit(1);
		}
		buf += n;
		size -= n;
	}
}

void byte_swap(void *p, char *desc)
{
	int n, i;
	while (*desc) {
		switch (*desc) {
		case 'i': {
			u32 *v;
			n = strtol(desc+1, &desc, 10);
			v = p;
			for (i=0;i<n;i++) v[i] = ntohl(v[i]);
			p = (void *)(v+n);
			break;
		}

		case 's': {
			u16 *v;
			n = strtol(desc+1, &desc, 10);
			v = p;
			for (i=0;i<n;i++) v[i] = ntohs(v[i]);
			p = (void *)(v+n);
			break;
		}

		case 'b': {
			n = strtol(desc+1, &desc, 10);
			p += n;
			break;
		}
		}
		while (*desc == ' ') desc++;
	}
}


u64 llseek(int fd, u64 offset, int whence)
{
	u64 result;
#ifdef __linux__
	if (syscall(__NR__llseek, fd, (u32)(offset>>32), (u32)(offset&0xffffffff), &result, whence) != 0) {
		fprintf(stderr,"llseek failed\n");
		exit(1);
	}
#else
	if ((result = lseek(fd, offset, whence)) != 0) {
		fprintf(stderr,"llseek failed\n");
		exit(1);
	}
#endif
	return result;
}

u32 read_sectors(int fd, void *buf, u32 sector, u32 count)
{
#ifdef TIVO
	struct FsIovec vec;
	struct FsIoRequest req;

	vec.pb = buf;
	vec.cb = count*SECTOR_SIZE;
	req.sector = sector;
	req.num_sectors = count;
	req.deadline = 0;
	
	return syscall(__NR_readsectors, fd, &vec, 1, &req);
#else
	llseek(fd, ((u64)sector)<<SECTOR_SHIFT, SEEK_SET);
	return read(fd, buf, count * SECTOR_SIZE)>>SECTOR_SHIFT;
#endif
}

u32 write_sectors(int fd, void *buf, u32 sector, u32 count)
{
#ifdef TIVO
	struct FsIovec vec;
	struct FsIoRequest req;

	vec.pb = buf;
	vec.cb = count*SECTOR_SIZE;
	req.sector = sector;
	req.num_sectors = count;
	req.deadline = 0;
	
	return syscall(__NR_writesectors, fd, &vec, 1, &req);
#else
	llseek(fd, ((u64)sector)<<SECTOR_SHIFT, SEEK_SET);
	return write(fd, buf, count * SECTOR_SIZE)>>SECTOR_SHIFT;
#endif
}

int bitcount32(u32 x)
{
	int count;
	for (count=0; x; count++)
		x &= (x-1);
	return count;
}
