/* plays a stream from a remote TiVo
 *
 * tivo@wingert.org, February 2003
 *
 * Copyright (C) 2003 Christopher R. Wingert
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/mman.h>
#include <unistd.h>
#include <time.h>
#include "mfs.h"

char *amorpm[] =
{ "am", "pm" };

char *dayofweek[] =
{ "Sun", "Mon", "Tues", "Wed", "Thur", "Fri", "Sat" };


void fsidtoparts( char *fsid, char *sliceIds[], int *max )
{
   struct mfs_obj_attr *obj;
   int                 pcount;
   int                 index;
   int                 ifsid;
   char                calcFsid[ 80 ];
   int                 iSliceId;
   char                sSliceId[ 16 ];

   ifsid = mfs_resolve( fsid );

   obj = query_object( ifsid, "Part", &pcount );
   for( index = 0 ; index < pcount ; index ++ )
   {
      sprintf( calcFsid, "Part/%d/File", obj[ index ].subobj );
      iSliceId = query_int( ifsid, calcFsid );
      sprintf( sSliceId, "%d", iSliceId );
      sliceIds[ index ] = malloc( strlen( sSliceId ) );
      strcpy( sliceIds[ index ], sSliceId );
   }
   free( obj );
   *max = pcount;
}


void my_query_streams( char *path )
{
   struct mfs_dirent   *dir;
   u32                 count;
   int                 index;
   u32                 i;
   time_t              recTime;
   struct tm           recTimetm;
   struct tm           airTimetm;
   int                 ampm = 0;
   char                *sliceIds[ 32 ];
   int                 numSlices;
   char                fsid[ 32 ];
   int                 duration;

   /*
    * StartEntry@ 103
    * RecObjFsid@ 1587820
    * FSID@ 1587851,1587853
    * State@ 1
    * Year@
    * AirDate@ 1/1/1970
    * Day@ Tue
    * Date@ 12/17
    * Time@  8:00 am
    * Duration@ 1:00
    * Title@ 1980:
    * Episode@
    * Episidenr@
    * Description@
    * Actors@
    * GuestStars@
    * Host@
    * Director@
    * ExecProd@
    * Producer@
    * Writer@
    * EndEntry@ 103
    */

   dir = mfs_dir( mfs_resolve( path ), &count );
   for ( i = 0 ; i < count ; i++ ) 
   {
      recTime = ( query_int( dir[ i ].fsid, "StartDate" ) * 86400 );
      recTime += query_int( dir[ i ].fsid, "StartTime" ) + 60;
      localtime_r( &recTime, &recTimetm );
      recTimetm.tm_year += 1900;
      ampm = 0;
      if ( recTimetm.tm_hour > 12 )
      {
         ampm = 1;
         recTimetm.tm_hour -= 12;
      }

      recTime = ( query_int( dir[ i ].fsid, "Showing/Program/OriginalAirDate" )
         * 86400 );
      localtime_r( &recTime, &airTimetm );
      airTimetm.tm_year += 1900;

      printf( "StartEntry@ %d\n", count );
      sprintf( fsid, "%d", query_int( dir[ i ].fsid, "Part" ) );
      printf( "RecObjFsid@ %s\n", fsid );
      printf( "FSID@ " );
      fsidtoparts( fsid, sliceIds, &numSlices );
      for( index = 0 ; index < numSlices ; index++ )
      {
         printf( "%s", sliceIds[ index ] );
         if ( index != ( numSlices - 1 ) )
         {
            printf( ", " );
         }
         free( sliceIds[ index ] );
      }
      printf( "\n" );
      printf( "State@ %d\n", query_int( dir[ i ].fsid, "State" ) );
      printf( "Year@ %d\n", recTimetm.tm_year );
      printf( "AirDate@ %d/%d/%d\n", airTimetm.tm_mon + 1, airTimetm.tm_mday, 
        airTimetm.tm_year );
      printf( "Day@ %s\n", dayofweek[ recTimetm.tm_wday ] );
      printf( "Date@ %d/%d\n", recTimetm.tm_mon + 1, recTimetm.tm_mday );
      printf( "Time@ %d:%2.2d %s\n", recTimetm.tm_hour, recTimetm.tm_min, 
         amorpm[ ampm ] );
      duration = query_int( dir[ i ].fsid, "StopDate" ) - 
         query_int( dir[ i ].fsid, "StartDate" );
      duration *= 86400;
      duration += ( query_int( dir[ i ].fsid, "StopTime" ) - 
         query_int( dir[ i ].fsid, "StartTime" ) );
      printf( "Duration@ %d\n", duration );
      printf( "Title@ %s\n", 
         query_string( dir[ i ].fsid, "Showing/Program/Title" ) );
      printf( "Episode@ %s\n", 
         query_string( dir[ i ].fsid, "Showing/Program/EpisodeTitle" ) );
      printf( "Episidenr@ %d\n",
         query_int( dir[ i ].fsid, "Showing/Program/EpisodeNum" ) );
      printf( "Description@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/Description" ) );
      printf( "Actors@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/Actor" ) );
      printf( "GuestStars@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/GuestStar" ) );
      printf( "Host@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/Host" ) );
      printf( "Director@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/Director" ) );
      printf( "ExecProd@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/ExecProducer" ) );
      printf( "Producer@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/Producer" ) );
      printf( "Writer@ %s\n",
         query_string( dir[ i ].fsid, "Showing/Program/Writer" ) );
      printf( "EndEntry@ %d\n", count );
   }
   if ( dir ) mfs_dir_free( dir );
}

int list_streams( void )
{
   char *path;

   mfs_init();

   	if ( mfs_resolve( "/Recording/NowShowingByClassic" ) != 0 ) {
		
		path = "/Recording/NowShowingByClassic";

	} else if (mfs_resolve( "/Recording/NowShowing") != 0 ){

		path = "/Recording/NowShowing";

	} else {
		path = "/Recording/Complete";
	}


#if 0
   if ( mfs_resolve( "/Recording/NowShowing" ) != 0 )
   {
      path = "/Recording/NowShowing";
   }
   else
   {
      path = "/Recording/Complete";
   }
#endif

   printf( "Listing streams in %s\n", path );
   my_query_streams( path );
   return( 0 );
}



int main( int argc, char *argv[] )
{
   list_streams( );
   return( 0 );
}


