#include <sys/wait.h>
#include <string.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <time.h>
#include "mfs.h"

#include "../portability.h"
#include "../../tydemux/tydemux.h"

char *amorpm[] =
{ "am", "pm" };

char *dayofweek[] =
{ "Sun", "Mon", "Tues", "Wed", "Thur", "Fri", "Sat" };

int zone_list[] =
{-5, -6, -7, -8, -9, -10, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, -1, -2, -3, -4, -11, -12};

static fsid_index_t * parse_nowshowing(char * path, int zone);

void fsidtoparts( char *fsid, char *sliceIds[], int *max )
{
   struct mfs_obj_attr *obj;
   int                 pcount;
   int                 index;
   int                 ifsid;
   char                calcFsid[ 80 ];
   int                 iSliceId;
   char                sSliceId[ 16 ];

   ifsid = mfs_resolve( fsid );
   obj = query_object( ifsid, "Part", &pcount );
   for( index = 0 ; index < pcount ; index ++ )
   {
      sprintf( calcFsid, "Part/%d/File", obj[ index ].subobj );
      iSliceId = query_int( ifsid, calcFsid );
      sprintf( sSliceId, "%d", iSliceId );
      sliceIds[ index ] = malloc( strlen( sSliceId ) );
      strcpy( sliceIds[ index ], sSliceId );
   }
   free( obj );
   *max = pcount;
}


static int get_zone(int zone, int daylight) {

	int tz;

	if(zone <= 0) {
		return(0);
	}

	if(daylight == -1) {
		daylight = 0;
	}

	tz = zone_list[zone -1];
	if(daylight) {
		tz++;
	}

	return(tz * 60 * 60);

}


fsid_index_t * parse_nowshowing_tivo() {

	char *path;
	fsid_index_t * fsid_index_list=NULL;
	int fsid;
	int fsid2;
	char 	txtbuf[100];
	int version = 0;
	int zone;
	int daylight;
	mfs_init();

	if ( mfs_resolve( "/Recording/NowShowingByClassic" ) != 0 ) {
		printf("Classic\n");
		path = "/Recording/NowShowingByClassic";

	} else if (mfs_resolve( "/Recording/NowShowing") != 0) {
		printf("Normal\n");
		path = "/Recording/NowShowing";

	} else {
		printf("Complete\n");
		path = "/Recording/Complete";
	}

	fsid = mfs_resolve("/SwSystem/ACTIVE");

	memset(txtbuf, '\0', 99);
	sprintf(txtbuf ,"%s", query_string(fsid, "Name"));

	if(txtbuf[0] == '3') {
		version++;
	}

	if(version) {
		fsid2 = mfs_resolve("/State/LocationConfig");
		zone = query_int(fsid2, "TimeZoneOld");
		daylight = query_int(fsid2, "DaylightSavingsPolicy");
	} else {
		fsid2 = mfs_resolve("/Setup");
		zone = query_int(fsid2, "TimeZone");
		daylight = query_int(fsid2, "DaylightSavingsPolicy");
	}
	zone = get_zone(zone, daylight);

	fsid_index_list = parse_nowshowing(path, zone);
	print_fsid_index_list(fsid_index_list);
	return(fsid_index_list);
}

static fsid_index_t * parse_nowshowing(char * path, int zone) {

	struct mfs_dirent	*dir;
	u32			count;
	int			index;
	u32			i, j;
	time_t			recTime;
	struct tm		recTimetm;
	struct tm		airTimetm;
	int			ampm = 0;
	char			*sliceIds[ 32 ];
	int			numSlices;
	char			fsid[ 32 ];
	int			ifsid;
	int			fsid_nr;
	int			duration, duration_m, duration_h;
	int			state;
	char 			txtbuf[600];
	char			*pt1;
	int 			size;
	int			ok;

	mfs_attribute_t  actors;
	fsid_index_t * fsid_index_list=NULL;
	fsid_index_t * fsid_index;
	struct stat fileinfo;

	dir = mfs_dir(mfs_resolve(path), &count);

	for ( i = 0 ; i < count ; i++ ) {

		//printf("I %i - Count %i\n", i, count);
		state = query_int(dir[i].fsid, "State");
		fsid_nr = dir[i].fsid;

		/* The recoding is done so parse the info */
		if(state) {

			fsid_index=new_fsid_index();

			recTime = (query_int(dir[i].fsid, "StartDate") * 86400);
			recTime += query_int(dir[i].fsid, "StartTime") + 60;
			recTime += zone;
			localtime_r( &recTime, &recTimetm );
			recTimetm.tm_year += 1900;
			ampm = 0;

			if ( recTimetm.tm_hour > 12 ) {
				ampm = 1;
				recTimetm.tm_hour -= 12;
			}


			recTime = (query_int(dir[i].fsid, "Showing/Program/OriginalAirDate") * 86400);
			localtime_r( &recTime, &airTimetm );
			airTimetm.tm_year += 1900;

			/* The REC FSID */
			add_fsid(query_int(dir[i].fsid, "Part"), fsid_index);
			sprintf( fsid, "%d", query_int( dir[ i ].fsid, "Part" ) );

			/* The Tystream FILE NAME */
			fsidtoparts( fsid, sliceIds, &numSlices );
			pt1 = txtbuf;
			memset(txtbuf, '\0', 599);

			for( index = 0 ; index < numSlices ; index++ ) {

				size = strlen(sliceIds[index]);
				strncpy(pt1,sliceIds[index],size);
				pt1 = pt1 + size;

				if (index != ( numSlices - 1 )) {
					pt1[0]=',';
					pt1++;
				}
				free( sliceIds[ index ] );
			}
			add_tystream(txtbuf, fsid_index);


			/* The State */
			add_state(state,fsid_index);

			/* YEAR */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%d", recTimetm.tm_year);
			if(strcmp("(null)", txtbuf)) {
				add_year(txtbuf, fsid_index);
			}

			/* Airdate */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf , "%d/%d/%d", airTimetm.tm_mon + 1, airTimetm.tm_mday, airTimetm.tm_year );
			if(strcmp("(null)", txtbuf)) {
				add_air(txtbuf, fsid_index);
			}

			/* Day */
			add_day(dayofweek[recTimetm.tm_wday],fsid_index);

			/* Date */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf , "%d/%d", recTimetm.tm_mon + 1, recTimetm.tm_mday );
			if(strcmp("(null)", txtbuf)) {
				add_date(txtbuf, fsid_index);
			}

			/* Time */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf , "%d:%2.2d %s", recTimetm.tm_hour, recTimetm.tm_min, amorpm[ ampm ] );
			if(strcmp("(null)", txtbuf)) {
				add_rectime(txtbuf, fsid_index);
			}

			/* Duration */
			duration = query_int(dir[i].fsid, "StopDate") -
				query_int(dir[i].fsid, "StartDate");
			duration *= 86400;

			duration += ( query_int(dir[i].fsid, "StopTime") -
				query_int(dir[i].fsid, "StartTime"));
			memset(txtbuf, '\0', 599);
			duration_h = duration / 3600;
			duration_m = (duration % 3600)/ 60;
			sprintf(txtbuf , "%d:%02d", duration_h, duration_m );
			if(strcmp("(null)", txtbuf)) {
				add_duration(txtbuf, fsid_index);
			}

			/* Title */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf , "%s", query_string(dir[i].fsid, "Showing/Program/Title"));
			if(strcmp("(null)", txtbuf)) {
				add_title(txtbuf, fsid_index);
			}

			/* Episode */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/EpisodeTitle"));
			if(strcmp("(null)", txtbuf)) {
				add_episode(txtbuf, fsid_index);
			}

			/* Episode Number */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%d", query_int(dir[i].fsid, "Showing/Program/EpisodeNum"));
			if(strcmp("-1", txtbuf)) {
				add_episodenr(txtbuf, fsid_index);
			}

			/* Description */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/Description"));
			if(strcmp("(null)", txtbuf)) {
				add_description(txtbuf, fsid_index);
			}

			/* Actors */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/Actor"));
			if(strcmp("(null)", txtbuf)) {
				add_actors(txtbuf, fsid_index);
			}

			/* Guest stars */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/GuestStar"));
			if(strcmp("(null)", txtbuf)) {
				add_gstar(txtbuf, fsid_index);
			}

			/* Host */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/Host"));
			if(strcmp("(null)", txtbuf)) {
				add_host(txtbuf, fsid_index);
			}

			/* Director */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/Director"));
			if(strcmp("(null)", txtbuf)) {
				add_director(txtbuf, fsid_index);
			}

			/* Exec Prod */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/ExecProducer"));
			if(strcmp("(null)", txtbuf)) {
				add_eprod(txtbuf, fsid_index);
			}

			/* Producer */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/Producer"));
			if(strcmp("(null)", txtbuf)) {
				add_prod(txtbuf, fsid_index);
			}
			
			/* Lets test the attribute stuff */
			sprintf( fsid, "%d", query_int( dir[ i ].fsid, "Showing/Program" ) );
			ifsid = mfs_resolve( fsid );
			printf(" %d -  %s - %d\n", ifsid, fsid, dir[ i ].fsid);
			//actors = (mfs_attribute_t *)malloc(sizeof(mfs_attribute_t));
			printf("Doing mfs get\n");
			ok = mfs_get_attribute(ifsid, "Program", -1, "Actor", &actors);
			printf("Done mfs get %i\n", ok);


			/* Writer */
			memset(txtbuf, '\0', 599);
			sprintf(txtbuf, "%s", query_string(dir[i].fsid, "Showing/Program/Writer"));
			if(strcmp("(null)", txtbuf)) {
				add_writer(txtbuf, fsid_index);
			}

			/* Index present or not */
			memset(txtbuf, '\0', 599);
			snprintf(txtbuf, 598,"/var/index/index/%i", fsid_index->fsid);
			if(!stat(txtbuf, &fileinfo)) {
				/* okay something is there - lets check if reg file */
				if(S_ISREG(fileinfo.st_mode)) {
					add_index_avalible(1, fsid_index);
				} else {
					printf("Warning indexfile not a regluar file - /var/index/index/%i\n", fsid_index->fsid);
				}
			} else {
				//perror("");
				//printf("/var/index/index/%i not avalible\n", fsid_index->fsid);
			}

			fsid_index_list = add_fsid_index(fsid_index_list, fsid_index);

			if(ok) {
				for(j=0; j < actors.n - 1; j++) {
					printf("%s:\n",actors.u.string[j]);
				}
			}



		}
		//printf("I %i - Count %i\n", i, count);
	}

	if(dir) {
		mfs_dir_free(dir);
	}
	return(fsid_index_list);
}

int main() {

	parse_nowshowing_tivo();
	return(0);
}
