/*
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdio.h>

#include "tycommon.h"
#include "tytypes.h"

/*
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>

#ifdef WIN32
#include <io.h>

#define read _read
#define write _write
#define open _open
#define close _close
#define lseek64(h,off,orig) _lseeki64(h,off,orig)
#define off64_t	__int64

#define OPEN_CREATE_FLAGS (_O_WRONLY | _O_TRUNC | _O_CREAT | _O_BINARY)
#define OPEN_READONLY_FLAGS (O_RDONLY | _O_BINARY)
#define OPEN_WRITE_PERMISSION (_S_IREAD | _S_IWRITE )

#else  / * Unix * /
#include <inttypes.h>
#include <unistd.h>

#ifdef __APPLE__

#include "portability.h"


#endif

#define OPEN_CREATE_FLAGS ( O_WRONLY | O_TRUNC | O_CREAT)
#define OPEN_READONLY_FLAGS O_RDONLY
#define OPEN_WRITE_PERMISSION ( S_IREAD | S_IWRITE )

#endif
*/

#define CHUNK_SIZE 131072

char buffer[CHUNK_SIZE + 1];

void Usage()
{
	printf( "This program chops tystreams into smaller files to allow them to be\n");
	printf( "sent to developers for analysis.\n");
	printf( "Although chunk numbers are passed as parameters, the chopping is\n");
	printf( "not precise enough to guarantee the chop will happen at that chunk\n");
	printf( "In particular any header information in the .ty file will throw things off.\n");
	printf( "Therefore chop well before and after the suspect part of the stream\n");
	printf( "\n");
	printf( "Usage:\n");
	printf( "tychopper infile start end outfile\n");
	printf( "infile - name of the tystream to extract from\n");
	printf( "start - Number of the first chunk to extract\n");
	printf( "end - Number of the last chunk to extract\n");
	printf( "outfile - name of the file to extract to\n");
}

int main(int argc, char *argv[])
{
	unsigned long ulStart = 0;
	unsigned long ulEnd = 0xffffffff;
	unsigned long ulChunks;
	unsigned long ulCount;

	char *inFileName;
	char *outFileName;
	off64_t seekPos;
	int inFile;
	int outFile;

	/* Command line parameters */
	if( argc != 5 )
	{
		Usage();
		return -1;
	}

	inFileName = argv[1];
	ulStart = atol( argv[2] );
	ulEnd = atol( argv[3] );
	outFileName = argv[4];

	/* Command line parameters */
	if( ulEnd <= ulStart )
	{
		fprintf(stderr, "End chunk number must be larger that the start chunk number\n" );
		return -1;
	}
	ulChunks = (ulEnd - ulStart) + 1;

	/* Open the file and seek to the start of chopping point */
	inFile = open(inFileName, OPEN_READONLY_FLAGS );
	if (inFile < 0)
	{
		fprintf(stderr, "Cannot open %s\n", inFileName );
		return -1;
	}

	if( ulStart > 0 )
	{
		seekPos = ulStart;
		seekPos *= CHUNK_SIZE;
		if( lseek64(inFile, seekPos, SEEK_SET) == -1L )
		{
			fprintf(stderr, "Failed to seek to start chunk\n");
			close( inFile );
			return -1;
		}
	}

	/* Open the output file */
	outFile = open(outFileName, OPEN_CREATE_FLAGS, OPEN_WRITE_PERMISSION );
	if (outFile < 0)
	{
		fprintf(stderr, "Cannot open %s\n", outFileName );
		close( inFile );
		return -1;
	}

	printf( "Copying %ld chunks from %s to %s\n", ulChunks, inFileName, outFileName );

	/* Start copying chunks */
	for( ulCount = 0; ulCount < ulChunks; ++ulCount )
	{
		unsigned int nBytesRead;
		nBytesRead = read( inFile, buffer, CHUNK_SIZE );
		if( nBytesRead > 0 )
		{
			write( outFile, buffer, nBytesRead );
		}
		else
		{
			fprintf(stderr, "Premature end of input file detected\n");
			break; // End of file
		}
		if( ulCount%100 == 0 )
			printf("%lu Chunks copied\r", ulCount );
	}

	close( inFile );
	close( outFile );
	return(0);
}
