#include "nowshowingimpl.h"

#include <qapplication.h>
#include <qmessagebox.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qregexp.h>

/* 
 *  Constructs a NowShowing which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
NowShowing::NowShowing( const char *tivoHost, CDemuxInterface *pDemux, QWidget* parent,  const char* name, bool modal, WFlags fl )
    : NowShowingBase( parent, name, modal, fl )
{
	m_pDemux = pDemux;
    m_pList = NULL;
	connect( lstNowShowing, SIGNAL(currentChanged( QListViewItem * )), this, SLOT(ListCurrentChanged ( QListViewItem *)) );
	connect( lstNowShowing, SIGNAL(doubleClicked( QListViewItem *)), this, SLOT(ListDoubleClick ( QListViewItem *)) );

	connect( btnRefresh, SIGNAL( clicked() ), this, SLOT( onRefresh() ) );
	connect( chkCopyToPC, SIGNAL( clicked() ), this, SLOT( onCopyToPC() ) );

    buttonOk->setEnabled( false );
	lstNowShowing->setSorting( 1 );

	m_CopyToPC = false;
	m_fsid = 0;

	if( tivoHost && tivoHost[0] ) {
		hostname->setText( tivoHost );
		onRefresh();
	} else {
		hostname->setText( "tivo" );
	}
}

/*  
 *  Destroys the object and frees any allocated resources
 */
NowShowing::~NowShowing()
{
	/* Free the list */
	if( m_pList ) {
		CTivoFile *pNext = m_pList->pNext;
		CTivoFile *pList = m_pList;

		while( pList ) {
			delete pList;
			pList = pNext;
			if( pList ) {
				pNext = pList->pNext;
			}
		}
	}
}

void NowShowing::ListCurrentChanged ( QListViewItem *pItem )
{
	if( lstNowShowing->selectedItem() ) {
		buttonOk->setEnabled( true );

		// Find the item in the list to get the description
		CTivoFile *pList = m_pList;
		while( pList ) {
			if( pItem->text(5).toInt() == pList->fsid ) {
				ShowDescription->setText( QString( pList->description ) );
				m_StreamName = pList->title;
				if( pList->episode && pList->episode[0] && strcmp(pList->episode,"Not an Episode") ) {
					m_StreamName += "-";
					m_StreamName += pList->episode;
				}
				// Replace illegal filename characters
				m_StreamName.replace( QRegExp("[.:$*]"), "_" );
				break;
			}
			pList = pList->pNext;
		}
	} else {
		m_StreamName = "";
		ShowDescription->setText( QString( "Select a show to load from the list above" ) );
		buttonOk->setEnabled( false );
	}

	if( pItem->text(0) == "Yes" ) {
		chkCopyToPC->setChecked( false );
	} else {
		chkCopyToPC->setChecked( true );
	}
	onCopyToPC();

	m_fsid = pItem->text(5).toInt();
}

void NowShowing::ListDoubleClick ( QListViewItem * )
{
	accept();
}

void NowShowing::onCopyToPC()
{
	m_CopyToPC = chkCopyToPC->isChecked();
}

void NowShowing::onRefresh()
{
	char host[MAXPATHLEN];
	
	strcpy( host, hostname->text() );
	if( host[0] == 0 ) {
		return;
	}

    QApplication::setOverrideCursor( waitCursor ); // this might take time
	CTivoFile *pList = m_pDemux->GetTivoFileList( host );
    QApplication::restoreOverrideCursor();	// restore original cursor

	if( !pList ) {
		QMessageBox::warning( this, "TY Editor", "Failed to obtain the Now Showing list from TIVO" );
		return;
	}

	/* Free the old list */
	if( m_pList ) {
		CTivoFile *pNext = m_pList->pNext;
		CTivoFile *pList = m_pList;

		while( pList ) {
			delete pList;
			pList = pNext;
			if( pList ) {
				pNext = pList->pNext;
			}
		}
	}

	m_pList = pList;

	UpdateScreenList();
}

void NowShowing::UpdateScreenList()
{
	lstNowShowing->clear();

	CTivoFile *pList = m_pList;
	CTivoFile *pNext = m_pList->pNext;
	while( pList ) {
		// Add everything to the now showing list
		QString fsid;
		fsid.setNum( pList->fsid );
		new QListViewItem( lstNowShowing,
							pList->indexed ? QString("Yes") : QString("No"),
							QString(pList->title),
							QString(pList->episode),
							QString(pList->date) + " " + QString(pList->rectime),
							QString(pList->duration),
							fsid
							);
		pList = pNext;
		if( pList ) {
			pNext = pList->pNext;
		}
	}
}
