#include "optionsimpl.h"

#include <qcombobox.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qtoolbutton.h>

#include <stdio.h>
#include <tycommon.h>

/* 
 *  Constructs a options which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
options::options( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : optionsBase( parent, name, modal, fl )
{
	cboVideoTo->insertItem( "DVD" );
	cboVideoTo->insertItem( "SVCD" );
	cboVideoTo->insertItem( "Generic MPEG2" );
	cboVideoTo->insertItem( "Elemental Stream" );
	cboAudioTo->insertItem( "48000 (DVD)" );
	cboAudioTo->insertItem( "AC3 (DVD)" );
	cboAudioTo->insertItem( "44100" );
	cboAudioTo->insertItem( "32000" );
	cboAudioTo->insertItem( "No Transcoding" );

	setCaption( "Qt TyEditor options" );

    connect( btnOutputBrowse, SIGNAL(clicked()), this, SLOT(OutputBrowse()) );
    connect( btnMarkerBrowse, SIGNAL(clicked()), this, SLOT(MarkerBrowse()) );
    connect( btnTyBrowse, SIGNAL(clicked()), this, SLOT(TyBrowse()) );

	/* Set up default values */
	m_AudioTo = "48000 (DVD)";
	m_VideoTo = "DVD";
	m_TivoHostName = "";
	m_DemuxParams = "";
	m_TranscodeParams = "";
	m_MplexParams = "-V";
	m_tyFileDir = "";
	m_OutputFileDir = "";
	m_MarkerFileDir = "";

	LoadSettings();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
options::~options()
{
    // no need to delete child widgets, Qt does it all for us
}


void options::updateControls()
{
	int n;

	// Update the controls with the current values
	for( n = 0; n < cboVideoTo->count(); ++n ) {
		if( cboVideoTo->text(n) == m_VideoTo ) {
			cboVideoTo->setCurrentItem(n);
			break;
		}
	}

	for( n = 0; n < cboAudioTo->count(); ++n ) {
		if( cboAudioTo->text(n) == m_AudioTo ) {
			cboAudioTo->setCurrentItem(n);
			break;
		}
	}

	editDemuxParams->setText(m_DemuxParams);
	editTranscodeParams->setText(m_TranscodeParams);
	editMplexParams->setText(m_MplexParams);
	editTyFiles->setText(m_tyFileDir);
	editOutputFiles->setText(m_OutputFileDir);
	editMarkerFiles->setText(m_MarkerFileDir);
	editTivoHostName->setText(m_TivoHostName);

}

void options::readControls()
{
	m_VideoTo = cboVideoTo->currentText();
	m_AudioTo = cboAudioTo->currentText();

	m_DemuxParams = editDemuxParams->text();
	m_TranscodeParams = editTranscodeParams->text();
	m_MplexParams = editMplexParams->text();
	m_tyFileDir = editTyFiles->text();
	m_OutputFileDir = editOutputFiles->text();
	m_MarkerFileDir = editMarkerFiles->text();
	m_TivoHostName = editTivoHostName->text();
}

void options::SaveSettings()
{
	FILE *fp;

	readControls();

	fp = fopen( "tyeditor.ini", WRITE_MODE );
	fputs( "# TyEditor configuration settings\n\n", fp);

	fprintf( fp, "VideoTo=%s\n", m_VideoTo.latin1() );
	fprintf( fp, "AudioTo=%s\n", m_AudioTo.latin1() );
	fprintf( fp, "DemuxParams=%s\n", m_DemuxParams.latin1() );
	fprintf( fp, "TranscodeParams=%s\n", m_TranscodeParams.latin1() );
	fprintf( fp, "MplexParams=%s\n", m_MplexParams.latin1() );
	fprintf( fp, "tyFileDir=%s\n", m_tyFileDir.latin1() );
	fprintf( fp, "OutputFileDir=%s\n", m_OutputFileDir.latin1() );
	fprintf( fp, "MarkerFileDir=%s\n", m_MarkerFileDir.latin1() );
	fprintf( fp, "TivoHostName=%s\n", m_TivoHostName.latin1() );

	fclose( fp );
}

void options::LoadSettings()
{
	FILE *fp;
	char buffer[1024];

	fp = fopen( "tyeditor.ini", READ_MODE );
	if( !fp ) {
		// No ini file so we will use the defaults
		updateControls();
		return;
	}

	while( fgets( buffer, 1023, fp ) ) {
		char *tag, *value;
		tag = buffer;

		// Skip leading whitespace
		while( isspace( *tag ) ) {
			++tag;
		}

		// ignore comment lines
		if( *buffer == '#' ) {
			continue;
		}

		value = tag;

		// Find tag name
		while( *value && *value != '=' ) {
			++value;
		}
		if( *value == 0 ) {
			// No equal sign
			continue;
		}

		// Terminate tag string and skip to value
		*value = 0;
		++value;

		if( strcmp( tag, "VideoTo" ) == 0 ) {
			m_VideoTo = value;
			m_VideoTo = m_VideoTo.stripWhiteSpace();
		} else if( strcmp( tag, "AudioTo" ) == 0 ) {
			m_AudioTo = value;
			m_AudioTo = m_AudioTo.stripWhiteSpace();
		} else if( strcmp( tag, "DemuxParams" ) == 0 ) {
			m_DemuxParams = value;
			m_DemuxParams = m_DemuxParams.stripWhiteSpace();
		} else if( strcmp( tag, "TranscodeParams" ) == 0 ) {
			m_TranscodeParams = value;
			m_TranscodeParams = m_TranscodeParams.stripWhiteSpace();
		} else if( strcmp( tag, "MplexParams" ) == 0 ) {
			m_MplexParams = value;
			m_MplexParams = m_MplexParams.stripWhiteSpace();
		} else if( strcmp( tag, "tyFileDir" ) == 0 ) {
			m_tyFileDir = value;
			m_tyFileDir = m_tyFileDir.stripWhiteSpace();
		} else if( strcmp( tag, "OutputFileDir" ) == 0 ) {
			m_OutputFileDir = value;
			m_OutputFileDir = m_OutputFileDir.stripWhiteSpace();
		} else if( strcmp( tag, "MarkerFileDir" ) == 0 ) {
			m_MarkerFileDir = value;
			m_MarkerFileDir = m_MarkerFileDir.stripWhiteSpace();
		} else if( strcmp( tag, "TivoHostName" ) == 0 ) {
			m_TivoHostName = value;
			m_TivoHostName = m_TivoHostName.stripWhiteSpace();
		}
	}

	fclose( fp );

	updateControls();

}

void options::accept ()
{
	SaveSettings();

	QDialog::accept();
}

void options::directoryBrowse( QLineEdit* editControl, const char *caption )
{
	QString directory = editControl->text();

	directory = QFileDialog::getExistingDirectory( directory, this, "", caption, true );
	if ( !(directory.isEmpty()) ) {
		editControl->setText( directory );
	}

}

void options::OutputBrowse()
{
	directoryBrowse( editOutputFiles, "Select Output Directory" );
}

void options::MarkerBrowse()
{
	directoryBrowse( editMarkerFiles, "Select Marker Save Directory" );
}

void options::TyBrowse()
{
	directoryBrowse( editTyFiles, "Select ty File Directory" );
}
