/*
 * Copyright (C) 2002, 2003  John Barrett (MrBassMan) <johnbarretthcc@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: processor.h,v 1.6 2003/03/30 01:41:14 mrbassman Exp $ */

#ifndef __PROCESSOR_H__
#define __PROCESSOR_H__

#include "tydemuxclass.h"
#include "../typrocess/typrocess.h"

/* Background processor task */
typedef struct {
	typrocess_params_t typrocess_params;
	int progress;
	int result;
	int complete;
} processor_thread_params_t;


class CDemuxProcessTask
{
public:
	typedef enum { AF_AC3, AF_MPEG } eAudioFormat;
	typedef enum { VF_DVD, VF_SVCD, VF_VCD, VF_MPEG1, VF_MPEG2, VF_ELEMENTAL } eVideoFormat;

	CDemuxProcessTask( const char *tyFile, const char *outfile, FILE *logFile );
	~CDemuxProcessTask();

	void AddCut( const char *CutString );
	void SetAudioFormat( eAudioFormat format, int rate = 0, int bitrate = 0 );
	void SetVideoFormat( eVideoFormat format );
	void SetAudioFile( const char *fileName );
	void SetRemoteFile( remote_holder_t *pRemote, int fsid );
	void SetModuleOptions( const char *demux, const char *transcode, const char *mplex );

	const char *GetVideoFile();
	bool StartJob();
	bool IsCompleted();
	bool CancelJob();
	int  GetResult();
	int  GetProgress();  // 0 - 100 (percent)
	const char *GetProgressText();  // Last text progress message

protected:
	processor_thread_params_t m_Params;

private:
	thread_handle_t m_th;
};

#endif //__PROCESSOR_H__
