/*
 * Copyright (C) 2002, 2003  John Barrett (MrBassMan) <johnbarretthcc@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: tyeditorwindow.cpp,v 1.34 2003/03/30 02:18:51 mrbassman Exp $ */

#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlistview.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <assert.h>
#include <ctype.h>

#include "tymainwindow.h"

tyeditorWindow::tyeditorWindow( tymainWindow* parent )
	: frmMainWindow( parent ), m_pCurrentPlayButtonPressed(0), m_mainWindow(0),
	 m_bFrameMode(true), m_PlayRate(10), m_pDemux(NULL), m_BadMarkersFound(false)
{
    int n;

    m_mainWindow = parent;

    connect( btnNavStart, SIGNAL( clicked() ), SLOT( NavStart() ) );
    connect( btnNavPlayBack, SIGNAL( clicked() ), SLOT( NavPlayBack() ) );
    connect( btnNavPlay, SIGNAL( clicked() ), SLOT( NavPlay() ) );
    connect( btnNavFBack, SIGNAL( clicked() ), SLOT( NavFBack() ) );
    connect( btnNavFFwd, SIGNAL( clicked() ), SLOT( NavFFwd() ) );
    connect( btnNavEnd, SIGNAL( clicked() ), SLOT( NavEnd() ) );
    connect( btnNavStop, SIGNAL( clicked() ), SLOT( NavStop() ) );

	Slider1->setRange( 0, 100 );
    connect( Slider1, SIGNAL( sliderMoved(int) ), SLOT( NavSlider1(int) ) );
	Slider2->setRange( 1, 10 );
    connect( Slider2, SIGNAL( valueChanged(int) ), SLOT( NavSlider2(int) ) );

    connect( &m_timer, SIGNAL(timeout()), this, SLOT(PlayTimer()) );

    connect( btnLoadTYFile, SIGNAL(clicked()), this, SLOT(LoadTY()) );
    connect( btnLoadTivoFile, SIGNAL(clicked()), this, SLOT(LoadTivo()) );

	connect( btnCutStart, SIGNAL(clicked()), this, SLOT(CutStart()) );
	connect( btnCutStop, SIGNAL(clicked()), this, SLOT(CutStop()) );
	connect( btnChapter, SIGNAL(clicked()), this, SLOT(ChapterMark()) );
	connect( btnLocate, SIGNAL(clicked()), this, SLOT(Locate()) );
	connect( btnRemove, SIGNAL(clicked()), this, SLOT(RemoveSelected()) );
	connect( btnRemoveAll, SIGNAL(clicked()), this, SLOT(RemoveAll()) );
	connect( btnProcess, SIGNAL(clicked()), this, SLOT(Process()) );
	connect( chkFrameMode, SIGNAL(clicked()), this, SLOT(FrameMode()) );
	chkFrameMode->setChecked( true );

	lstMarkers->clear();
	lstMarkers->setColumnText( 0, "Type" );
	lstMarkers->addColumn( "Frame" );
	lstMarkers->addColumn( "Time" );
	lstMarkers->setSorting( 1 );
	lstMarkers->setAllColumnsShowFocus( true );

	connect( lstMarkers, SIGNAL(currentChanged( QListViewItem * )), this, SLOT(MarkerCurrentChanged ( QListViewItem *)) );
	connect( lstMarkers, SIGNAL(clicked( QListViewItem * )), this, SLOT(MarkerCurrentChanged ( QListViewItem *)) );
	connect( lstMarkers, SIGNAL(doubleClicked( QListViewItem *)), this, SLOT(MarkerDoubleClick ( QListViewItem *)) );

	cboVideoOut->insertItem( "DVD" );
	cboVideoOut->insertItem( "SVCD" );
	/* No MPEG1 - we will get into trouble if the user choose this
	if we go into video transcoding we can but we aren't there 
	yet */
	//cboVideoOut->insertItem( "VCD" );
	//cboVideoOut->insertItem( "Generic MPEG1" );
	cboVideoOut->insertItem( "Generic MPEG2" );
	cboVideoOut->insertItem( "Elemental Stream" );
	cboTranscode->insertItem( "48000 (DVD)" );
	cboTranscode->insertItem( "AC3 (DVD)" );
	cboTranscode->insertItem( "44100" );
	cboTranscode->insertItem( "32000" );
	cboTranscode->insertItem( "No Transcoding" );

	// Set default values
	for( n = 0; n < cboVideoOut->count(); ++n ) {
		if( cboVideoOut->text(n) == m_mainWindow->GetOptions()->m_VideoTo ) {
			cboVideoOut->setCurrentItem(n);
			break;
		}
	}

	for( n = 0; n < cboTranscode->count(); ++n ) {
		if( cboTranscode->text(n) == m_mainWindow->GetOptions()->m_AudioTo ) {
			cboTranscode->setCurrentItem(n);
			break;
		}
	}

	EnableControls();
}

tyeditorWindow::~tyeditorWindow()
{
}

bool tyeditorWindow::HasCutsToSave()
{
	return ( lstMarkers->childCount() > 0 );
}

void tyeditorWindow::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
	case Key_Right:
		if( e->state() & ControlButton ) {
			NavEnd();
		} else if( e->state() & ShiftButton ) {
			NavFFwd();
		} else {
			NavPlay();
		}
		break;
	case Key_Left:
		if( e->state() & ControlButton ) {
			NavStart();
		} else if( e->state() & ShiftButton ) {
			NavFBack();
		} else {
			NavPlayBack();
		}
		break;
	case Key_Down:
		if( Slider2->value() > 1 ) {
			if( e->state() & ControlButton ) {
				Slider2->setValue(1);
			} else {
				Slider2->setValue(Slider2->value() - 1);
			}
		}
		break;
	case Key_Up:
		if( Slider2->value() < 10 ) {
			if( e->state() & ControlButton ) {
				Slider2->setValue(10);
			} else {
				Slider2->setValue(Slider2->value() + 1);
			}
		}
		break;
	default:
		switch ( e->ascii() ) {
		case ' ':
			if( e->state() & ControlButton ) {
				CutStop();
			} else 	if( e->state() & ShiftButton ) {
				CutStart();
			} else {
				chkFrameMode->setChecked( !(chkFrameMode->isChecked()) );
				FrameMode();
			}
			break;
		}
	}
}

void tyeditorWindow::SetCounterMode( int mode )
{
	m_CounterMode = mode;
	if( mode ) {
		char timeString[25];
		char frameString[25];

		txtStreamTime->show();

		if(  m_pDemux ) {
			FrameAsString( m_pDemux->GetStreamCurrentFrame(), frameString );
			TimeAsString( m_pDemux->GetStreamCurrentTime(), timeString);
		} else {
			frameString[0] = 0;
			timeString[0] = 0;
		}

		if( m_CounterMode == 1) {
			txtStreamTime->setText( frameString );
		} else if( m_CounterMode == 2) {
			txtStreamTime->setText( timeString );
		}
	}
	else
		txtStreamTime->hide();
}

void tyeditorWindow::LoadTY()
{
	assert( m_mainWindow  );
	m_mainWindow->fileOpenTy();
	if( m_pDemux ) {
		NavStart();
	}
}

void tyeditorWindow::LoadTivo()
{
	assert( m_mainWindow  );
	m_mainWindow->fileOpenTivo();
	if( m_pDemux ) {
		NavStart();
	}
}

bool tyeditorWindow::SetupProcessor( CDemuxProcessTask *processor )
{
	char timeStart[MAX_CUTSTRING_SIZE];
	char timeStop[MAX_CUTSTRING_SIZE];
	char cutStart[MAX_CUTSTRING_SIZE];
	char cutStop[MAX_CUTSTRING_SIZE];
	char cutString[MAX_CUTSTRING_SIZE];
	MarkerListItem *pCutStart = NULL;

	/* Create a list of the current markers */
	lstMarkers->setSorting(1);

	// See if a system cut is needed to cut off reset GOPs
	if( m_pDemux->IsSystemStartCutNeeded() ) {
		m_pDemux->GetSystemStartCutStart().GetCutAsString( cutStart );
		m_pDemux->GetSystemStartCutStart().GetTimeAsString( timeStart );
		m_pDemux->GetSystemStartCutStop().GetCutAsString( cutStop );
		m_pDemux->GetSystemStartCutStop().GetTimeAsString( timeStop );
		sprintf( cutString, "%s-%s %s\t%s\tsystem\n", timeStart, timeStop, cutStart, cutStop );
		processor->AddCut( cutString );
	}

	MarkerListItem *pItem = (MarkerListItem *)(lstMarkers->firstChild());

	while( pItem ) {

		switch( pItem->m_Cut.m_Type ) {
		case CDemuxCutPoint::cutstart:
			pCutStart = pItem;
			break;
		case CDemuxCutPoint::cutstop:
			assert( pCutStart ); // Start/stops out of order should not happen !!!
			pCutStart->m_Cut.GetCutAsString( cutStart );
			pCutStart->m_Cut.GetTimeAsString( timeStart );
			pItem->m_Cut.GetCutAsString( cutStop );
			pItem->m_Cut.GetTimeAsString( timeStop );
			sprintf( cutString, "%s-%s %s\t%s\tremove\n", timeStart, timeStop, cutStart, cutStop );
			processor->AddCut( cutString );
			pCutStart = NULL;
			break;
		case CDemuxCutPoint::chapter:
			pItem->m_Cut.GetCutAsString( cutStart );
			pItem->m_Cut.GetTimeAsString( timeStart );
			sprintf( cutString, "%s-%s %s\t%s\tchapter\n", timeStart, timeStart, cutStart, cutStart );
			processor->AddCut( cutString );
			break;
		default:
			break; // Ignore unknown marker types
		}

		pItem = (MarkerListItem *)(pItem->itemBelow());
	}

	if( pCutStart ) {
		// Found a start without a stop so cut till the end
		NavEnd();

		pItem = new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::cutstop ) );
		pCutStart->m_Cut.GetCutAsString( timeStart );
		pItem->m_Cut.GetCutAsString( cutStop );
		pItem->m_Cut.GetTimeAsString( timeStop );
		sprintf( cutString, "%s-%s %s\t%s\tremove\n", timeStart, timeStop, cutStart, cutStop );
		processor->AddCut( cutString );
	}

	if( m_pDemux->IsSystemEndCutNeeded() ) {
		m_pDemux->GetSystemEndCutStart().GetCutAsString( cutStart );
		m_pDemux->GetSystemEndCutStart().GetTimeAsString( timeStart );
		m_pDemux->GetSystemEndCutStart().GetCutAsString( cutStop );
		m_pDemux->GetSystemEndCutStart().GetTimeAsString( timeStop );
		sprintf( cutString, "%s-%s %s\t%s\tsystem\n", timeStart, timeStop, cutStart, cutStop );
		processor->AddCut( cutString );
	}

	/* Set the output audio and video formats */
	QString audioformat = cboTranscode->currentText();

	if( audioformat == "AC3 (DVD)" ) {
		processor->SetAudioFormat( CDemuxProcessTask::AF_AC3, 48000 );
	}
	if( audioformat == "48000 (DVD)" ) {
		processor->SetAudioFormat( CDemuxProcessTask::AF_MPEG, 48000 );
	}
	if( audioformat == "44100" ) {
		processor->SetAudioFormat( CDemuxProcessTask::AF_MPEG, 44100 );
	}
	if( audioformat == "32000" ) {
		processor->SetAudioFormat( CDemuxProcessTask::AF_MPEG, 32000 );
	}

	QString videoformat = cboVideoOut->currentText();
	if( videoformat == "DVD" ) {
		processor->SetVideoFormat( CDemuxProcessTask::VF_DVD );
	}
	if( videoformat == "SVCD" ) {
		processor->SetVideoFormat( CDemuxProcessTask::VF_SVCD );
	}
	/* We can't do MPEG1 
	if( videoformat == "VCD" ) {
		processor->SetVideoFormat( CDemuxProcessTask::VF_VCD );
	}
	if( videoformat == "Generic MPEG1" ) {
		processor->SetVideoFormat( CDemuxProcessTask::VF_MPEG1 );
	}
	*/
	if( videoformat == "Generic MPEG2" ) {
		processor->SetVideoFormat( CDemuxProcessTask::VF_MPEG2 );
	}
	if( videoformat == "Elemental Stream" ) {
		processor->SetVideoFormat( CDemuxProcessTask::VF_ELEMENTAL );

		// Also need an audio out file
		/* get the output file name */
		QString filter;
		if( audioformat == "AC3 (DVD)" ) {
			filter = "Audio files (*.ac3)";
		} else {
			filter = "Audio files (*.m2a)";
		}
		QString audioFile = processor->GetVideoFile();
		// Replace the extension with .m2a or .ac3
		int index = audioFile.findRev('.');
		if( index != -1 ) {
			audioFile = audioFile.mid( 0, index );
		}
		if( audioformat == "AC3 (DVD)" ) {
			audioFile += ".ac3";
		} else {
			audioFile += ".m2a";
		}

		audioFile = QFileDialog::getSaveFileName( audioFile, filter );
		if ( audioFile.isEmpty() ) {
			return false;
		}

		processor->SetAudioFile( audioFile );
	}

	return( true );
}

const char *tyeditorWindow::GetVideoFileExtension()
{
	QString videoformat = cboVideoOut->currentText();
	if( videoformat == "Elemental Stream" ) {
		return( ".m2v" );
	} else {
		return( ".mpg" );
	}
}

bool tyeditorWindow::LoadCutFile( const char *filename )
{
	int nchar;
	char sznextline[ MAX_CUTSTRING_SIZE ];
	FILE *fh = fopen( filename, READ_MODE );

	if( !fh ) {
		return 0;
	}

	RemoveAll();

	// Now extract the cutpoint list

	while( fgets( sznextline, 1024, fh ) != NULL ) {
		nchar = 0;

		/* Skip leading white space  */
		while( isspace( sznextline[nchar] ) ) {
			++nchar;
		}

		/* only bother with lines that have a number as the first non-space character  */
		if( isdigit( sznextline[nchar] ) ) {
			char *szstart, *szstop, *sztype;
			int64_t tStart, tStop;

			szstart = &( sznextline[nchar] );
			szstop = strchr( &( sznextline[nchar] ), '-' );
			if( !szstop ) {
				continue;
			}
			*szstop = 0;
			++szstop;
			/* Skip leading white space  */
			while( isspace( *szstop ) ) {
				++szstop;
			}
			sztype = szstop;

			/* Skip to 4th @ character */
			int numAts = 0;
			while( *sztype && numAts < 4 ) {
				if( *sztype == '@' ) {
					++numAts;
				}
				++sztype;
			}

			/* Skip leading white space  */
			while( isspace( *sztype ) ) {
				++sztype;
			}

			if( !(*sztype) )
				continue;

			// Convert the start and stop times to ticks
			tStart = cutpoint_convert_time_to_ticks( szstart);
			tStop = cutpoint_convert_time_to_ticks( szstop);

			if( strncasecmp( sztype, "system", 6 ) == 0 ) {
				// Ignore these - they will be recreated when we save the cut file
			} else if( strncasecmp( sztype, "chapter", 7 ) == 0 ) {
				m_pDemux->SetStreamTime( tStart, true, true );
				new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::chapter ) );
			} else {
				m_pDemux->SetStreamTime( tStart, true, true );
				new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::cutstart ) );

				m_pDemux->SetStreamTime( tStop, true, true );
				new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::cutstop ) );
			}
		}
	}

	fclose( fh );

	m_pDemux->SetStreamCurrentFrame( 0 );
	CheckBadMarkers();

	return true;
}

bool tyeditorWindow::SaveCutFile( const char *filename )
{
	char timeStart[MAX_CUTSTRING_SIZE];
	char timeStop[MAX_CUTSTRING_SIZE];
	char cutStart[MAX_CUTSTRING_SIZE];
	char cutStop[MAX_CUTSTRING_SIZE];
	MarkerListItem *pCutStart = NULL;

	if(m_BadMarkersFound) {
		QMessageBox::warning( this, "TY Editor", "Bad markers found - correct these first." );
		return false;
	}

	FILE *fh = fopen( filename, WRITE_MODE );

	if( !fh ) {
		QMessageBox::warning( this, "TY Editor", "Cannot open the marker file for writing." );
		return false;
	}

	/* Write the file headers */
	fprintf( fh, "# TYEditor marker file\n" );

	// See if a system cut is needed to cut off reset GOPs
	if( m_pDemux->IsSystemStartCutNeeded() ) {
		m_pDemux->GetSystemStartCutStart().GetCutAsString( cutStart );
		m_pDemux->GetSystemStartCutStart().GetTimeAsString( timeStart );
		m_pDemux->GetSystemStartCutStop().GetCutAsString( cutStop );
		m_pDemux->GetSystemStartCutStop().GetTimeAsString( timeStop );
		fprintf( fh, "%s-%s %s\t%s\tsystem\n", timeStart, timeStop, cutStart, cutStop );
	}

	/* Create a list of the current markers */
	lstMarkers->setSorting(1);

	MarkerListItem *pItem = (MarkerListItem *)(lstMarkers->firstChild());

	while( pItem ) {

		switch( pItem->m_Cut.m_Type ) {
		case CDemuxCutPoint::cutstart:
			pCutStart = pItem;
			break;
		case CDemuxCutPoint::cutstop:
			assert( pCutStart ); // Start/stops out of order should not happen !!!
			pCutStart->m_Cut.GetCutAsString( cutStart );
			pCutStart->m_Cut.GetTimeAsString( timeStart );
			pItem->m_Cut.GetCutAsString( cutStop );
			pItem->m_Cut.GetTimeAsString( timeStop );
			fprintf( fh, "%s-%s %s\t%s\tremove\n", timeStart, timeStop, cutStart, cutStop );
			pCutStart = NULL;
			break;
		case CDemuxCutPoint::chapter:
			pItem->m_Cut.GetCutAsString( cutStart );
			pItem->m_Cut.GetTimeAsString( timeStart );
			fprintf( fh, "%s-%s %s\t%s\tchapter\n", timeStart, timeStart, cutStart, cutStart );
			break;
		default:
			break; // Ignore unknown marker types
		}

		pItem = (MarkerListItem *)(pItem->itemBelow());
	}

	if( pCutStart ) {
		// Found a start without a stop so cut till the end
		NavEnd();

		pItem = new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::cutstop ) );
		pCutStart->m_Cut.GetCutAsString( cutStart );
		pCutStart->m_Cut.GetTimeAsString( timeStart );
		pItem->m_Cut.GetCutAsString( cutStop );
		pItem->m_Cut.GetTimeAsString( timeStop );
		fprintf( fh, "%s-%s %s\t%s\tremove\n", timeStart, timeStop, cutStart, cutStop );
	}

	if( m_pDemux->IsSystemEndCutNeeded() ) {
		m_pDemux->GetSystemEndCutStart().GetCutAsString( cutStart );
		m_pDemux->GetSystemEndCutStart().GetTimeAsString( timeStart );
		m_pDemux->GetSystemEndCutStart().GetCutAsString( cutStop );
		m_pDemux->GetSystemEndCutStart().GetTimeAsString( timeStop );
		fprintf( fh, "%s-%s %s\t%s\tsystem\n", timeStart, timeStop, cutStart, cutStop );
	}

	fprintf( fh, "# End of list\n" );
	fclose( fh );

	return true;
}

/* Timer is called at 10Hz */
void tyeditorWindow::PlayTimer()
{
	static bool bNoReEntry = false;

	if( bNoReEntry )
		return;
	bNoReEntry = true;

	if( !m_pDemux || !m_pCurrentPlayButtonPressed ) {
		bNoReEntry = false;
		return;  /* No TY file loaded or no play button pressed */
	}

	// Find out how much time has actually elapsed since the last update
	int nElapsed = m_LastDisplayTime.restart();
	int64_t lastFrame = m_pDemux->GetStreamCurrentFrame();
	int64_t moveby = (TICKS_PER_MILLISECOND * nElapsed * m_PlaySpeed * m_PlayRate)/10;

	m_pDemux->SetStreamTimeRelative( moveby, false );

	if( lastFrame != m_pDemux->GetStreamCurrentFrame() ) {
		ReLoadImage();
	} else if( m_CounterMode == 2) {
		char timeString[25];
		TimeAsString( m_pDemux->GetStreamCurrentTime(), timeString);
		txtStreamTime->setText( timeString );
	}

	/* Check for end of stream conditions */
	if( m_PlaySpeed > 0 && m_pDemux->GetStreamFrames() == m_pDemux->GetStreamCurrentFrame() ) {
		playButtonPressed(NULL);
	}
	if( m_PlaySpeed < 0 && m_pDemux->GetStreamCurrentFrame() == 0 ) {
		playButtonPressed(NULL);
	}
	bNoReEntry = false;
}

void tyeditorWindow::EnableControls()
{
	bool bEnable = m_pDemux ? true:false;

	if( bEnable ) {
		pixMapImage->setBackgroundMode( NoBackground );
	    btnLoadTYFile->hide();
		btnLoadTivoFile->hide();
	} else {
		pixMapImage->setBackgroundMode( PaletteBackground );
	    btnLoadTYFile->show();
		btnLoadTivoFile->show();
	}

    btnNavStop->setEnabled( bEnable );
    btnNavStart->setEnabled( bEnable );
    btnNavPlayBack->setEnabled( bEnable );
    btnNavPlay->setEnabled( bEnable );
    btnNavFBack->setEnabled( bEnable );
    btnNavFFwd->setEnabled( bEnable );
    btnNavEnd->setEnabled( bEnable );
    Slider1->setEnabled( bEnable );
    Slider2->setEnabled( bEnable );
	chkFrameMode->setEnabled( bEnable );

    lstMarkers->setEnabled( bEnable );
    btnRemove->setEnabled( bEnable );
    btnRemoveAll->setEnabled( bEnable );

    btnCutStart->setEnabled( bEnable );
    btnCutStop->setEnabled( bEnable );
//    btnChapter->setEnabled( bEnable );
    btnChapter->hide();
    btnLocate->setEnabled( bEnable );

	cboVideoOut->setEnabled( bEnable );
	cboTranscode->setEnabled( bEnable );

	if( bEnable )
		btnProcess->setEnabled( !m_BadMarkersFound );
	else
		btnProcess->setEnabled( false );

	if( lstMarkers->selectedItem() ) {
		btnLocate->setEnabled( true );
		btnRemove->setEnabled( true );
	} else {
		btnLocate->setEnabled( false );
		btnRemove->setEnabled( false );
	}
	setFocus();
}

void tyeditorWindow::SetDemuxObject( CDemuxInterface *demux )
{
	int n;

	m_pDemux = demux;

	// Remove all markers first
	RemoveAll();

	if( m_pDemux ) {
		int64_t frames = m_pDemux->GetStreamFrames();
		int64_t totalmins = m_pDemux->GetStreamTime()/(90*60000);

		/* Set the image selection slider */
		Slider1->setRange( 0, (int) frames );
		Slider1->setValue( 0 );

		// Place tick marks every 1/5/15 minutes depending on stream duration
		if( totalmins ) {
			int64_t framesperminute = frames/totalmins;
			int ticks = 15;
			if( totalmins < 10 )
				ticks = 1;
			else if( totalmins < 30 )
				ticks = 5;

			Slider1->setTickInterval( (int)(framesperminute*ticks) );
		}

		m_PlayRate = 10;

		Slider2->setRange( 1, m_PlayRate );
		Slider2->setValue( m_PlayRate );

		pixMapImage->setPixmap( m_Image );

		/* Load the first frame */
		ReLoadImage();
	} else {
		/* Clear the current image */
		pixMapImage->setText( "No TY file Loaded" );
	}

	// Set default values
	for( n = 0; n < cboVideoOut->count(); ++n ) {
		if( cboVideoOut->text(n) == m_mainWindow->GetOptions()->m_VideoTo ) {
			cboVideoOut->setCurrentItem(n);
			break;
		}
	}

	for( n = 0; n < cboTranscode->count(); ++n ) {
		if( cboTranscode->text(n) == m_mainWindow->GetOptions()->m_AudioTo ) {
			cboTranscode->setCurrentItem(n);
			break;
		}
	}

	EnableControls();
}


QString MarkerListItem::text( int column ) const
{
	switch ( column ) {
	case 0: /* Marker type */
		switch( m_Cut.m_Type ) {
		case CDemuxCutPoint::cutstart:
			return "Cut Start";
			break;
		case CDemuxCutPoint::cutstop:
			return "Cut Stop";
			break;
		case CDemuxCutPoint::chapter:
			return "Chapter";
			break;
		default:
			return "Unknown";
			break;
		}
		break;
	case 1: /* Frame */
		char frameString[25];
		FrameAsString( m_Cut.frame, frameString);
		return ((const char *)frameString );
		break;
	case 2: /* Time */
		char timeString[25];
		TimeAsString( m_Cut.time, timeString);
		return ( (const char *)timeString );
		break;
	}

	return(NULL);
}

void MarkerListItem::paintCell( QPainter *p, const QColorGroup &cg,
				 int column, int width, int alignment )
{
    QColorGroup _cg( cg );
    QColor c = _cg.text();

    if ( m_BadEntry )
	_cg.setColor( QColorGroup::Text, Qt::red );

    QListViewItem::paintCell( p, _cg, column, width, alignment );

    _cg.setColor( QColorGroup::Text, c );
}

void tyeditorWindow::CheckBadMarkers()
{
	lstMarkers->setSorting(1);

	m_BadMarkersFound = false;

	/* Go through the list checking for bad markers */
	CDemuxCutPoint::eCutType LastType = CDemuxCutPoint::unknown;
	MarkerListItem *pItem = (MarkerListItem *)(lstMarkers->firstChild());
	while( pItem ) {
		bool BadEntry = false;

		if( pItem->m_Cut.m_Type == CDemuxCutPoint::cutstop && LastType != CDemuxCutPoint::cutstart ) {
			BadEntry = true;
			m_BadMarkersFound = true;
		}

		if( pItem->m_Cut.m_Type == CDemuxCutPoint::cutstart && 
			( LastType != CDemuxCutPoint::cutstop && LastType != CDemuxCutPoint::unknown ) ) {
			BadEntry = true;
			m_BadMarkersFound = true;
		}

		if( pItem->m_Cut.m_Type == CDemuxCutPoint::cutstart ||
			pItem->m_Cut.m_Type == CDemuxCutPoint::cutstop ) {
			LastType = pItem->m_Cut.m_Type;
		}
		if( pItem->m_BadEntry != BadEntry ) {
			pItem->m_BadEntry = BadEntry;
			pItem->repaint();
		}
		pItem = (MarkerListItem *)(pItem->itemBelow());
	}
	EnableControls();
}

void tyeditorWindow::ReLoadImage()
{
	if( !m_pDemux )
		return;

	/* Load the current frame */
	ImagePtr_t Image;
	int size;
	int64_t currentFrame = m_pDemux->GetStreamCurrentFrame();

	if( m_pDemux->GetCurrentImage( Image, size ) ) {
		LoadImage( size, Image );
		Slider1->setValue( currentFrame );
	}

	if( m_CounterMode == 1) {
		char frameString[25];
		FrameAsString( currentFrame, frameString);
		txtStreamTime->setText( frameString );
	} else if( m_CounterMode == 2) {
		char timeString[25];
		TimeAsString( m_pDemux->GetStreamCurrentTime(), timeString);
		txtStreamTime->setText( timeString );
	}
}

bool tyeditorWindow::LoadImage( int size, uint8_t *pData )
{
	bool bOK = m_Image.loadFromData( (const uchar *) pData, size, "PPM" );
	pixMapImage->setPixmap( m_Image );

	return ( bOK );
}

bool tyeditorWindow::SaveImage( const char *filename, const char *format )
{
	bool bOK = m_Image.save( filename, format );

	return ( bOK );
}

void tyeditorWindow::NavSlider1( int value )
{
	m_pDemux->SetStreamCurrentFrame( value );
	ReLoadImage();
}

void tyeditorWindow::NavSlider2( int value )
{
	bool bToggle;

	m_PlayRate = value;

	if( m_bFrameMode && m_pCurrentPlayButtonPressed ) {
		m_pCurrentPlayButtonPressed->setOn(false);
		m_pCurrentPlayButtonPressed = NULL;
	}

	/* If playing then make some buttons toggle buttons */
	bToggle = ( !m_bFrameMode ) ? true : false;
    btnNavPlayBack->setToggleButton( bToggle );
    btnNavPlay->setToggleButton( bToggle );
    btnNavFBack->setToggleButton( bToggle );
    btnNavFFwd->setToggleButton( bToggle );
}

void tyeditorWindow::FrameMode()
{
	m_bFrameMode = chkFrameMode->isChecked();

	if( m_bFrameMode ) {
		// Stop any playback
		if( m_pCurrentPlayButtonPressed ) {
			m_pCurrentPlayButtonPressed->setOn(false);
			m_pCurrentPlayButtonPressed = NULL;
		}
		// Stop the playback timer
		m_timer.stop();
	}

	/* If playing then make some buttons toggle buttons */
	bool bToggle = ( !m_bFrameMode ) ? true : false;
    btnNavPlayBack->setToggleButton( bToggle );
    btnNavPlay->setToggleButton( bToggle );
    btnNavFBack->setToggleButton( bToggle );
    btnNavFFwd->setToggleButton( bToggle );

	if( !m_bFrameMode ) {
		// Start playing
		NavPlay();
	} 

}

void tyeditorWindow::playButtonPressed( QPushButton *btn )
{
	if( (m_bFrameMode || !btn) && m_pCurrentPlayButtonPressed ) {
		/* Turn off playing button */
		m_pCurrentPlayButtonPressed->setOn( false );
		m_pCurrentPlayButtonPressed = NULL;
	}

	if( !m_bFrameMode && btn ) {
		if( btn != m_pCurrentPlayButtonPressed ) {
			/* Toggle it on and old button off */
			if( m_pCurrentPlayButtonPressed ) {
				m_pCurrentPlayButtonPressed->setOn( false );
			}
			m_pCurrentPlayButtonPressed = btn;
			m_pCurrentPlayButtonPressed->setOn( true );
		} else {
			/* Toggle it on/off */
			if( m_pCurrentPlayButtonPressed ) {
				btn->setOn( false );
				m_pCurrentPlayButtonPressed = NULL;
			} else {
				btn->setOn( true );
				m_pCurrentPlayButtonPressed = btn;
			}
		}
	}

	if( m_pCurrentPlayButtonPressed ) {
		// Start the playback timer
		m_LastDisplayTime.start();
		m_timer.start( 100, false );
	} else {
		// Stop the playback timer
		m_timer.stop();
	}
	setFocus();
}

void tyeditorWindow::NavStart()
{
	/* Toggle if we are playing back */
	playButtonPressed( NULL );

	m_pDemux->SetStreamCurrentFrame( 0 );
	ReLoadImage();
}

void tyeditorWindow::NavStop()
{
	/* Switch off playback */
	playButtonPressed( NULL );
}

void tyeditorWindow::NavPlayBack()
{
	m_PlaySpeed = -1;

	/* Toggle if we are playing back */
	playButtonPressed( btnNavPlayBack );

	if( m_bFrameMode ) {
		m_pDemux->SetStreamCurrentFrameRelative( -1 );
		ReLoadImage();
	}
}

void tyeditorWindow::NavPlay()
{
	m_PlaySpeed = 1;

	/* Toggle if we are playing back */
	playButtonPressed( btnNavPlay );

	if( m_bFrameMode ) {
		m_pDemux->SetStreamCurrentFrameRelative( 1 );
		ReLoadImage();
	}
}

void tyeditorWindow::NavFBack()
{
	m_PlaySpeed = -100;

	/* Toggle if we are playing back */
	playButtonPressed( btnNavFBack );

	if( m_bFrameMode ) {
		m_pDemux->SetStreamCurrentFrameRelative( -10 * m_PlayRate );
		ReLoadImage();
	}
}

void tyeditorWindow::NavFFwd()
{
	m_PlaySpeed = 100;

	/* Toggle if we are playing back */
	playButtonPressed( btnNavFFwd );

	if( m_bFrameMode ) {
		m_pDemux->SetStreamCurrentFrameRelative( 10 * m_PlayRate );
		ReLoadImage();
	}
}

void tyeditorWindow::NavEnd()
{
	/* Toggle if we are playing back */
	playButtonPressed( NULL );

	m_pDemux->SetStreamCurrentFrame((int)(m_pDemux->GetStreamFrames() ));
	ReLoadImage();
}

void tyeditorWindow::CutStart()
{
	new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::cutstart ) );
	CheckBadMarkers();
}

void tyeditorWindow::CutStop()
{
	new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::cutstop ) );
	CheckBadMarkers();
}

void tyeditorWindow::ChapterMark()
{
	new MarkerListItem( lstMarkers, m_pDemux->GetCurrentCut( CDemuxCutPoint::chapter ) );
	CheckBadMarkers();
}

void tyeditorWindow::Locate()
{
	MarkerListItem *pItem = (MarkerListItem *)(lstMarkers->selectedItem());
	if( pItem ) {
		m_pDemux->JumpToCut( pItem->m_Cut );
		ReLoadImage();
	}
}

void tyeditorWindow::MarkerCurrentChanged ( QListViewItem * )
{
	EnableControls();
}

void tyeditorWindow::MarkerDoubleClick ( QListViewItem * )
{
	Locate();
}

void tyeditorWindow::RemoveSelected()
{
	MarkerListItem *pItem = (MarkerListItem *)(lstMarkers->selectedItem());
	if( pItem ) {
		lstMarkers->takeItem( pItem );
		delete pItem;
	}
	CheckBadMarkers();
	setFocus();
}

void tyeditorWindow::RemoveAll()
{
	lstMarkers->clear();
	setFocus();
}

void tyeditorWindow::Process()
{
	m_mainWindow->Process();
}
