/*
 * Copyright (C) 2002, 2003  John Barrett (MrBassMan) <johnbarretthcc@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: tyeditorwindow.h,v 1.13 2003/03/30 01:41:14 mrbassman Exp $ */

#include <qmainwindow.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qsortedlist.h>
#include <qdatetime.h>
#include <qlistview.h>

#include "tydemuxclass.h"
#include "processor.h"
#include "EditorWidget.h"

class CDemuxInterface;
class tymainWindow;

class MarkerListItem: public QListViewItem
{
public:
	MarkerListItem( QListView * parent, CDemuxCutPoint Cut ) 
		: QListViewItem(parent), m_Cut(Cut), m_BadEntry(0) {};
    QString text( int column ) const;
    virtual void paintCell( QPainter *p, const QColorGroup &cg,
			    int column, int width, int alignment );

	CDemuxCutPoint m_Cut;
	bool	m_BadEntry;
};

class tyeditorWindow : public frmMainWindow
{
	Q_OBJECT
public:
	tyeditorWindow( tymainWindow* parent = 0 );
	~tyeditorWindow();

	void EnableControls();
	void SetCounterMode( int mode );
	bool LoadImage( int size, uint8_t *pData );
	bool SaveImage( const char *filename, const char *format );
	void ReLoadImage();
	void CheckBadMarkers();
	const char *GetVideoFileExtension();

	bool LoadCutFile( const char *filename );
	bool SaveCutFile( const char *filename );
	bool SetupProcessor( CDemuxProcessTask *processor );
	bool HasCutsToSave();

	void SetDemuxObject( CDemuxInterface *demux );

protected:
	void keyPressEvent ( QKeyEvent * e );

public slots:
	void NavStart();
	void NavPlayBack();
	void NavPlay();
	void NavFBack();
	void NavFFwd();
	void NavEnd();
	void NavStop();
	void NavSlider1( int value );
	void NavSlider2( int value );
	void FrameMode();
	void PlayTimer();
	void LoadTY();
	void LoadTivo();
	void CutStart();
	void CutStop();
	void ChapterMark();
	void Locate();
	void MarkerCurrentChanged ( QListViewItem *newCurrent );
	void MarkerDoubleClick ( QListViewItem * item);
	void RemoveSelected();
	void RemoveAll();
	void Process();

private:
	QPixmap m_Image;
	QTimer  m_timer;
	QTime   m_LastDisplayTime;
	QPushButton *m_pCurrentPlayButtonPressed;
	tymainWindow *m_mainWindow;
	bool    m_bFrameMode;

	int		m_CounterMode;
	int		m_PlayRate;
	int		m_PlaySpeed;
	CDemuxInterface *m_pDemux;
	bool m_BadMarkersFound;
	void playButtonPressed( QPushButton *btn );
};

