/*
 * Copyright (C) 2002, 2003  John Barrett (MrBassMan) <johnbarretthcc@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: tymainwindow.cpp,v 1.44 2003/03/30 01:41:14 mrbassman Exp $ */

#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qmessagebox.h> 
#include <qimage.h>
#include <qprogressdialog.h>
#include <qvbox.h>
#include <qprogressbar.h>

#include <assert.h>

#include "tymainwindow.h"
#include "nowshowingimpl.h"

#undef NEWLINE
#define NEWLINE "\n"
#include "../libs/about.c"

tymainWindow::tymainWindow( QWidget* parent, const char* name )
: QMainWindow( parent, name ), m_editWindow(NULL), m_pDemux(0)
{
	m_Icon.load( "tyeditor.png", "PNG" );
	setIcon ( m_Icon );

	/* Set a log file */
	unlink("editorlog.txt");
	m_Logfile = fsopen( "editorlog.txt", WRITE_MODE, SHARE_DENY_NONE );
	logger_init( log_info, m_Logfile, NULL, 0 );

	/* Set up the menu bar */
    menuBar()->setSeparator( QMenuBar::InWindowsStyle );
    QAction *fileNewAction = new QAction( "New", "&New Marker File", CTRL+Key_N, this, "new" );
    QAction *fileOpenAction = new QAction( "Open", "Open &Marker File...", CTRL+Key_M, this, "open" );
    QAction *fileOpenTyFileAction = new QAction( "Open TyFile", "Open &ty File...", CTRL+Key_T, this, "openty" );
    QAction *fileOpenTyTivoAction = new QAction( "Open TyTivo", "&Open Now Showing...", CTRL+Key_O, this, "opentivo" );
    QAction *fileSaveAction = new QAction( "SaveCuts", "&Save Markers", CTRL+Key_S, this, "save" );
    QAction *fileSaveAsAction = new QAction( "Save As", "Save Markers &As..", CTRL+Key_A, this, "saveas" );
    QAction *fileProcessAction = new QAction( "Process", "&Process", CTRL+Key_P, this, "process" );
    QAction *fileQuitAction = new QAction( "Quit", "&Quit", CTRL+Key_Q, this, "quit" );

    QPopupMenu *fileMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&File", fileMenu );

	/* A list of image save formats go in a sub-menu */
    QStrList fmt = QImage::outputFormats();
    fileSaveImageMenu = new QPopupMenu();
    for (const char* fmat = fmt.first(); fmat; fmat = fmt.next()) {
		fileSaveImageMenu->insertItem( fmat );
    }

    fileNewAction->addTo( fileMenu );
    fileOpenAction->addTo( fileMenu );
    fileOpenTyTivoAction->addTo( fileMenu );
    fileOpenTyFileAction->addTo( fileMenu );
	fileMenu->insertSeparator ();
    fileSaveAction->addTo( fileMenu );
    fileSaveAsAction->addTo( fileMenu );
    fileMenu->insertItem("Save &Image...", fileSaveImageMenu);
	fileMenu->insertSeparator ();
    fileProcessAction->addTo( fileMenu );
	fileMenu->insertSeparator ();
    fileQuitAction->addTo( fileMenu );

	connect( fileNewAction, SIGNAL( activated() ), this, SLOT( fileNew() ) );
	connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
	connect( fileOpenTyFileAction, SIGNAL( activated() ), this, SLOT( fileOpenTy() ) );
	connect( fileOpenTyTivoAction, SIGNAL( activated() ), this, SLOT( fileOpenTivo() ) );
	connect( fileSaveAction, SIGNAL( activated() ), this, SLOT( fileSave() ) );
	connect( fileSaveAsAction, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
	connect( fileProcessAction, SIGNAL( activated() ), this, SLOT( Process() ) );
	connect( fileQuitAction, SIGNAL( activated() ), qApp, SLOT( quit() ) );
    connect( fileSaveImageMenu, SIGNAL(activated(int)), this, SLOT(fileSaveImage(int)) );

    QPopupMenu *viewMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&View", viewMenu );
    viewFrameCounter = new QAction( "Frame Counter", "&Frame Counter", CTRL+Key_F, this, "framecounter", TRUE );
    viewTimeCounter = new QAction( "Time Counter", "&Time Counter", CTRL+Key_T, this, "timecounter", TRUE );
    viewLogFile = new QAction( "Log File", "&Log File", CTRL+Key_L, this, "logfile" );
    viewFrameCounter->addTo( viewMenu );
    viewTimeCounter->addTo( viewMenu );
	viewMenu->insertSeparator ();
    viewLogFile->addTo( viewMenu );
	connect( viewFrameCounter, SIGNAL( activated() ), this, SLOT( FrameCounter() ) );
	connect( viewTimeCounter, SIGNAL( activated() ), this, SLOT( TimeCounter() ) );
	connect( viewLogFile, SIGNAL( activated() ), this, SLOT( LogFile() ) );
	m_pLog = new logfile( this, "Log File", FALSE );

    QPopupMenu *toolsMenu = new QPopupMenu( this );

    menuBar()->insertItem( "&Tools", toolsMenu );
    QAction *toolsOptionsAction = new QAction( "Options", "&Options", CTRL+Key_O, this, "options" );
    toolsOptionsAction->addTo( toolsMenu );

	connect( toolsOptionsAction, SIGNAL( activated() ), this, SLOT( toolsOptions() ) );

    QPopupMenu *helpMenu = new QPopupMenu( this );

    menuBar()->insertItem( "&Help", helpMenu );
    QAction *helpKeysAction = new QAction( "Keyboard", "&Keyboard Shortcuts", CTRL+Key_K, this, "keyshortcuts" );
    QAction *helpAboutAction = new QAction( "About", "&About", CTRL+Key_A, this, "about" );
    helpKeysAction->addTo( helpMenu );
    helpAboutAction->addTo( helpMenu );

	connect( helpKeysAction, SIGNAL( activated() ), this, SLOT( helpKeys() ) );
	connect( helpAboutAction, SIGNAL( activated() ), this, SLOT( helpAbout() ) );

	/* Create an options dialog */
	m_pOptions = new options( this, "QT TyEditor Options" );

	/* Now create an editor windows which contains the main controls */
	m_editWindow = new tyeditorWindow( this );
	viewTimeCounter->setOn( true );
	m_editWindow->SetCounterMode( 2 );

	setCentralWidget( m_editWindow );

	/* Resize ourselves to the same size as the editor window - allowing for the menu bar height and borders */
	QSize newsize = m_editWindow->size();
	newsize.setHeight( newsize.height() + 20 );
	setMinimumSize( newsize.width(), newsize.height() );

	SetStreamName( "No stream loaded" );
}

tymainWindow::~tymainWindow()
{
	delete m_pDemux;

	logger_free();
	fclose(m_Logfile);
}

void tymainWindow::SetStreamName( const char *name )
{
	// Remove the path component if a file
	const char *slash = strrchr( name, '/' );
	if( !slash ) {
		slash = strrchr( name, '\\' );
	}
	if( !slash ) {
		slash = strrchr( name, ':' );
	}
	if( slash ) {
		m_StreamName = slash+1;
	} else {
		m_StreamName = name;
	}

	setCaption( "Qt TyStudio Editor: " + m_StreamName);
}

void tymainWindow::fileNew()
{
	m_CutFile = "";
	m_editWindow->RemoveAll();
}

void tymainWindow::fileOpen()
{
	if( !m_pDemux )
		return;

	QString filter = "Cut files (*.cut)";

    QString f = GetOptions()->m_MarkerFileDir;

	f = QFileDialog::getOpenFileName( f, filter );
    if ( !f.isEmpty() ) {
        m_CutFile = f;

		// Do the loading
		m_editWindow->LoadCutFile( m_CutFile );
	}
}

void tymainWindow::fileOpenTy()
{
	QString filter = "Tivo streams (*.ty)";

    QString f = GetOptions()->m_tyFileDir;
	f = QFileDialog::getOpenFileName( f, filter, this );
    if ( !f.isEmpty() ) {
        m_tyFile = f;
		startNewDemuxSession();

		SetStreamName( m_tyFile );
    }
}

bool tymainWindow::doCopyFile( int fsid, const char *dest )
{
	bool bResult = false;
	bool bCancelled = false;
	int  pcComplete = -1;

	/* Create an index */
    QProgressDialog *progress = new QProgressDialog( "Copying file from TIVO...", "Cancel", 100,
                                this, "progress", TRUE );
    QString Caption("Qt ");
	Caption += dest;
	progress->setCaption( Caption );

	CDemuxCopyTask *copier = m_pDemux->CreateCopier( fsid, dest, m_Logfile );
	copier->StartJob();

   	while( !bCancelled && !copier->IsCompleted() ) {
		qApp->processEvents();
		if( pcComplete != copier->GetProgress() ) {
			pcComplete = copier->GetProgress();
			progress->setProgress( pcComplete );
		}

		bCancelled = progress->wasCancelled();
   	}
   	progress->setProgress(100);

	if( copier->GetResult() == 0 ) {
		bResult = true;
	}

	delete copier;

    if ( bCancelled ) {
		delete m_pDemux;
		m_pDemux = NULL;
		m_editWindow->SetDemuxObject( NULL );
		SetStreamName( "No Stream Loaded" );
		return false;
	}

	return( bResult );
}

void tymainWindow::fileOpenTivo()
{
	delete m_pDemux;
	m_pDemux = NULL;
	m_editWindow->SetDemuxObject( NULL );
	SetStreamName( "No Stream Loaded" );

	m_pDemux = new CDemuxInterface();
	assert( m_pDemux );

	NowShowing *dialog = new NowShowing( GetOptions()->m_TivoHostName, m_pDemux, this, "Now Showing", true);
	dialog->exec();
	int fsid = dialog->GetFsid();
	if( fsid && dialog->result() == QDialog::Accepted ) {

		if( dialog->CopyToPC() ) {
			/* Transfer the file locally */
			QString filter = "TY files (*.ty)";

			QString streamName = dialog->GetStreamName();

			/* get the output file name */
			QString outfile;
			outfile = GetOptions()->m_tyFileDir;
			if( isdigit( streamName[0].latin1() ) ) {
				// Bug in QT - name must start with non-numeric to show in getSaveFileName
				outfile += "ty";
			}
			outfile += streamName;
			outfile += ".ty";
			outfile = QFileDialog::getSaveFileName( outfile, filter );
			if ( !outfile.isEmpty() && doCopyFile( fsid, outfile ) ) {
				m_tyFile = outfile;
				SetStreamName( m_tyFile );
				startNewDemuxSession();
			} else {
				delete m_pDemux;
				m_pDemux = NULL;
			}
		} else {
			/* Use the file remotely */

			QApplication::setOverrideCursor( waitCursor ); // this might take time

			if( !m_pDemux->OpenRemoteTivoFile( fsid ) ) {
				QApplication::restoreOverrideCursor();	// restore original cursor
				QMessageBox::critical( this, "TY Editor", "Failed to open the selected show." );
				SetStreamName( "No Stream Loaded" );
				delete m_pDemux;
				m_pDemux = NULL;
				return;
			} else {
				QApplication::restoreOverrideCursor();	// restore original cursor
				SetStreamName( dialog->GetStreamName() );
				doIndex();
			}
		}
	}
}

void tymainWindow::fileSave()
{
	if( !m_pDemux || !m_editWindow->HasCutsToSave() )
		return;

	if( m_CutFile.isEmpty() ) {
		fileSaveAs();
	} else if ( !m_CutFile.isEmpty() ) {
		// Do the saving
		m_editWindow->SaveCutFile( m_CutFile );
	}
}

void tymainWindow::fileSaveAs()
{
	if( !m_pDemux || !m_editWindow->HasCutsToSave() )
		return;

	QString filter = "Cut files (*.cut)";

	/* get the output file name */
	QString outfile;
	outfile = GetOptions()->m_MarkerFileDir;

	if( isdigit( m_StreamName[0].latin1() ) ) {
		// Bug in QT - name must start with non-numeric to show in getSaveFileName
		outfile += "ty";
	}
	outfile += m_StreamName;
	int index = outfile.findRev( '.' );
	if( index != -1 ) {
		outfile = outfile.mid( 0, index );
	}
	outfile += ".cut";

	outfile = QFileDialog::getSaveFileName( outfile, filter );
	if ( !outfile.isEmpty() ) {
		m_CutFile = outfile;

		// Do the saving
		m_editWindow->SaveCutFile( m_CutFile );
	}
}


void tymainWindow::fileSaveImage(int n)
{
	if( !m_pDemux ) {
		return;
	}

	QString filter = "Image files (*.";
	filter += fileSaveImageMenu->text(n);
	filter += ")";

	/* get the output file name */
	QString outfile;
	outfile = GetOptions()->m_OutputFileDir;
	if( isdigit( m_StreamName[0].latin1() ) ) {
		// Bug in QT - name must start with non-numeric to show in getSaveFileName
		outfile += "ty";
	}
	outfile += m_StreamName;
	int index = outfile.findRev( '.' );
	if( index != -1 ) {
		outfile = outfile.mid( 0, index );
	}
	outfile += ".";
	outfile += fileSaveImageMenu->text(n);
    outfile = QFileDialog::getSaveFileName( outfile, filter, this );
    if ( !outfile.isEmpty() ) {
		m_editWindow->SaveImage( outfile, fileSaveImageMenu->text(n) );
	}
}

void tymainWindow::FrameCounter()
{
	viewTimeCounter->setOn( false );
	if( viewFrameCounter->isOn() )
		m_editWindow->SetCounterMode( 1 );
	else
		m_editWindow->SetCounterMode( 0 );
}

void tymainWindow::TimeCounter()
{
	viewFrameCounter->setOn( false );
	if( viewTimeCounter->isOn() )
		m_editWindow->SetCounterMode( 2 );
	else
		m_editWindow->SetCounterMode( 0 );
}

void tymainWindow::toolsOptions()
{
	m_pOptions->exec();
}

void tymainWindow::helpAbout()
{
	char txtbuf[2000];
	memset(txtbuf, '\0', 1999);
	snprintf(txtbuf,1999,"TyStream editor and processor\n\nVersion 0.5.0\n\nTyEditor %s", about);
	QMessageBox::about( this, "TyEditor", txtbuf);
		
}

void tymainWindow::helpKeys()
{
	QMessageBox::about( this, "TY Editor", "Arrow Keys\n\n" 
		"Right - Play or step to next frame\nShift-Right - Fast Forward\nCTRL-Right - Jump to end of stream\n\n" 
		"Left - Play Backwards or step to previous frame\nShift-Left - Fast Backward\nCTRL-Left - Jump to start of stream\n\n" 
		"Up - Play speed increase\nShift-Up - Play full speed\n" 
		"Down - Play speed decrease\nShift-Down - Slowest play speed\n" 
		"\nSpacebar - Toggle between frame mode and play\n" 
		"\nShift Spacebar - Add cut start marker\nCTRL+Spacebar - Add cut stop marker" 
		);
}

void tymainWindow::doIndex()
{
	bool bcancelled = false;
	int  pcComplete = -1;

 	/* Create an index */
    QString LabelText("Indexing...");
    QProgressDialog *progress = new QProgressDialog( LabelText, "Cancel", 100,
                                this, "progress", TRUE );

    QString Caption("Qt ");
	Caption += m_StreamName;
	progress->setCaption( Caption );
	QSize newsize = progress->size();
	newsize.setWidth( newsize.width() * 2 );
	progress->setMinimumSize( newsize.width(), newsize.height() );

	CDemuxIndexTask *indexer = m_pDemux->CreateIndexer(m_Logfile);
	indexer->StartJob();

    while( !bcancelled && !indexer->IsCompleted() ) {
        qApp->processEvents();
        if( LabelText != indexer->GetProgressText() ) {
			LabelText = indexer->GetProgressText();
	        progress->setLabelText( LabelText );
		}
		if( pcComplete != indexer->GetProgress() ) {
			pcComplete = indexer->GetProgress(); 
	        progress->setProgress( pcComplete );
		}
        bcancelled = progress->wasCancelled();
    }
    progress->setProgress(100);

	if( indexer->GetResult() != 0 ) {
		QMessageBox::critical( this, "TY Editor", "Indexing failed - check the log file." );
		bcancelled = true;
		m_pLog->show();
		m_pLog->refresh();
	}

	delete indexer;

    if ( bcancelled ) {
		delete m_pDemux;
		m_pDemux = NULL;
		SetStreamName( "No Stream Loaded" );
	}

	m_editWindow->SetDemuxObject( m_pDemux );
}

void tymainWindow::startNewDemuxSession()
{
	m_editWindow->SetDemuxObject( NULL );

	if( m_tyFile.isEmpty() ) {
		QMessageBox::warning( this, "TY Editor", "No TY file loaded, please select a ty file to load." );
		fileOpenTy();
		return;
	}

	delete m_pDemux;
	m_pDemux = NULL;

	m_pDemux = new CDemuxInterface();
	assert( m_pDemux );

    QApplication::setOverrideCursor( waitCursor ); // this might take time

	/* Start a new demux session for this file */
	if( !m_pDemux->LoadTY( m_tyFile ) ) {
	    QApplication::restoreOverrideCursor();	// restore original cursor
		QMessageBox::critical( this, "TY Editor", "Failed to load the selected TY file." );
		delete m_pDemux;
		m_pDemux = NULL;
		SetStreamName( "No Stream Loaded" );
		return;
	}
	SetStreamName( m_tyFile );

    QApplication::restoreOverrideCursor();	// restore original cursor

	doIndex();
}

void tymainWindow::Process()
{
	if( m_pDemux ) {
		/* get the output file name */
		QString filter = "MPEG files (*.mpg)";

		/* get the output file name */
		QString outfile;
		outfile = GetOptions()->m_OutputFileDir;
		if( isdigit( m_StreamName[0].latin1() ) ) {
			// Bug in QT - name must start with non-numeric to show in getSaveFileName
			outfile += "ty";
		}

		outfile += m_StreamName;

		// Replace the extension with .m2a
		int index = outfile.findRev('.');
		if( index != -1 ) {
			outfile = outfile.mid( 0, index );
		}
		outfile += m_editWindow->GetVideoFileExtension();

		outfile = QFileDialog::getSaveFileName( outfile, filter );
		if ( outfile.isEmpty() ) {
			return;
		}

		/* Process the file with any cuts */
		QString LabelText("Processing...");
		QProgressDialog *progress = new QProgressDialog( LabelText, "Cancel", 100,
						this, "progress", TRUE );
		QString Caption("Qt ");
		Caption += m_StreamName;
		progress->setCaption( Caption );

		CDemuxProcessTask processor( m_tyFile, outfile, m_Logfile );

		/* Set user configured options */
		processor.SetModuleOptions( GetOptions()->m_DemuxParams, GetOptions()->m_TranscodeParams,
									GetOptions()->m_MplexParams );

		/* Configure the processor */
		m_pDemux->SetupProcessor( &processor );
		if( !m_editWindow->SetupProcessor( &processor ) )
			return;

		if( m_pDemux->IsStreamRemote() ) {
			/* Close the current TY stream */
			m_editWindow->SetDemuxObject( NULL );
			delete m_pDemux;
			m_pDemux = NULL;
			SetStreamName( "No Stream Loaded" );
		}

		processor.StartJob();
		int pcComplete = -1;
		bool bWasCancelled = false;
		while( !processor.IsCompleted() ) {
			qApp->processEvents();
        	if( LabelText != processor.GetProgressText() && processor.GetProgressText()[0] ) {
				LabelText = processor.GetProgressText();
        		progress->setLabelText( LabelText );
			}
			if( pcComplete != processor.GetProgress() ) {
				pcComplete = processor.GetProgress(); 
				progress->setProgress( pcComplete );
			}

			if ( progress->wasCancelled() ) {
				bWasCancelled = true;
				break;
			}
		}

        progress->setProgress(100);

		if( bWasCancelled ) {
			QMessageBox::information( this, "TY Editor", "Processing cancelled." );
		} else if( processor.GetResult() == 0 ) {
			QMessageBox::information( this, "TY Editor", "Processing completed successfully." );
		} else {
			QString Msg = "Processing failed - see the editor log for more information.\nLast message was:\n";
			Msg += LabelText;

			QMessageBox::warning( this, "TY Editor", Msg );
			m_pLog->show();
			m_pLog->refresh();
		}
	}
}

void tymainWindow::LogFile()
{
	if( m_pLog->isVisible() ) {
		m_pLog->hide();
	} else {
		m_pLog->show();
		m_pLog->refresh();
	}
}

