/*
 * Copyright (C) 2002  John Barrett <johnbarretthcc@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* "$Id: typrocess.h,v 1.12 2003/03/26 11:52:42 mrbassman Exp $" */
/* TyProcess library header file */

#ifndef __TYPROCESS_H__
#define __TYPROCESS_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LOG_MODULE
#define LOG_MODULE "typrocess"
#endif

#include "tylogging.h"
#include "tycommon.h"

/* Demux module specific startup parameters */
typedef struct {
	int	argc;				/* Additional command line parameters */
	char **argv;			/* As above */
	char **cutlist;			/* List of cuts */
	int cutcount;			/* Number of cuts in the list */
	void *remote_holder;	/* Remote holder if the file is still on TIVO */
	int fsid;				/* fsid of remote file */
} demux_params_t;

/* Transcode module specific startup parameters */
typedef struct {
	int	argc;						/* Additional command line parameters */
	char **argv;					/* As above */
	int format;						/* 0 for default, 1 for AC3, 2 for MPEG */
	int rate;						/* New sample rate 0 for default */
	int bitrate;					/* New bitrate 0 for default */
	int dbadjust;					/* New db adjust level DEFAULT_DB_ADJUST default */
} transcode_params_t;

/* Mplex module specific startup parameters */
typedef struct {
	int	argc;						/* Additional command line parameters */
	char **argv;					/* As above */
	int format;						/* Mplex -f parameter (+1) over-ride 0 for default */
} mplex_params_t;

typedef struct {
	int demux;									/* Run demux flag */
	int transcode;								/* Run transcode flag */
	int mplex;									/* Run mplex flag */
	int use_pipes;								/* Use stdin for .ty and stdout for output flag */
	int *progress;								/* %complete progress pointer */
	log_hook_function	loghookfn;
	loglevel_t			loglevel;
	FILE	*			logfile;

	char in_audio_file[ MAXPATHLEN + 1];		/* Input audio file if not demuxing */
	char in_video_file[ MAXPATHLEN + 1];		/* Input video file if not demuxing */
	char in_ty_file[ MAXPATHLEN + 1];			/* Input tyfile if demuxing */
	char out_mpeg_file[ MAXPATHLEN + 1];		/* Output mpeg file if mplexing and not use_pipes */
	char out_video_file[ MAXPATHLEN + 1];		/* Output video file if not mplexing */
	char out_audio_file[ MAXPATHLEN + 1];		/* Output audio file if not mplexing */

	demux_params_t demux_params;				/* demux specific parameters */
	transcode_params_t transcode_params;		/* transcode specific parameters */
	mplex_params_t mplex_params;				/* mplex specific parameters */

} typrocess_params_t;

/* Main entry point function
 * Returns: 0 - success, 1 - Usage error, else other error code */
extern int typrocess( typrocess_params_t *params );

#ifdef _WIN32

#ifdef _EXPORTING
extern "C" __declspec( dllexport ) int TyProcessDll( typrocess_params_t FAR *lpParams );
#else
extern "C" __declspec( dllimport ) int TyProcessDll( typrocess_params_t FAR *lpParams );
#endif

#endif // _WIN32

#ifdef __cplusplus
}
#endif

#endif /* __TYPROCESS_H__ */
