// ProgressDlg.cpp : implementation file
//

#include "stdafx.h"
#include "typrocessgui.h"
#include "ProgressDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static CProgressDlg *pDlg = 0;
static bool bThreadShouldDie = false;
static bool bThreadRunning = false;

extern "C" {
/////////////////////////////////////////////////////////////////////////////
// CProgressDlg dialog

int logger_hook(int level, const char *module, const char *msg )
{
	if( bThreadShouldDie )
	{
		bThreadRunning = false;
		AfxEndThread(-1);  /* Die */
	}

	if( pDlg )
		return( pDlg->LoggerHook( level, module, msg ) );

	return 0;
}
}

UINT ProcessThread( LPVOID pParam )
{
	int result = -1;
	typrocess_params_t *pParams;

	bThreadRunning = true;

	pParams = ( typrocess_params_t * ) pParam;
	result = TyProcessDll( pParams );

	if( !bThreadShouldDie && pDlg )
		pDlg->ThreadComplete( result );

	bThreadRunning = false;

	return( result );
}

CProgressDlg::CProgressDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CProgressDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CProgressDlg)
	//}}AFX_DATA_INIT
}

CProgressDlg::~CProgressDlg()
{
	if( m_thread )
		delete m_thread;
}

void CProgressDlg::ThreadComplete( int result )
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState( ));

	m_cs.Lock();
	if( result != 0 )
	{
		LoggerHook(0, "", "***** PROCESS COMPLETED WITH FAILURE *****" );
	}
	else
	{
		LoggerHook(0, "", "***** PROCESS COMPLETED OK *****" );
	}

	m_ButtonCancel.SetWindowText( "OK" );

	pDlg = NULL;

	m_cs.Unlock();
}

int CProgressDlg::LoggerHook(int level, const char *module, const char *msg )
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState( ));

	m_cs.Lock();

	CString LogMsg = msg;
	LogMsg.TrimRight(" \t\r\n");

	if( level == log_progress )
	{
		if( strcmp( module, "demux" ) == 0 )
		{
			m_DemuxProgressWnd.SetWindowText( LogMsg );
		}
		else if( strcmp( module, "transcode" ) == 0 )
		{
			m_TranscodeProgressWnd.SetWindowText( LogMsg );
		}
		else if( strcmp( module, "mplex" ) == 0 )
		{
			m_MplexProgressWnd.SetWindowText( LogMsg );
		}
	}
	else
	{
		/* Append to the log window */
		LogMsg += "\r\n";
		m_LogWindow.SetSel( -1, -1, TRUE );
		m_LogWindow.ReplaceSel( LogMsg );
	}

	m_cs.Unlock();
	return 0;
}


void CProgressDlg::SetParams( typrocess_params_t &Params )
{
	m_Params = Params;
	m_Params.loghookfn = logger_hook;

	pDlg = this;
	bThreadShouldDie = false;
}

void CProgressDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CProgressDlg)
	DDX_Control(pDX, IDC_TRANSCODE_PROGRESS, m_TranscodeProgressWnd);
	DDX_Control(pDX, IDC_MPLEX_PROGRESS, m_MplexProgressWnd);
	DDX_Control(pDX, IDC_DEMUX_PROGRESS, m_DemuxProgressWnd);
	DDX_Control(pDX, IDCANCEL, m_ButtonCancel);
	DDX_Control(pDX, IDC_LOGWINDOW, m_LogWindow);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CProgressDlg, CDialog)
	//{{AFX_MSG_MAP(CProgressDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgressDlg message handlers

int CProgressDlg::DoModal() 
{
	return CDialog::DoModal();
}

BOOL CProgressDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_thread = AfxBeginThread( ProcessThread, &m_Params );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CProgressDlg::OnCancel() 
{
	if( bThreadRunning )
	{
		bThreadShouldDie = true;  /* Causes the thread to die during the next log line output */

		BeginWaitCursor();

		while( bThreadRunning )
			Sleep( 1000 );

		EndWaitCursor();
	}

	CDialog::OnCancel();
}
