/*
 * Copyright (C) 2002  Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MPEG_AUDIO_CONVERT_H
#define MPEG_AUDIO_CONVERT_H

/*
  first call InitMpegAudioConverter, then for each audio frame
  call InputFrame, you then need to call GetOutputFrame until it returns NULL
  then finaly call GetLastFrame to see if we had a partial frame at the end.

  NOTE: The frame pointer from the GetX functions will remain valid until
        the next InputFrame call is made. Also the GetX functions parameter
        Hdr can be NULL if no info is required
*/

// errors that can be returned
typedef enum  {
  AC_OK                     =  0,
  AC_ERROR                  = -1,
  AC_NO_FRAME_FOUND         = -2,
  AC_NO_FRAME_INFO_AVAIL    = -3,
  AC_NOT_MPEG1              = -4,
  AC_NOT_LAYER2             = -5,
  AC_TRANSCODE_FAILED       = -6,
  AC_DECODE_FAILED          = -7,
  AC_INVALID_SAMPLE_RATE    = -8,
  AC_INVALID_BIT_RATE       = -9,
  AC_A52_LIB_INIT_FAILED    = -10,
  AC_A52_FRAME_FAILED       = -11,
  AC_A52_BLOCK_FAILED       = -12,
  AC_RAN_OUT_OF_INPUT_BYTES = -13,
  AC_INVALID_OUTPUT_FORMAT  = -14,
  AC_MPEG_ENC_INIT_FAILED   = -15,
  AC_AC3_ENC_INIT_FAILED    = -16,
  AC_DIM
} AC_ERRORS;

typedef enum  {
  MPEG_FRAME_SYNC = 0x7ff,
  AC3_FRAME_SYNC  = 0xb77
} FRAME_SYNCS;

typedef enum  {
  MPEG_1_VER   = 3,
  MPEG_2_5_VER = 0,
  MPEG_2_VER   = 2
} MPEG_VERS;

typedef enum  {
  MPEG_LAYER1 = 3,
  MPEG_LAYER2 = 2,
  MPEG_LAYER3 = 1
} MPEG_LAYERS;

typedef enum  {
  SAMP_32000 = 32000,
  SAMP_44100 = 44100,
  SAMP_48000 = 48000
} SAMPLE_RATES;

typedef enum  {
  CHANNEL_MODE_STEREO,
  CHANNEL_MODE_JOINT_STEREO,
  CHANNEL_MODE_DUAL_STEREO,
  CHANNEL_MODE_SINGLE_MONO
} MPEG_CHANNEL_MODE;

typedef enum  {
  PSYCO_MODEL_NONE,
  PSYCO_MODEL_1,
  PSYCO_MODEL_2,
} MPEG_ENCODE_PSYCO_MODEL;

typedef enum  {
  AC3_CHANNEL_C,      // channel
  AC3_CHANNEL_LR,     // stereo (left right)
  AC3_CHANNEL_3F,     // left, center, right
  AC3_CHANNEL_2F2R,   // left, right, rear left, rear right
  AC3_CHANNEL_3F2R,   // left, center, right, rear left, rear right
  AC3_CHANNEL_3F2RLFE // left, center, right, rear left, rear right, LEF (low freq effects)
} AC3_CHANNELS;

typedef struct {
  FRAME_SYNCS FrameSync;
  union  {
    struct {
      MPEG_VERS         AudioVer;
      MPEG_LAYERS       LayerDescrition;
      unsigned long     Protection;
      unsigned long     BitrateIndex;
      unsigned long     SampleRateIndex;
      unsigned long     Padding;
      unsigned long     Private;
      MPEG_CHANNEL_MODE ChannelMode;
      unsigned long     ModeExtension;
      unsigned long     Copyright;
      unsigned long     Original;
      unsigned long     Emphasis;
      unsigned short    HdrCrc;

      unsigned long     BitRate;
      unsigned long     SampleRate;
      unsigned long     FrameLen;
    } mpeg;
    struct {
      int               SampleRate;
      int               BitRate;
      int               Flags;
      unsigned long     FrameLen;
      int               Channels;
    } ac3;
  } u;
} FRAME_HEADER_INFO, *PFRAME_HEADER_INFO;


#endif
