/*
 * Copyright (C) 2002  Rowan McFarland <tytranscode@hotmail.com>
 *
 *  parts extracted from mad - MPEG audio decoder and toolame audio encoder
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MPEG_AUDIO_H
#define MPEG_AUDIO_H

#include "tytypes.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <math.h>
#include <assert.h>

#include "../libs/threadlib.h"		   // Threading and pipe library

#define LOG_MODULE "transcode"
#include "tylogging.h"		   // Logging library

#define NB_BLOCKS             6 // number of PCM blocks inside an AC3 frame

#define MPEG_FRAME_SIZE       1152
#define AC3_FRAME_SIZE        (NB_BLOCKS * 256)
#define MAX_FRAME_SIZE        1536

#define STREAM_BUFFERSIZE 4096

#define MAX_DB_RANGE      50
#define DB_STEP           1

#include "audio_convert.h"
#include "bit_stream.h"
#include "transcode.h"
#include "layer2_encoder.h"
#include "ac3.h"
#include "../liba52/inttypes.h"
#include "../liba52/mm_accel.h"
#include "../liba52/a52.h"


#ifndef min
#define min(a, b)  (((a) < (b)) ? (a) : (b))
#endif

# if SIZEOF_INT >= 4
typedef   signed int mad_fixed_t;

typedef   signed int mad_fixed64hi_t;
typedef unsigned int mad_fixed64lo_t;
# else
typedef   signed long mad_fixed_t;

typedef   signed long mad_fixed64hi_t;
typedef unsigned long mad_fixed64lo_t;
# endif


#define F_FRACBITS            28
#define f_mul(x, y)           ((((x) + (1L << 11)) >> 12) *  (((y) + (1L << 15)) >> 16))
#define MAD_F(x)              ((mad_fixed_t) (x##L))
#define MAD_F_ONE             MAD_F(0x10000000)
#define mad_f_todouble(x)     ((double)  ((x) / (double) (1L << F_FRACBITS)))
#define mad_f_tofixed(x)	    ((mad_fixed_t)  ((x) * (double) (1L << F_FRACBITS) + 0.5))
#define timer_sign( timer )   timer_compare( (timer), timer_zero )
#define MAD_TIMER_RESOLUTION  352800000UL


#define LOG_TY_USERDIAG( msg ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_info, LOG_MODULE, __FILE__, __LINE__, msg );\
  }\
  else {\
    DebugTrace( msg );\
  }
#define LOG_TY_USERDIAG1( msg, p1 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_info, LOG_MODULE, __FILE__, __LINE__, msg, p1 );\
  }\
  else {\
    DebugTrace( msg, p1 );\
  }
#define LOG_TY_USERDIAG2( msg, p1, p2 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_info, LOG_MODULE, __FILE__, __LINE__, msg, p1, p2 );\
  }\
  else {\
    DebugTrace( msg, p1, p2 );\
  }
#define LOG_TY_USERDIAG3( msg, p1, p2, p3 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_info, LOG_MODULE, __FILE__, __LINE__, msg, p1, p2, p3 );\
  }\
  else {\
    DebugTrace( msg, p1, p2, p3 );\
  }

#define LOG_TY_ERROR( msg ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_error, LOG_MODULE, __FILE__, __LINE__, msg );\
  }\
  else {\
    DebugTrace( msg );\
  }
#define LOG_TY_ERROR1( msg, p1 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_error, LOG_MODULE, __FILE__, __LINE__, msg, p1 );\
  }\
  else {\
    DebugTrace( msg, p1 );\
  }
#define LOG_TY_ERROR2( msg, p1, p2 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_error, LOG_MODULE, __FILE__, __LINE__, msg, p1, p2 );\
  }\
  else {\
    DebugTrace( msg, p1, p2 );\
  }
#define LOG_TY_ERROR4( msg, p1, p2, p3, p4 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_error, LOG_MODULE, __FILE__, __LINE__, msg, p1, p2, p3, p4 );\
  }\
  else {\
    DebugTrace( msg, p1, p2, p3, p4 );\
  }

#define LOG_TY_FATAL( msg ) {\
  if ( m_RunAsLib ) {\
	  logger_log( log_fatal, LOG_MODULE, __FILE__, __LINE__, msg );\
  }\
  else {\
    DebugTrace( msg );\
  }\
	exit( -1 ); }
#define LOG_TY_FATAL1( msg, p1 ) {\
  if ( m_RunAsLib ) {\
	  logger_log( log_fatal, LOG_MODULE, __FILE__, __LINE__, msg, p1 );\
  }\
  else {\
    DebugTrace( msg, p1 );\
  }\
	exit( -1 ); }
#define LOG_TY_FATAL2( msg, p1, p2 ) {\
  if ( m_RunAsLib ) {\
	  logger_log( log_fatal, LOG_MODULE, __FILE__, __LINE__, msg, p1, p2 );\
  }\
  else {\
    DebugTrace( msg, p1, p2 );\
  }\
	exit( -1 ); }
#define LOG_TY_PROGRESS1( msg, p1 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_progress, LOG_MODULE, __FILE__, __LINE__, msg, p1 );\
  }\
  else {\
    DebugTrace( msg, p1 );\
  }
#define LOG_TY_PROGRESS2( msg, p1, p2 ) \
  if ( m_RunAsLib ) {\
	  logger_log( log_progress, LOG_MODULE, __FILE__, __LINE__, msg, p1, p2 );\
  }\
  else {\
    DebugTrace( msg, p1, p2 );\
  }

#define BAD_BITRATE_IDX 0xf

typedef struct {
  FRAME_SYNCS FrameSync;
  union  {
    struct {
      SAMPLE_RATES            SampleRate;
      int32_t                 BitRate;
      MPEG_CHANNEL_MODE       ChannelMode;
      MPEG_ENCODE_PSYCO_MODEL PsychoModel;
    } mpeg;
    struct {
      SAMPLE_RATES            SampleRate;
      int32_t                 BitRate;
    } ac3;
  } u;
} ENCODE_SETTINGS, *PENCODE_SETTINGS;

typedef struct {
  long          Seconds;    /* whole seconds */
  unsigned long Fraction; /* 1/MAD_TIMER_RESOLUTION seconds */
} TIME_T, *PTIME_T;

typedef struct {
  unsigned long ClippedSamples;
  mad_fixed_t   PeakClipping;
  mad_fixed_t   PeakSample;
  TIME_T        TotalPlayTime;
} AUDIO_STATS, *PAUDIO_STATS;

typedef struct {
  mad_fixed_t Error[3];
  mad_fixed_t Random;
} AUDIO_DITHER, *PAUDIO_DITHER;

typedef struct {
  unsigned int    SampleRate;         /* sampling frequency (Hz) */
  unsigned short  Channels;           /* number of channels */
  unsigned short  Length;             /* number of samples per channel */
  mad_fixed_t     Samples[2][MAX_FRAME_SIZE];   /* PCM output samples [ch][sample] */
} PCM_TYPE, *PPCM_TYPE;

typedef struct {
  mad_fixed_t  Filter[2][2][2][16][8]; /* polyphase filterbank outputs */
                                       /* [ch][eo][peo][s][v] */

  unsigned int Phase;     /* current processing phase */

  PCM_TYPE     Pcm;     /* PCM output */
} SYNTH_TYPE, *PSYNTH_TYPE;

typedef struct {
  mad_fixed_t Bandfactors[2][32];
} EQUALIZER_TYPE;

typedef struct {
  FRAME_HEADER_INFO FrameHdr;
  mad_fixed_t       SbSample[2][36][32];  /* synthesis subband filter samples */
  TIME_T            FrameDuration;
  int               FrameCnt;
  short             OutputChannels;           // number of channels that will be used to create output


  // persistent data, live's via all frames processed
  mad_fixed_t       MpegAttenuation;
  int               Ac3Attenuation;
  SYNTH_TYPE        Synth;
  AUDIO_DITHER      LeftDither;
  AUDIO_DITHER      RightDither;
  AUDIO_STATS       Stats;

  short             DecodedSamples[MAX_CHANNELS][MAX_FRAME_SIZE];

  a52_state_t      *A52State;

  // stream control
  BIT_STREAM_TYPE   BitStream;

} AUDIO_DECODE, *PAUDIO_DECODE;

typedef struct {
  int nb_channels;
  int nb_all_channels;
  int lfe_channel;
  int bit_rate;
  int sample_rate;
  int bsid;
  int frame_size_min; // minimum frame size in case rounding is necessary
  int frame_size;     // current frame size in words
  int halfratecod;
  int frmsizecod;
  int fscod;          // frequency
  int acmod;
  int lfe;
  int bsmod;
  short last_samples[AC3_MAX_CHANNELS][256];
  int chbwcod[AC3_MAX_CHANNELS];
  int nb_coefs[AC3_MAX_CHANNELS];

  // bitrate allocation control
  int sgaincod, sdecaycod, fdecaycod, dbkneecod, floorcod;
  AC3BitAllocParameters bit_alloc;
  int csnroffst;
  int fgaincod[AC3_MAX_CHANNELS];
  int fsnroffst[AC3_MAX_CHANNELS];
  // mantissa encoding
  int mant1_cnt, mant2_cnt, mant4_cnt;

  PBIT_STREAM_TYPE   BitStream;
} AC3_ENCODE, *PAC3_ENCODE;

typedef struct {
  int16_t             TranscodedOutput[MAX_CHANNELS][MAX_FRAME_SIZE];
  unsigned long       RateConvertIdx;                       // index of created rate converters
//RLM - need to malloc this?
  rateconvert_t       RateConvert[MAX_RATE_CONVERT];        // max three different rate conversions
  short               TranscodedSamples[MAX_CHANNELS][MAX_FRAME_SIZE * 2];  // output, need extra room when going from 32->44.1 or 48
  TRANSCODED_SAMPLES  Samples;
  PCHANNEL_MAP        SamplesChannelMap;
} AUDIO_TRANSCODE;

typedef struct {
  // encode settings
  ENCODE_SETTINGS   EncodeSettings;   // user requested encode settings
  int16_t           Channels;         // number of requested output channels
  FRAME_HEADER_INFO FrameHdr;         //
  AC3_ENCODE        AC3;

  // stream control and output buffer
  BIT_STREAM_TYPE   BitStream;
  unsigned char     Frame[4096];      // output frame buffer
} AUDIO_ENCODE, *PAUDIO_ENCODE;

typedef struct {
  // decoder part
  AUDIO_DECODE  Decode;

  // transcode part
  AUDIO_TRANSCODE Transcode;

  // encoder part
  AUDIO_ENCODE  Encode;

  // pipe stream control
  PIPE_STREAM_TYPE		 InputStream;
} AUDIO_CONVERT_TYPE, *PAUDIO_CONVERT_TYPE;

extern void   DebugTrace( const char *str, ... );
extern int    m_RunAsLib;

extern int    LayerIIHdrCheck( PAUDIO_CONVERT_TYPE audioConvert );
extern int    LayerIIDecode( PAUDIO_CONVERT_TYPE audioConvert );
extern void   SynthFrame( PAUDIO_CONVERT_TYPE audioConvert );
extern double m_DbAdjustCoefTbl[(MAX_DB_RANGE * 2) + 1];

extern void timer_add( PTIME_T timer, TIME_T incr );
extern void timer_set( PTIME_T timer, unsigned long seconds, unsigned long numer, unsigned long denom );
extern void timer_string( TIME_T timer, char *dest, char const *format );

extern int AC3_encode_init( PAUDIO_CONVERT_TYPE audioConvert );
extern int AC3_encode_frame( PAUDIO_CONVERT_TYPE audioConvert );
extern int AC3_encode_close( PAUDIO_CONVERT_TYPE audioConvert );

// transcode.c
void      TranscodeOpen( PAUDIO_CONVERT_TYPE audioConvert );
void      TranscodeClose( PAUDIO_CONVERT_TYPE audioConvert );
uint32_t  TranscodeChangeRate( PAUDIO_CONVERT_TYPE audioConvert );
void     *TranscodeGetFullFrame( PAUDIO_CONVERT_TYPE audioConvert, int32_t requestedSamples, int16_t *channels );
void     *TranscodeGetParitalFrame( PAUDIO_CONVERT_TYPE audioConvert, int32_t *obtainedSamples, int16_t *channels );
void      TranscodeDoneWithCurrentFrame( PAUDIO_CONVERT_TYPE audioConvert, int32_t usedSamples );

// functions in audio_convert.c
// functions that can be called
// 0 = default of 44100, else 32000 or 44100 or 48000
// newBitRate == 0 user source bit rate, else a valid MPEG1 Layer II bitrate i.e. 192000, 256000, ...
// if newChannelMode == CHANNEL_MODE_DIM then use source channelmode
int AudioInitConverter( PAUDIO_CONVERT_TYPE audioConvert, PENCODE_SETTINGS encodeSettings, int dbAdjust );

// this function should be called when the audio converter is no longer needed
// or the program is shutting down, we will clean up our mess
void AudioShutDownConverter( PAUDIO_CONVERT_TYPE audioConvert );

// find a frame hdr
int AudioFindFrameHeader( PAUDIO_CONVERT_TYPE audioConvert, PFRAME_HEADER_INFO hdr );

// decode and transcode the frame
int AudioProcessFrame( PAUDIO_CONVERT_TYPE audioConvert );

// will return null when no more frame are avaliable
void *AudioGetOutputFrame( PAUDIO_CONVERT_TYPE audioConvert, int *frameLength );

// same as above but will return any leftover audio with padded silence
void *AudioGetLastFrame( PAUDIO_CONVERT_TYPE audioConvert, int *frameLength );

// function to encode silence, generates a silence frame based on the last frame that was processed,
// null if no other frames have been processed
void *AudioCreateSilenceFrame( PAUDIO_CONVERT_TYPE audioConvert, uint8_t *masterFrame, int *silenceFrameBytes );

// psycho_I.c
void      psycho_i( short buffer[2][1152], double scale[2][SBLIMIT], double ltmin[2][SBLIMIT], frame_params * fr_ps );

// psycho_II.c
void      psycho_ii( short int *buffer, short int savebuf[1056], int chn, FLOAT snr32[32], double sfreq );
void      psycho_ii_init( double sfreq );

// tables.c
int       pick_table( frame_params * fr_ps );

// FFT.c
void      fft( FLOAT * x_real, FLOAT * x_imag, FLOAT * energy, FLOAT * phi, int N );
void      fft2( FLOAT * x_real, FLOAT *energy, int N );

// Layer2Encoder.c
int       js_bound( int m_ext );
void      LayerIIEncodeInit( PAUDIO_CONVERT_TYPE audioConvert );
void      LayerIIEncodeFrame( PAUDIO_CONVERT_TYPE audioConvert );

// encode.c
void      encode_info( frame_params * fr_ps );
void      combine_LR( double sb_sample[2][3][SCALE_BLOCK][SBLIMIT], double joint_sample[3][SCALE_BLOCK][SBLIMIT], int sblimit );
void      scale_factor_calc( double sb_sample[][3][SCALE_BLOCK][SBLIMIT], unsigned int scalar[][3][SBLIMIT], int stereo, int sblimit );
void      pick_scale( unsigned int scalar[2][3][SBLIMIT], frame_params * fr_ps, double max_sc[2][SBLIMIT] );
void      transmission_pattern( unsigned int scalar[2][3][SBLIMIT], unsigned int scfsi[2][SBLIMIT], frame_params * fr_ps );
void      encode_scale( unsigned int bit_alloc[2][SBLIMIT], unsigned int scfsi[2][SBLIMIT],
                        unsigned int scalar[2][3][SBLIMIT], frame_params * fr_ps );
int       bits_for_nonoise (double perm_smr[2][SBLIMIT], unsigned int scfsi[2][SBLIMIT], frame_params * fr_ps);

void      main_bit_allocation( double perm_smr[2][SBLIMIT], unsigned int scfsi[2][SBLIMIT], unsigned int bit_alloc[2][SBLIMIT],
                                int *adb, frame_params * fr_ps );
void      maxmnr (double mnr[2][SBLIMIT], char used[2][SBLIMIT], int sblimit, int stereo, int *min_sb, int *min_ch);

int       a_bit_allocation( double perm_smr[2][SBLIMIT], unsigned int scfsi[2][SBLIMIT],
                            unsigned int bit_alloc[2][SBLIMIT], int *adb, frame_params * fr_ps );
void      subband_quantization( unsigned int scalar[2][3][SBLIMIT], double sb_samples[2][3][SCALE_BLOCK][SBLIMIT],
                                 unsigned int j_scale[3][SBLIMIT], double j_samps[3][SCALE_BLOCK][SBLIMIT],
                                 unsigned int bit_alloc[2][SBLIMIT], unsigned int sbband[2][3][SCALE_BLOCK][SBLIMIT],
                                 frame_params * fr_ps );
void      encode_bit_alloc( unsigned int bit_alloc[2][SBLIMIT], frame_params * fr_ps );
void      sample_encoding( unsigned int sbband[2][3][SCALE_BLOCK][SBLIMIT], unsigned int bit_alloc[2][SBLIMIT],
                            frame_params * fr_ps );


#endif
