/*
 * Copyright (C) 2002  Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _BIT_STREAM_H_
#define _BIT_STREAM_H_

typedef struct  {
  pipe_handle_t  PipeHandle;       // Handle to input pipe
  unsigned char *BufferPtr;        // bit stream buffer pointer
  unsigned long  BufferSize;       // size in bytes
  unsigned long  BufferHighWater;  // Buffer high water mark
  unsigned long  StreamPos;        // byte pos in stream
  int            StreamBitPos;     // bit pos in stream pos
  int            WriteBuffer;      // Read or write buffer flag
  int            BytCounter;
} PIPE_STREAM_TYPE, *PPIPE_STREAM_TYPE;

typedef struct  {
  unsigned char *StreamPtr;          // bit stream char pointer
  unsigned long  StreamSize;         // 0 = unlimited, else size in bytes
  unsigned long  StreamPos;          // byte pos in stream
  int            StreamBitPos;       // bit pos in stream pos
} BIT_STREAM_TYPE, *PBIT_STREAM_TYPE;

extern void           PipeInit( PPIPE_STREAM_TYPE stream, pipe_handle_t  pipehandle, int write, unsigned long bufferlen );
extern unsigned long  PipePeekBytes( PPIPE_STREAM_TYPE stream, unsigned char *buffer, unsigned long nbytes );
extern int            PipeCheckAvailBytes( PPIPE_STREAM_TYPE stream, unsigned long byts );
extern int            PipeSkipBytes( PPIPE_STREAM_TYPE stream, unsigned long byts );
extern unsigned long  PipeGetBits( PPIPE_STREAM_TYPE stream, int bits );
extern void           PipePutBits( PPIPE_STREAM_TYPE stream, int bits, unsigned long val );
extern void           PipeFlushWrite( PPIPE_STREAM_TYPE stream );

extern void           StreamInit( PBIT_STREAM_TYPE stream, void *memPtr, unsigned long len );
extern unsigned char *StreamGetBytePtr( PBIT_STREAM_TYPE stream );
extern int            StreamCheckAvailBytes( PBIT_STREAM_TYPE stream, int byts );
extern int            StreamSkipBytes( PBIT_STREAM_TYPE stream, int byts );
extern unsigned long  StreamGetBits( PBIT_STREAM_TYPE stream, int bits );
extern void           StreamPutBits( PBIT_STREAM_TYPE stream, int bits, unsigned long val );
extern void           StreamFlush( PBIT_STREAM_TYPE stream );
extern void           StreamPadBytes( PBIT_STREAM_TYPE stream, int byts );

#endif
