/*
 * toolame - an optimized mpeg 1/2 layer 2 audio encoder
 * Copyright (C) 2001 Michael Cheng
 *
 *  Modifyed by Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "audio_convert_structs.h"
#include "enwindow.h"


// global data
//RLM - needs to be moved into encode struct
frame_params        fr_ps;
short              *WinBuf[2];
double              SbSample[2][3][SCALE_BLOCK][SBLIMIT];
double              JSample[3][SCALE_BLOCK][SBLIMIT];
double              WinQue[2][HAN_SIZE];
unsigned int        SubBand[2][3][SCALE_BLOCK][SBLIMIT];
unsigned int        BitAlloc[2][SBLIMIT];
unsigned int        Scfsi[2][SBLIMIT];
unsigned int        Scalar[2][3][SBLIMIT];
unsigned int        JScale[3][SBLIMIT];
double              LtMin[2][SBLIMIT];
double              LgMin[2][SBLIMIT];
double              MaxSc[2][SBLIMIT];
FLOAT               Snr32[32];
short               Sam[2][1344];
unsigned int        m_Crc;
SLOTS               Slots;

// external data
extern const unsigned long  m_SampleRates[3];
extern const long           m_MpegBitRates[5][16];
extern const double         s_freq[2][4];
extern const int            bitrate[2][15];


/************************************************************************
*
* window_subband()
*
* PURPOSE:  Overlapping window on PCM samples
*
* SEMANTICS:
* 32 16-bit pcm samples are scaled to fractional 2's complement and
* concatenated to the end of the window buffer #x#. The updated window
* buffer #x# is then windowed by the analysis window #c# to produce the
* windowed sample #z#
*
************************************************************************/

void window_subband( short **buffer, double z[64], int k )
{
  typedef double XX[2][HAN_SIZE];
  static XX *x;
  double *xk;
  int i;
  static int off[2] = { 0, 0 };
  static char init = 0;
  double t;
  double *ep0, *ep1, *ep2, *ep3, *ep4, *ep5, *ep6, *ep7;
  if (!init)
    {
      x = (XX *) calloc( 1, sizeof( XX ) );
      memset (x, 0, 2 * HAN_SIZE * sizeof (double));
      init = 1;
    }
  xk = (*x)[k];

  /* replace 32 oldest samples with 32 new samples */
  for (i = 0; i < 32; i++)
    xk[31 - i + off[k]] = (double) *(*buffer)++ / SCALE;

  ep0 = &enwindow[0];
  ep1 = &enwindow[64];
  ep2 = &enwindow[128];
  ep3 = &enwindow[192];
  ep4 = &enwindow[256];
  ep5 = &enwindow[320];
  ep6 = &enwindow[384];
  ep7 = &enwindow[448];

  /* shift samples into proper window positions */
  for ( i = 0; i < 64; i++ ) {
      t = xk[(i + off[k]) & (512 - 1)] * *ep0++;
      t += xk[(i + 64 + off[k]) & (512 - 1)] * *ep1++;
      t += xk[(i + 128 + off[k]) & (512 - 1)] * *ep2++;
      t += xk[(i + 192 + off[k]) & (512 - 1)] * *ep3++;
      t += xk[(i + 256 + off[k]) & (512 - 1)] * *ep4++;
      t += xk[(i + 320 + off[k]) & (512 - 1)] * *ep5++;
      t += xk[(i + 384 + off[k]) & (512 - 1)] * *ep6++;
      t += xk[(i + 448 + off[k]) & (512 - 1)] * *ep7++;
      z[i] = t;
    }

  off[k] += 480;		/*offset is modulo (HAN_SIZE-1) */
  off[k] &= HAN_SIZE - 1;

}


/************************************************************************
*
* filter_subband()
*
* PURPOSE:  Calculates the analysis filter bank coefficients
*
* SEMANTICS:
*      The windowed samples #z# is filtered by the digital filter matrix #m#
* to produce the subband samples #s#. This done by first selectively
* picking out values from the windowed samples, and then multiplying
* them by the filter matrix, producing 32 subband samples.
*
************************************************************************/
void create_dct_matrix( double filter[16][32] )
{
  register int i, k;

  for ( i = 0; i < 16; i++ )
    for ( k = 0; k < 32; k++ ) {
	    if ( (filter[i][k] = 1e9 * cos ((double) ((2 * i + 1) * k * PI64))) >= 0 )
	      modf( filter[i][k] + 0.5, &filter[i][k] );
	    else
	      modf( filter[i][k] - 0.5, &filter[i][k] );
	    filter[i][k] *= 1e-9;
   }
}

// -----------------------------------------------------------------------
void filter_subband( double z[HAN_SIZE], double s[SBLIMIT] )
{
  double yprime[32];
  register int i, j;

  static double m[16][32];
  static int init = 0;

  if ( init == 0 ) {
    init++;
    create_dct_matrix( m );
  }

  yprime[0] = z[16];
  for (i = 1; i <= 16; i++)
    yprime[i] = z[i + 16] + z[16 - i];
  for (i = 17; i <= 31; i++)
    yprime[i] = z[i + 16] - z[80 - i];

  for ( i = 15; i >= 0; i-- ) {
    register double s0 = 0.0, s1 = 0.0;
    register double *mp = m[i];
    register double *xinp = yprime;
    for ( j = 0; j < 8; j++ ) {
	    s0 += *mp++ * *xinp++;
	    s1 += *mp++ * *xinp++;
	    s0 += *mp++ * *xinp++;
	    s1 += *mp++ * *xinp++;
	  }
    s[i] = s0 + s1;
    s[31 - i] = s0 - s1;
  }
}

// -----------------------------------------------------------------------
void update_CRC( unsigned int data, unsigned int length, unsigned int *crc )
{
  unsigned int masking, carry;

  masking = 1 << length;

  while ( (masking >>= 1) ) {
    carry = *crc & 0x8000;
    *crc <<= 1;
    if ( !carry ^ !(data & masking) )
	    *crc ^= CRC16_POLYNOMIAL;
  }
  *crc &= 0xffff;
}

// -----------------------------------------------------------------------
void CRC_calc( frame_params * fr_ps, unsigned int bit_alloc[2][SBLIMIT], unsigned int scfsi[2][SBLIMIT], unsigned int *crc, PAUDIO_CONVERT_TYPE audioConvert )
{
  int i, k;
  int stereo = fr_ps->stereo;
  int sblimit = fr_ps->sblimit;
  int jsbound = fr_ps->jsbound;
  al_table *alloc = &fr_ps->alloc;

  *crc = 0xffff;		/* changed from '0' 92-08-11 shn */
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.BitrateIndex, 4, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.SampleRateIndex, 2, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.Padding, 1, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.Private, 1, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.ChannelMode, 2, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.ModeExtension, 2, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.Copyright, 1, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.Original, 1, crc );
  update_CRC( audioConvert->Encode.FrameHdr.u.mpeg.Emphasis, 2, crc );

  for ( i = 0; i < sblimit; i++ )
    for ( k = 0; k < ((i < jsbound) ? stereo : 1); k++ )
      update_CRC( bit_alloc[k][i], (*alloc)[i][0].bits, crc );

  for ( i = 0; i < sblimit; i++ )
    for ( k = 0; k < stereo; k++ )
      if ( bit_alloc[k][i] )
	update_CRC( scfsi[k][i], 2, crc );
}

// -----------------------------------------------------------------------
int js_bound( int m_ext )
{
  /* layer 2 only */
  static int jsb_table[4] = { 4, 8, 12, 16 };

  if ( m_ext < 0 || m_ext > 3 ) {
      LOG_TY_FATAL1 ( "js_bound bad modext (%d)\n", m_ext);
  }
  return( jsb_table[m_ext] );
}

// -----------------------------------------------------------------------
void HdrToFrps( frame_params *fr_ps, PAUDIO_CONVERT_TYPE audioConvert )
{
  fr_ps->actual_mode = audioConvert->Encode.FrameHdr.u.mpeg.ChannelMode;
  fr_ps->stereo = (audioConvert->Encode.FrameHdr.u.mpeg.ChannelMode == CHANNEL_MODE_SINGLE_MONO) ? 1 : 2;

  fr_ps->sblimit = pick_table( fr_ps );

  if ( audioConvert->Encode.FrameHdr.u.mpeg.ChannelMode == CHANNEL_MODE_JOINT_STEREO ) {
    fr_ps->jsbound = js_bound( audioConvert->Encode.FrameHdr.u.mpeg.ModeExtension );
  }
  else {
    fr_ps->jsbound = fr_ps->sblimit;
  }
  /* alloc, tab_num set in pick_table */
}

// -----------------------------------------------------------------------
int AvailableBits( PAUDIO_CONVERT_TYPE audioConvert )
{
  int           adb;

  Slots.Extra = 0;    /* be default, no extra slots */

  Slots.Average = (1152.0 / s_freq[audioConvert->Encode.FrameHdr.u.mpeg.AudioVer & 1][audioConvert->Encode.FrameHdr.u.mpeg.SampleRateIndex]) *
    ((double) bitrate[audioConvert->Encode.FrameHdr.u.mpeg.AudioVer & 1][audioConvert->Encode.FrameHdr.u.mpeg.BitrateIndex] / 8.0);

  Slots.Whole = (int) Slots.Average;
  Slots.Frac = Slots.Average - ((double)Slots.Whole);
  if ( Slots.Frac != 0 ) {

    // no padding for this frame
    if ( Slots.Lag > (Slots.Frac - 1.0) ) {
      Slots.Lag -= Slots.Frac;
      Slots.Extra = 0;
      audioConvert->Encode.FrameHdr.u.mpeg.Padding = 0;
    }
    else {
      /* padding */
      Slots.Extra = 1;
      audioConvert->Encode.FrameHdr.u.mpeg.Padding = 1;
      Slots.Lag += (1 - Slots.Frac);
    }
  }
  adb = (Slots.Whole + Slots.Extra) * 8;
  return adb;
}

// -----------------------------------------------------------------------
void LayerIIEncodeInit( PAUDIO_CONVERT_TYPE audioConvert )
{
  int           i;
  PAUDIO_ENCODE encode = &audioConvert->Encode;

  fr_ps.header            = &encode->FrameHdr;
  fr_ps.tab_num           = -1;   /* no table loaded */
  fr_ps.stereo            = encode->Channels; //(control->Decode.Frame.ChannelMode != CHANNEL_MODE_SINGLE_MONO) ? 2 : 1;
  encode->FrameHdr.FrameSync              = MPEG_FRAME_SYNC;
  encode->FrameHdr.u.mpeg.AudioVer        = MPEG_1_VER;
  encode->FrameHdr.u.mpeg.LayerDescrition = MPEG_LAYER2;
  for ( i = 1; i < 15; i++ ) {
    if ( m_MpegBitRates[1][i] == (long)encode->FrameHdr.u.mpeg.BitRate ) {
      encode->FrameHdr.u.mpeg.BitrateIndex = i;
      break;
    }
  }
  if ( i >= 15 ) {
    LOG_TY_ERROR1( "Invalid MPEG1 Layer II BitRate = %d\n", encode->FrameHdr.u.mpeg.BitRate );
    return;
  }
  for ( i = 0; i < 3; i++ ) {
    if ( m_SampleRates[i] == encode->FrameHdr.u.mpeg.SampleRate ) {
      encode->FrameHdr.u.mpeg.SampleRateIndex = i;
      break;
    }
  }
  if ( i >= 3 ) {
    LOG_TY_ERROR1( "Invalid MPEG1 Layer II SampleRate = %d\n", encode->FrameHdr.u.mpeg.SampleRate );
    return;
  }
  encode->FrameHdr.u.mpeg.Protection   = 1;
  switch ( encode->FrameHdr.u.mpeg.ChannelMode ) {
    case CHANNEL_MODE_JOINT_STEREO:
      encode->FrameHdr.u.mpeg.ChannelMode     = CHANNEL_MODE_JOINT_STEREO;
      encode->FrameHdr.u.mpeg.ModeExtension   = 2;
      break;

    default:
    case CHANNEL_MODE_STEREO:
      encode->FrameHdr.u.mpeg.ChannelMode     = CHANNEL_MODE_STEREO;
      encode->FrameHdr.u.mpeg.ModeExtension   = 0;
      break;
  }

  HdrToFrps( &fr_ps, audioConvert );
}

// -----------------------------------------------------------------------
void LayerIIEncodeFrame( PAUDIO_CONVERT_TYPE audioConvert )
{
  int adb;
  int i;
  int j;
  int k;
  static FLOAT snrdef[32] = {
    30, 17, 16, 10, 3, 12, 8, 2.5, 5, 5, 6, 6, 5, 6, 10, 6, -4, -10, -21,
    -30, -42, -55, -68, -75, -75, -75, -75, -75, -91, -107, -110, -108
  };

  // setup output bit stream
  StreamInit( &audioConvert->Encode.BitStream, audioConvert->Encode.Frame, sizeof( audioConvert->Encode.Frame ) );

  // setup pointers to each channel
  WinBuf[0] = &audioConvert->Transcode.TranscodedOutput[0][0];
  WinBuf[1] = &audioConvert->Transcode.TranscodedOutput[1][0];

  adb = AvailableBits( audioConvert );

  for ( i = 0; i < 3; i++ ) {
    for ( j = 0; j < SCALE_BLOCK; j++ ) {
      for ( k = 0; k < fr_ps.stereo; k++ ) {
        window_subband( &WinBuf[k], &WinQue[k][0], k );
        filter_subband( &WinQue[k][0], &SbSample[k][i][j][0] );
      }
    }
  }

  scale_factor_calc( SbSample, Scalar, fr_ps.stereo, fr_ps.sblimit );
  pick_scale( Scalar, &fr_ps, MaxSc );
  if ( fr_ps.actual_mode == CHANNEL_MODE_JOINT_STEREO ) {
    combine_LR( SbSample, JSample, fr_ps.sblimit );
    scale_factor_calc( &JSample, &JScale, 1, fr_ps.sblimit );
  }

  switch ( audioConvert->Encode.EncodeSettings.u.mpeg.PsychoModel ) {
    default:
    case 0:
      for ( k = 0; k < fr_ps.stereo; k++ ) {
        for ( i = 0; i < SBLIMIT; i++ ) {
          LtMin[k][i] = snrdef[i];
        }
      }
      break;

    case 1:
      psycho_i( (short (*)[1152])audioConvert->Transcode.TranscodedOutput, MaxSc, LtMin, &fr_ps );
      break;

    case 2:
      for ( k = 0; k < fr_ps.stereo; k++ ) {
        psycho_ii( &audioConvert->Transcode.TranscodedOutput[k][0], &Sam[k][0], k, Snr32,
                   (FLOAT) s_freq[audioConvert->Encode.FrameHdr.u.mpeg.AudioVer & 1][audioConvert->Encode.FrameHdr.u.mpeg.SampleRateIndex] * 1000 );
        for ( i = 0; i < SBLIMIT; i++ ) {
          LtMin[k][i] = (double)Snr32[i];
        }
      }
      break;
  }

  transmission_pattern( Scalar, Scfsi, &fr_ps );
  main_bit_allocation( LtMin, Scfsi, BitAlloc, &adb, &fr_ps );
  if ( !audioConvert->Encode.FrameHdr.u.mpeg.Protection ) {
    CRC_calc( &fr_ps, BitAlloc, Scfsi, &m_Crc, audioConvert );
  }

  encode_info( &fr_ps );

  encode_bit_alloc( BitAlloc, &fr_ps );
  encode_scale( BitAlloc, Scfsi, Scalar, &fr_ps );
  subband_quantization( Scalar, SbSample, JScale, JSample, BitAlloc, SubBand, &fr_ps );
  sample_encoding( SubBand, BitAlloc, &fr_ps );
  for ( i = 0; i < adb; i++ ) {
    StreamPutBits( &audioConvert->Encode.BitStream, 1, 0 );
  }
}
