/*
 * toolame - an optimized mpeg 1/2 layer 2 audio encoder
 * Copyright (C) 2001 Michael Cheng
 *
 *  Modifyed by Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LAYERII_ENCODER_H
#define LAYERII_ENCODER_H

#define         SCALE_BLOCK             12
#define         SBLIMIT                 32
#define         HAN_SIZE                512
#define         FLOAT                   float
#define         INLINE
#define         FALSE                   0
#define         TRUE                    1

#ifndef PI
#define         PI                      3.14159265358979
#endif
#define         PI64                    PI/64
#define         LN_TO_LOG10             0.2302585093
#define         SCALE                   32768
#define         CRC16_POLYNOMIAL        0x8005
#define         NOISY_MIN_MNR           0.0
#define         FFT_SIZE                1024

/* Psychacoustic Model 1 Definitions */

#define CB_FRACTION     0.33
#define MAX_SNR         1000
#define NOISE           10
#define TONE            20
#define DBMIN           -200.0
#define LAST            -1
#define STOP            -100
#define POWERNORM       90.3090 /* = 20 * log10(32768) to normalize */
/* max output power to 96 dB per spec */

#define DBTAB 1000

/* Psychoacoustic Model 2 Definitions */
#define LOGBLKSIZE      10
#define BLKSIZE         1024
#define HBLKSIZE        513
#define CBANDS          63
#define LXMIN           32.0


typedef double  D1408[1408];
typedef double  DFFT[FFT_SIZE];
typedef int     ICB[CBANDS];
typedef int     IHBLK[HBLKSIZE];
typedef FLOAT   F32[32];
typedef FLOAT   F2_32[2][32];
typedef FLOAT   FCB[CBANDS];
typedef FLOAT   FCBCB[CBANDS][CBANDS];
typedef FLOAT   FBLK[BLKSIZE];
typedef FLOAT   FHBLK[HBLKSIZE];
typedef FLOAT   F2HBLK[2][HBLKSIZE];
typedef FLOAT   F22HBLK[2][2][HBLKSIZE];
typedef double  P2DCB[CBANDS];

typedef struct {
  int line;
  double bark, hear, x;
} g_thres, *g_ptr;

typedef struct {
  double x;
  int type, next, map;
} mask, *mask_ptr;

typedef struct {
  double Average;
  double Frac;
  int    Whole;
  double Lag;
  int    Extra;
} SLOTS;

typedef struct {
  unsigned int steps;
  unsigned int bits;
  unsigned int group;
  unsigned int quant;
} sb_alloc, *alloc_ptr;
typedef sb_alloc al_table[SBLIMIT][16];

typedef struct {
  FRAME_HEADER_INFO  *header;       /* raw header information */
  int                 actual_mode;  /* when writing IS, may forget if 0 chs */
  al_table            alloc;        /* bit allocation table read in */
  int                 tab_num;      /* number of table as loaded */
  int                 stereo;       /* 1 for mono, 2 for stereo */
  int                 jsbound;      /* first band of joint stereo coding */
  int                 sblimit;      /* total number of sub bands */
} frame_params;


#endif
