/*
 * Copyright (C) 2002  Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef WIN32
#include <stdio.h>
#include <windows.h>
#endif

#include "audio_convert_structs.h"
#include "../libs/about.c"
#define TYTRANSCODE_VERSION "0.5.0"

/* All these are in tytranscode.c */
extern int ProcessArgs( int argc, char *argv[], int runaslib );
extern void ParseFile();
extern pipe_handle_t   m_hPipe_In;
extern pipe_handle_t   m_hPipe_Out;
extern char           *m_InputFn;
extern char           *m_OutputFn;
extern int             m_DumpAllHdrs;

// ---------------------------------------------------------------------
int main( int argc, char* argv[] )
{

//  logger_init( 2, stderr, 0 );
  logger_init( log_info, stdout, 0, 0 );

  // process args
  if ( ProcessArgs( argc, argv, 0 ) ) {
	  printf("tytranscode "TYTRANSCODE_VERSION" %s", about);
    return( 1 );
  }

  // Open a pipe on the input file
  m_hPipe_In = thread_pipe_create_from_file( m_InputFn );
  if( !m_hPipe_In ) {
    LOG_TY_FATAL("\nERROR: Unable to open audio file\n" );
  }

  if ( !m_DumpAllHdrs ) {
	// create output file
	m_hPipe_Out = thread_pipe_create_to_file( m_OutputFn, 1 );
	if( !m_hPipe_Out ) {
	  LOG_TY_FATAL1( "\nERROR: Unable to create output file '%s'\n", m_OutputFn );
	}
  }

  // parse file
  ParseFile();

  // close pipes
  if ( m_hPipe_In ) {
    thread_pipe_free( m_hPipe_In );
  }
  if ( m_hPipe_Out ) {
    thread_pipe_free( m_hPipe_Out );
  }

  logger_free();

  return 0;
}
