/*
 * mad - MPEG audio decoder
 * Copyright (C) 2000-2001 Robert Leslie
 *
 *  Modifyed by Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "audio_convert_structs.h"

TIME_T  timer_zero = { 0, 0 };

// ---------------------------------------------------------------------
int timer_compare( TIME_T timer1, TIME_T timer2)
{
  signed long diff;

  diff = timer1.Seconds - timer2.Seconds;
  if (diff < 0)
    return -1;
  else if (diff > 0)
    return +1;

  diff = timer1.Fraction - timer2.Fraction;
  if (diff < 0)
    return -1;
  else if (diff > 0)
    return +1;

  return 0;
}

// ---------------------------------------------------------------------
static unsigned long gcd( unsigned long num1, unsigned long num2 )
{
  unsigned long tmp;

  while ( num2 ) {
    tmp  = num2;
    num2 = num1 % num2;
    num1 = tmp;
  }
  return num1;
}

// ---------------------------------------------------------------------
static void reduce_rational( unsigned long *numer, unsigned long *denom )
{
  unsigned long factor;

  factor = gcd( *numer, *denom );

  if ( !factor ) {
    // error
    return;
  }

  *numer /= factor;
  *denom /= factor;
}

// ---------------------------------------------------------------------
static unsigned long scale_rational( unsigned long numer, unsigned long denom, unsigned long scale )
{
  reduce_rational( &numer, &denom );
  reduce_rational( &scale, &denom );

  if ( !denom ) {
    // error
    return 0;
  }

  if ( denom < scale )
    return numer * (scale / denom) + numer * (scale % denom) / denom;
  if ( denom < numer )
    return scale * (numer / denom) + scale * (numer % denom) / denom;

  return numer * scale / denom;
}

// ---------------------------------------------------------------------
void timer_negate( PTIME_T timer)
{
  timer->Seconds = -timer->Seconds;

  if ( timer->Fraction ) {
    timer->Seconds -= 1;
    timer->Fraction = MAD_TIMER_RESOLUTION - timer->Fraction;
  }
}

// ---------------------------------------------------------------------
TIME_T timer_abs( TIME_T timer)
{
  if ( timer_sign( timer ) < 0 )
    timer_negate( &timer );

  return timer;
}

// ---------------------------------------------------------------------
void timer_string( TIME_T timer, char *dest, char const *format )
{
  unsigned long hours, minutes, seconds, sub;
  unsigned int  frac;
  unsigned long denom;

  timer = timer_abs( timer );

  seconds = timer.Seconds;
  frac = sub = 0;


  denom = MAD_TIMER_RESOLUTION / 10;

  frac = timer.Fraction / denom;
  sub  = scale_rational( timer.Fraction % denom, denom, 0 );

  minutes = seconds / 60;
  hours   = minutes / 60;

  sprintf( dest, format, hours,
    (unsigned int) (minutes % 60),
    (unsigned int) (seconds % 60),
    frac, sub );
}

// ---------------------------------------------------------------------
static void reduce_timer( PTIME_T timer)
{
  timer->Seconds  += timer->Fraction / MAD_TIMER_RESOLUTION;
  timer->Fraction %= MAD_TIMER_RESOLUTION;
}

// ---------------------------------------------------------------------
void timer_set( PTIME_T timer, unsigned long seconds, unsigned long numer, unsigned long denom )
{
  timer->Seconds = seconds;
  if (numer >= denom && denom > 0) {
    timer->Seconds += numer / denom;
    numer %= denom;
  }

  switch (denom) {
  default:
  case 0:
  case 1:
    timer->Fraction = 0;
    break;

  case MAD_TIMER_RESOLUTION:
    timer->Fraction = numer;
    break;

  case 8000:
    timer->Fraction = numer * (MAD_TIMER_RESOLUTION /  8000);
    break;

  case 16000:
    timer->Fraction = numer * (MAD_TIMER_RESOLUTION / 16000);
    break;

  case 24000:
    timer->Fraction = numer * (MAD_TIMER_RESOLUTION / 24000);
    break;

  case 32000:
    timer->Fraction = numer * (MAD_TIMER_RESOLUTION / 32000);
    break;

  case 44100:
    timer->Fraction = numer * (MAD_TIMER_RESOLUTION / 44100);
    break;

  case 48000:
    timer->Fraction = numer * (MAD_TIMER_RESOLUTION / 48000);
    break;
  }

  if ( timer->Fraction >= MAD_TIMER_RESOLUTION )
    reduce_timer( timer );
}

// ---------------------------------------------------------------------
void timer_add( PTIME_T timer, TIME_T incr )
{
  timer->Seconds  += incr.Seconds;
  timer->Fraction += incr.Fraction;

  if ( timer->Fraction >= MAD_TIMER_RESOLUTION )
    reduce_timer( timer );
}

