/*
 * Copyright (C) 2002  Rowan McFarland <tytranscode@hotmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DECIMATION_H
#define DECIMATION_H

#define MAX_CHANNELS      6
#define MAX_RATE_CONVERT  3     // number of different type of conversions we can handle at the same time
#define MAX_QUEUE_SIZE    (MAX_FRAME_SIZE * 4)
#define MY_BUFSIZE        50000   //RLM - ? what should this really be

typedef struct {
   double Factor;       // Factor = Fout/Fin sample rates
   double rolloff;      // roll-off frequency
   double beta;         // passband/stopband tuning magic
   int quadr;           // non-zero to use qprodUD quadratic interpolation
   int32_t Nmult;
   int32_t Nwing;
   int32_t Nq;
   double *Imp;         // impulse [Nwing+1] Filter coefficients

   double Time;         // Current time/pos in input sample
   int32_t dhb;

   int32_t a,b;         // gcd-reduced input,output rates
   int32_t t;           // Current time/pos for exact-coeff's method

   int32_t Xh;          // number of past/future samples needed by filter
   int32_t Xoff;        // Xh plus some room for creep
   int32_t Xread;       // X[Xread] is start-position to enter new samples
   int32_t Xp;          // X[Xp] is position to start filter application
   int32_t Xsize,Ysize; // size (Floats) of X[],Y[]
   double *X, *Y;       // I/O buffers
} resample_t, *presample_t;

typedef struct {
  int32_t      inputrate;           // input sampling rate
  int32_t      outputrate;          // output sampling rate
  int32_t      obuf[MY_BUFSIZE/2];  // output buffer
  resample_t   priv;                // private area for effect
} rate_filter_t, *prate_filter_t;

typedef struct {
  uint32_t       SrcRate;              // source rate
  uint32_t       DstRate;              // destination rate
  rate_filter_t  Filter[MAX_CHANNELS]; // channels to support mono/stereo rate changes
} rateconvert_t, *prateconvert_t;

typedef struct {
    uint32_t  Size;                 // current size of queue
    uint32_t  Head;                 // head pointer
    uint32_t  Tail;                 // tail pointer
    int16_t   Item[MAX_QUEUE_SIZE]; // pointer to queue items
} QUEUE_TYPE, *PQUEUE_TYPE;

typedef struct {
  int16_t     Samples[MAX_CHANNELS][MAX_FRAME_SIZE];  // output buffer
  QUEUE_TYPE  SampleQueue[MAX_CHANNELS];              // queue of samples, one for each channel
} TRANSCODED_SAMPLES, *PTRANSCODED_SAMPLES;

typedef struct _CHANNEL_MAP {
  struct _CHANNEL_MAP  *Next;
  int32_t               Samples;                               // number of input channels
  int16_t               Channels;                              // channels that are contained
} CHANNEL_MAP, *PCHANNEL_MAP;


#endif
