#include "mpeg2Int.h"
#include "tables/mb_addr_increment.h"
#include "tables/motion_code.h"
#include "tables/dct_dc_size_table.h"
#include "tables/block_pattern_table.h"
#include "tables/mb_mode_table.h"
#include "tables/dct_coeff_table_v2.h"

/*
 * 4096 element crop table.
 */

extern unsigned char theCropTable[4096];
//#define CROP(n) (unsigned char)theCropTable[(n) + 2048]
unsigned char CROP(int n) {
	if(n > 2047) {
		n = 2046;
	} else if (n < -2048) {
		n = -2048;
	}
	return((unsigned char)theCropTable[(n) + 2048]);
}



/* 
 * Interface to jrevdct.c routines (from JPEG decoder)
 */

#define DCTELEM short
#define DCTSIZE 8
#define DCTSIZE2 64

typedef DCTELEM DCTBLOCK[DCTSIZE2];
void DctToByte(DCTBLOCK, unsigned char *, int);
void DctToByteSparse(DCTBLOCK, unsigned char *, int);
void j_rev_dct(DCTBLOCK);
void j_rev_dct_sparse(DCTBLOCK, int);

/*  Function prototype declarations. */

static int Mpeg2DecodeMBAddrIncrement(BitParser *bp, int *off);
static int Mpeg2DecodeINonScalableMBMode(BitParser *bp, Mpeg2MBInfo *mbi, int concealment_flag, int pic_structure_code, int frame_pred_frame_flag);
static int Mpeg2DecodePNonScalableMBMode(BitParser *bp, Mpeg2MBInfo *mbi, int concealment_flag, int pic_structure_code, int frame_pred_frame_flag);
static int Mpeg2DecodeMotionVectors(Mpeg2PicHdr *fh, BitParser *bp, int back_flag, Mpeg2MBInfo *mbi);
static int Mpeg2DecodeMVector(Mpeg2PicHdr *fh, BitParser *bp, int vector_select, int back_flag, int mv_format, int dmv_flag, Mpeg2MBInfo *mbi);
static int Mpeg2DecodeMotionCode(BitParser *bp, int *off);
static int Mpeg2DecodeIntraBlock(BitParser *bp,  int block_num,  ScBlock *dest,  int lum_flag,  short *dc_pred,  int dct_coeff_table_flag, char *unzigzag, int dc_quant_scale, unsigned char *qmatrix, int qscale);
static int Mpeg2DecodeBlockPattern(BitParser *bp, int *off, int chroma_format_code);
static int Mpeg2DecodeNonIntraBlock(BitParser *bp, int block_num, ScBlock *dest, char *unzigzag, unsigned char *qmatrix, int qscale);

static void DeterminePredictionSources(short h, short v, int *np, unsigned char **s1, unsigned char **s2, unsigned char **s3, unsigned char **s4, unsigned char *src, int src_w);
static void Block_Avg8_16_X_8(unsigned char *s1, int s1w, unsigned char *s2, int s2w, unsigned char *s3, int s3w, unsigned char *s4, int s4w, unsigned char *s5, int s5w, unsigned char *s6, int s6w, unsigned char *s7, int s7w, unsigned char *s8, int s8w, unsigned char *d, int dw);
static void Block_Avg6_16_X_8(unsigned char *s1, int s1w, unsigned char *s2, int s2w, unsigned char *s3, int s3w, unsigned char *s4, int s4w, unsigned char *s5, int s5w, unsigned char *s6, int s6w, unsigned char *d, int dw);
static void Block_Avg5_16_X_8(unsigned char *s1, int s1w, unsigned char *s2, int s2w, unsigned char *s3, int s3w, unsigned char *s4, int s4w, unsigned char *s5, int s5w, unsigned char *d, int dw);
static void Block_Avg4_16_X_8(unsigned char *s1, int s1w, unsigned char *s2, int s2w, unsigned char *s3, int s3w, unsigned char *s4, int s4w, unsigned char *d, int dw);
static void Block_Avg3_16_X_8(unsigned char *s1, int s1w, unsigned char *s2, int s2w, unsigned char *s3, int s3w, unsigned char *d, int dw);
static void Block_Avg2_16_X_8(unsigned char *s1, int s1w, unsigned char *s2, int s2w, unsigned char *d, int dw);
static void Block_Copy_16_X_8(unsigned char *s, int sw, unsigned char *d, int dw);
static void BlockShortToByteCopy(short *temp, unsigned char *dest, int dest_w);
static void BlockShortToByteAdd(short *temp, unsigned char *dest, int dest_w);

static int Mpeg2DecodeBNonScalableMBMode(BitParser *bp, Mpeg2MBInfo *mbi, int concealment_flag, int pic_structure_code, int frame_pred_frame_flag);

static void Do_16_X_16_Prediction(int num_preds, unsigned char *s1, int s1_w, unsigned char *s2, int s2_w, unsigned char *s3, int s3_w, unsigned char *s4, int s4_w, unsigned char *s5, int s5_w, unsigned char *s6, int s6_w, unsigned char *s7, int s7_w, unsigned char *s8, int s8_w, unsigned char *dest, int dest_w);
static void Do_16_X_8_Prediction(int num_preds, unsigned char *s1, int s1_w, unsigned char *s2, int s2_w, unsigned char *s3, int s3_w, unsigned char *s4, int s4_w, unsigned char *s5, int s5_w, unsigned char *s6, int s6_w, unsigned char *s7, int s7_w, unsigned char *s8, int s8_w, unsigned char *dest, int dest_w);
static void Do_8_X_16_Prediction(int num_preds, unsigned char *s1, int s1_w, unsigned char *s2, int s2_w, unsigned char *s3, int s3_w, unsigned char *s4, int s4_w, unsigned char *s5, int s5_w, unsigned char *s6, int s6_w, unsigned char *s7, int s7_w, unsigned char *s8, int s8_w, unsigned char *dest, int dest_w);
static void Do_8_X_8_Prediction(int num_preds, unsigned char *s1, int s1_w, unsigned char *s2, int s2_w, unsigned char *s3, int s3_w, unsigned char *s4, int s4_w, unsigned char *s5, int s5_w, unsigned char *s6, int s6_w, unsigned char *s7, int s7_w, unsigned char *s8, int s8_w, unsigned char *dest, int dest_w);
static void Do_8_X_4_Prediction(int num_preds, unsigned char *s1, int s1_w, unsigned char *s2, int s2_w, unsigned char *s3, int s3_w, unsigned char *s4, int s4_w, unsigned char *s5, int s5_w, unsigned char *s6, int s6_w, unsigned char *s7, int s7_w, unsigned char *s8, int s8_w, unsigned char *dest, int dest_w);

/*
 * Zig-zag encoding patterns. Indexed by zig-zagged index
 * value, returns row-major 8x8 array index.
 */

static unsigned char unzigzag_pattern[64] = {
  0,  1,  8,  16,  9,  2,  3, 10, 
  17, 24, 32, 25, 18, 11,  4, 5, 
  12, 19, 26, 33, 40, 48, 41, 34,
  27, 20, 13, 6,  7,  14, 21, 28,
  35, 42, 49, 56, 57, 50, 43, 36,
  29, 22, 15, 23, 30, 37, 44, 51,
  58, 59, 52, 45, 38, 31, 39, 46,
  53, 60, 61, 54, 47, 55, 62, 63};

static unsigned char alt_unzigzag_pattern[64] = {
  0,  8,  16, 24,  1,  9,  2, 10,
  17, 25, 32, 40, 48, 56, 57, 49,
  41, 33, 26, 18,  3, 11,  4, 12,
  19, 27, 34, 42, 50, 58, 35, 43,
  51, 59, 20, 28, 5,  13,  6, 14,
  21, 29, 36, 44, 52, 60, 37, 45, 
  53, 61, 22, 30, 7,  15, 23, 31,
  38, 46, 54, 62, 39, 47, 55, 63};

/* Non-linear quantization scale table */

static int nonlinear_q_scale_table[32] = {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 16, 18, 20, 22, 24,
  28, 32, 36, 40, 44, 48, 52, 56, 64, 72, 80, 88, 96, 104, 112};

/* Macro for copying run-length encoded DCT coefficients from
 * an ScBlock structure into an 8x8 array of shorts and applying
 * the IDCT.
 */

#ifdef NO_MMX
#define DO_IDCT(block, temp)                                      \
{                                                                 \
  int i;                                                          \
  memset(temp, 0, 64*sizeof(short));                              \
  temp[0] = (block->dc);                                          \
  for (i=0; i<block->numOfAC; i++) {                              \
    temp[(int)block->index[i]] = block->value[i];                 \
  }                                                               \
  Fast_IDCT(temp);                                                \
}
#else
#define DO_IDCT(block, temp)                                      \
{                                                                 \
  int i, tidx;                                                    \
  memset(temp, 0, 64*sizeof(short));                              \
  temp[0] = (block->dc)<<4;                                       \
  for (i=0; i<block->numOfAC; i++) {                              \
    tidx = ((((int)block->index[i])%8)*8)+(((int)block->index[i])/8);\
    temp[tidx] = ((short) block->value[i])<<4;                    \
  }                                                               \
  Fast_IDCT(temp);                                                \
}
#endif

/* 
 * Debugging routing for printing DCT coefficients of a macroblock
 */

static void PrintMacroBlock(ScImage *y, ScImage *u, ScImage *v, 
			    int mb_addr, int mb_row, int mb_col) {
  ScBlock *b;
  int r, c, i, n;
  short temp_b[64];
  printf("Macroblock: %d", mb_addr);
  for (r = 0; r < 2; r++) {
    for (c = 0; c < 2; c++) {
      printf("   Block %d:\n", r*2+c);
      b = &(y->firstBlock[y->parentWidth*(mb_row*2+r)+(mb_col*2+c)]);
      temp_b[0] = b->dc - 1024;
      for (i=1; i<64; i++) {
	temp_b[i] = 0;
      }
      for (n=0; n<b->numOfAC; n++) {
	temp_b[(int)b->index[n]] = b->value[n];
      }
      for (i=0; i<64; i += 8) {
	printf("    %+5d %+5d %+5d %+5d %+5d %+5d %+5d %+5d\n", 
	       temp_b[i], temp_b[i+1], temp_b[i+2], temp_b[i+3], 
	       temp_b[i+4], temp_b[i+5], temp_b[i+6], temp_b[i+7]);
      }
    }
  }
  printf("   Block 4:\n");
  b = &(u->firstBlock[u->parentWidth*mb_row+mb_col]);
  temp_b[0] = b->dc - 1024;
  for (i=1; i<64; i++) {
    temp_b[i] = 0;
  }
  for (n=0; n<b->numOfAC; n++) {
    temp_b[(int)b->index[n]] = b->value[n];
  }
  for (i=0; i<64; i += 8) {
    printf("    %+5d %+5d %+5d %+5d %+5d %+5d %+5d %+5d\n", 
	   temp_b[i], temp_b[i+1], temp_b[i+2], temp_b[i+3], 
	   temp_b[i+4], temp_b[i+5], temp_b[i+6], temp_b[i+7]);
  }
  printf("   Block 5:\n");
  b = &(v->firstBlock[v->parentWidth*mb_row+mb_col]);
  temp_b[0] = b->dc - 1024;
  for (i=1; i<64; i++) {
    temp_b[i] = 0;
  }
  for (n=0; n<b->numOfAC; n++) {
    temp_b[(int)b->index[n]] = b->value[n];
  }
  for (i=0; i<64; i += 8) {
    printf("    %+5d %+5d %+5d %+5d %+5d %+5d %+5d %+5d\n", 
	   temp_b[i], temp_b[i+1], temp_b[i+2], temp_b[i+3], 
		 temp_b[i+4], temp_b[i+5], temp_b[i+6], temp_b[i+7]);
  }
  printf("\n");
}


/*-----------------------------------------
 * Mpeg2ParseI
 *
 * Parses an I field or frame.
 *
 * Parameters:
 *   BitParser *bp     -- BitParser to parse from
 *   Mpeg2SeqHdr *sh   -- sequence header info
 *   Mpeg2PicHdr *fh   -- picture header info
 *   ScImage *y        -- Destination for Y DCT coefficients
 *   SCImage *cr       -- Destination for Cr DCT coefficients
 *   SCImage *cb       -- Destination for Cb DCT coefficients
 *   Mpeg2MBInfoArray *mbi_array -- Destination for macroblock info
 *   int field_flag    -- 1 indicates that I-field is being parsed,
 *                        0 indicates taht I-frame is being parsed.
 *
 * Effects:
 *   Bitstream is parsed. 
 *
 * Return value:
 *   Number of bytes parsed.
 *
 * Notes:
 *   The dimensions of y, cr, cb, and mbi_array must appropriately
 *   match dimensions of encoded video. This means that if a field 
 *   is being parsed, each must be 1/2 as tall as full frame
 *   dimensions. No validity check is made for whether bit stream 
 *   is really at an encoded I field or frame.
 *-----------------------------------------*/

int
Mpeg2ParseI(BitParser *bp, 
	    Mpeg2SeqHdr *sh, 
	    Mpeg2PicHdr *fh,
	    ScImage *y,
	    ScImage *u,
	    ScImage *v,
	    Mpeg2MBInfoArray *mbi_array,
	    int field_flag)
{
  unsigned int data;
  int total = 0;
  int off;
  unsigned char *unzigzag;
  int block_count;
  unsigned char *lum_q_matrix, *chr_q_matrix;
  int q_scale_type;
  int dc_quant_scale;
  int qscale;
  int dct_coeff_table_flag;
  unsigned int byte_start;
  unsigned int byte_end;
  unsigned int bit_count_start;
  unsigned int bit_count_end;

  byte_start = (unsigned int) bp->offsetPtr;
  bit_count_start = bp->bitCount;
  
  Bp_InByteAlign(bp);
  Bp_GetInt(bp, data);
  
  total = 32;

  /* 
   * Choose appropriate Huffman code table for intra blocks 
   */

  if (Mpeg2PicHdrGetIntraVLCFormat(fh)) {
    /* Set to table 1. */
    dct_coeff_table_flag = 1;
  } else {
    dct_coeff_table_flag = 0;
  }

  /* Choose appropriate scan pattern */

  if (Mpeg2PicHdrGetAltScan(fh)) {
    /* Set unzigzag to alt. scan */
    unzigzag = alt_unzigzag_pattern;
  } else {
    unzigzag = unzigzag_pattern;
  }

  /* Determine number of blocks per macroblock */

  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
    block_count = 6;
  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
    block_count = 8;    
  } else {
    /* 4:4:4 is only other choice */
    block_count = 12;
  }

  /* Get pointers to quant matrices */

  lum_q_matrix = Mpeg2SeqHdrGetLumIQuantMatrix(sh);
  chr_q_matrix = Mpeg2SeqHdrGetChrIQuantMatrix(sh);

  /* Determine quant scale type */
  q_scale_type = Mpeg2PicHdrGetQScaleType(fh);

  /* Parse and decode slices. */

  while ((data >= MPEG2_SLICE_MIN_START_CODE) &&
	 (data <= MPEG2_SLICE_MAX_START_CODE)) {
    int slice_vertical_pos;
    int qscale_code;
    int intra_slice;
    int prev_mb_addr;
    int mb_addr;
    int mb_row, mb_col;
    int i;
    short dc_lum_pred;
    short dc_cr_pred;
    short dc_cb_pred;
    short *dc_pred;

    /* Reset the motion vector predictors b/c 
     * at beginning of slice.
     */

    Mpeg2PicHdrResetPMV(fh);

    /*
     * Reset dc predictors - this seems to be very different
     * from what the MSSG guys do!!!!
     */

    switch (Mpeg2PicHdrGetIntraDCPrecision(fh)) {
    case 0:
      dc_lum_pred = dc_cr_pred = dc_cb_pred = 128; 
      dc_quant_scale = 3;
      break;
    case 1:
      dc_lum_pred = dc_cr_pred = dc_cb_pred = 256; 
      dc_quant_scale = 2;
      break;
    case 2:
      dc_lum_pred = dc_cr_pred = dc_cb_pred = 512; 
      dc_quant_scale = 1;
      break;
    case 3:
      dc_lum_pred = dc_cr_pred = dc_cb_pred = 1024; 
      dc_quant_scale = 0;
      break;
    default:
      /* Totally bogus, but eliminates initialization warnings. */
      dc_lum_pred = dc_cr_pred = dc_cb_pred = 0;
      dc_quant_scale = 0;
    }

    /* Determine slice vertical position. */

    slice_vertical_pos = data & 0xff;

    if (Mpeg2SeqHdrGetHeight(sh) > 2800) {
      /* Get slice vertical pos extension */
      Bp_GetBits(bp, 3, data);
      total += 3;
      slice_vertical_pos += (data << 7);
    }
  
    slice_vertical_pos--;

    if (Mpeg2SeqHdrScalableExtPresent(sh)) {
      /* XXX - scalable extensions not implemented */
    }

    /* Get quantizer scale code and calculate quantizer
     * scale value.
     */
    Bp_GetBits(bp, 5, data);
    total += 5;
    qscale_code = data;

    if (q_scale_type == MPEG2_NON_LINEAR_Q_SCALE) {
      qscale = nonlinear_q_scale_table[qscale_code];
    } else {
      qscale = qscale_code * 2;
    }

    /* Check for intra slice info */
    intra_slice = 0;
    Bp_GetBits(bp, 1, data);
    total += 1;
    if (data) {
      /* Intra slice info is there. */
      Bp_GetBits(bp, 1, data);
      total += 1;
      intra_slice = data;
      
      /* Parse off 7 reserved bits. */
      Bp_FlushBits(bp, 7);
      total += 7;

      /* Parse off extra slice info. */
      Bp_GetBits(bp, 1, data);
      total += 1;
      while(data) {
	Bp_FlushBits(bp, 8);
	total += 8;
	Bp_GetBits(bp, 1, data);
	total += 1;
      }
    }

    /* Need to use Bp_LargePeekBits to get more than 16 bits
     * since all the other Bp macros assume that only up to
     * 16 bits are taken at a time.
     */
    Bp_LargePeekBits(bp, 23, data);

    /* Calculate previous macroblock address */

    prev_mb_addr = (slice_vertical_pos * Mpeg2SeqHdrGetMBWidth(sh)) - 1;

    /* While not at end of slice, decode macroblocks. */

    while(data != 0) {
      Mpeg2MBInfo *curr_mb_info;

      /* Macroblock decoding here. 
       * First, calculate macroblock address
       */
    
      mb_addr = prev_mb_addr + Mpeg2DecodeMBAddrIncrement(bp, &off);
      total += off;

      /* Since this is an I picture, no need to check for 
       * skipped macroblocks.
       */

      prev_mb_addr = mb_addr;

      /* Calculate macroblock row and column index to make
       * array indexing later easier.
       */

      mb_row = mb_addr / Mpeg2SeqHdrGetMBWidth(sh);
      mb_col = mb_addr % Mpeg2SeqHdrGetMBWidth(sh);

      /* Get pointer to appropriate MBInfo structure where
       * macroblock info (i.e, type, motion vectors, etc.) go.
       */

      curr_mb_info = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);

      /* Decode macroblock mode */
      
      if (Mpeg2SeqHdrScalableExtPresent(sh)) {
	/* XXX - Decode macroblock mode based on scalability.
	 * Not yet supported. 
	 */
      } else {
	total += Mpeg2DecodeINonScalableMBMode(bp, curr_mb_info,
					       Mpeg2PicHdrGetConcealmentMVFlag(fh),
					       Mpeg2PicHdrGetStructureCode(fh),
					       Mpeg2PicHdrGetFramePredFrameDCT(fh)
					       );
      }

      /* If quant code encoded, parse it and calculate quantizer value */

      if (Mpeg2MBInfoGetQuant(curr_mb_info)) {
	Bp_GetBits(bp, 5, data);
	total += 5;
	qscale_code = data;
	
	if (q_scale_type == MPEG2_NON_LINEAR_Q_SCALE) {
	  qscale = nonlinear_q_scale_table[qscale_code];
	} else {
	  qscale = qscale_code * 2;
	}
      }
      
      /* Save qscale for posterity */
      Mpeg2MBInfoSetQuant(curr_mb_info, qscale);
					      

      /* If concealment is on, need to decode concealment motion
       * vectors even though we're not doing anything with them.
       */
      if (Mpeg2PicHdrGetConcealmentMVFlag(fh)) {
	total += Mpeg2DecodeMotionVectors(fh, bp, 0, curr_mb_info);

	if (field_flag) {
	  Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FIELD_MOTION_TYPE_FIELD);
	} else {
	  Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FRAME_MOTION_TYPE_FRAME);
	}

	/* Flush marker bit */
	Bp_FlushBits(bp, 1);
	total += 1;
      }
      
      /* No need to update motion vectors. Never used and even if 
       * we did, no need to maintain the second forward vector 
       * for I pictures.
       */

      for (i=0; i<block_count; i++) {
	/* Decode blocks here. */
	ScBlock *dest;
	int block_col, block_row;
	int lum_flag;
	unsigned char *qmatrix;

	/* Determine which plane (Y, Cr, and Cb) the block
	 * is from and get pointer to ScBlock from appropriate
	 * ScImage. Also sets lum_flag, dc_pred, and qmatrix
	 * as appropriate.
	 */

	if (i < 4) {
	  block_col = (mb_col * 2)+(i%2);
	  block_row = (mb_row * 2)+(i/2);
	  dest = ScGetBlock(y, block_col, block_row);
	  lum_flag = 1;
	  dc_pred = &(dc_lum_pred);
	  qmatrix = lum_q_matrix;
	} else {
	  qmatrix = chr_q_matrix;
	  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
	    if (i == 4) {
	      dest = ScGetBlock(u, mb_col, mb_row);
	      dc_pred =&(dc_cb_pred);
	    } else {
	      dest = ScGetBlock(v, mb_col, mb_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
	    if ((i % 2) == 0 ) {
	      block_row = (mb_row * 2) + ((i-4)/2);
	      dest = ScGetBlock(u, mb_col, block_row);
	      dc_pred = &(dc_cb_pred);
	    } else {
	      block_row = (mb_row * 2) + ((i-5)/2);
	      dest = ScGetBlock(v, mb_col, block_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  } else {
	    /* 4:4:4 case */
	    if ((i%2) == 0) {
	      block_row = (mb_row * 2) + (((i - 4)/2)%2);
	      block_col = (mb_col * 2) + (((i-4)/2)/2);
	      dest = ScGetBlock(u, block_col, block_row);
	      dc_pred =&(dc_cb_pred);
	    } else {
	      block_row = (mb_row * 2) + (((i - 5)/2)%2);
	      block_col = (mb_col * 2) + (((i-5)/2)/2);
	      dest = ScGetBlock(v, block_col, block_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  }
	  lum_flag = 0;
	}

	/* Set flag values for ScBlock that are known for
	 * all blocks.
	 */
	dest->intracoded = 1;
	dest->skipMB = 0;
	dest->skipBlock = 0;

	total += Mpeg2DecodeIntraBlock(bp, i, dest, lum_flag, dc_pred, 
				       dct_coeff_table_flag, 
				       unzigzag, dc_quant_scale,
				       qmatrix, qscale);
      }
      Bp_LargePeekBits(bp, 23, data);
    }
   
    Bp_InByteAlign(bp);
    data = NextStartCode(bp, &off);
    total += (off * 8);
  }
  Bp_RestoreInt(bp);
  total -= 32;

  byte_end = (unsigned int) bp->offsetPtr;
  bit_count_end = bp->bitCount;

  return (((byte_end - byte_start) * 8) + bit_count_start - bit_count_end)/8;
}

/*-----------------------------------------
 * Mpeg2DecodeMotionCode
 *
 * Given BitParser *bp and int *off, parsed, decodes
 * and returns motion code. Off contains number
 * of bits parsed.
 *-----------------------------------------*/


static int
Mpeg2DecodeMotionCode(BitParser *bp, int *off)
{
  unsigned int data;
  int ret_val;
  
  /* Check first bit for 1, most common case. */
  Bp_GetBits(bp, 1, data);
  
  if (data) {
    *off = 1;
    return 0;
  }

  /* First bit is zero, do VLC lookup on
   * next 10 bits.
   */

  Bp_PeekBits(bp, 10, data);

  ret_val = motion_code_value_table[data];
  *off = motion_code_size_table[data];
  Bp_FlushBits(bp, *off);

  /* Add one to number of bits parsed for
   * the first bit we checked above before
   * doing table lookup.
   */

  *off += 1;
  
  return ret_val;
}

/*-----------------------------------------
 * Mpeg2DecodeMBAddrIncrement
 *
 * Given BitParser *bp and int *off, parses,
 * decodes, and returns macroblock address
 * increment. Off set to number of bits parsed.
 *-----------------------------------------*/
  
static int
Mpeg2DecodeMBAddrIncrement(BitParser *bp, int *off) 
{
  unsigned int data;
  int ret_val;

  /* Check first bit for 1, most common case. */
  Bp_GetBits(bp, 1, data);

  if (data) {
    *off = 1;
    return 1;
  }

  /* Get next 10 bits and check table */
  Bp_PeekBits(bp, 10, data);
  ret_val = mb_addr_increment_value_table[data];
  *off = mb_addr_increment_size_table[data];
  Bp_FlushBits(bp, *off);

  /* Add 1 to off for first bit checked before
   * table lookup.
   */
  *off += 1;

  /* If increment is escape coded, recursively decode
   * increment. By adding 33 to return value and 11 to
   * number of bits parsed, we account for encoding
   * of the address increment escape.
   */
  if (ret_val == MB_ADDR_INCREMENT_ESCAPE) {
    ret_val = Mpeg2DecodeMBAddrIncrement(bp, off) + 33;
    *off += 11;
  }
  return ret_val;
}

/*-----------------------------------------
 * Mpeg2DecodeINonScalableMBMode
 *
 * Parameters:
 *   BitParser *bp       -- source of bits to parse/decode
 *   Mpeg2MBInfo *mbi    -- structure that holds macroblock specific
 *                          info coded into mb mode.
 *   int concealment_flag -- indicates presense of concealment 
 *                           motion vectors
 *   int pic_structure_code -- indicates frame or field structure
 *   int frame_pred_frame_flag -- value of frame_pred_frame_flag from
 *                                picture coding extension header.
 * Return value:
 *   Number of bits parsed.
 *
 * Notes:
 *   Macroblock mode decoded and appropriate fields of 
 *   mbi are filled. Concealment_flag, pic_structure_flag, and
 *   frame_pref_frame_flag are required because mode encoding 
 *   syntax and interpretation is affected by these fields.
 *
 *-----------------------------------------*/

static int
Mpeg2DecodeINonScalableMBMode(BitParser *bp,
			      Mpeg2MBInfo *mbi,
			      int concealment_flag,
			      int pic_structure_code,
			      int frame_pred_frame_flag)

{
  unsigned int data;
  int total;

  /* Only two modes possible for I fields and pictures.
   * Which mode used can be determined by first bit of
   * VLC. The two codes are "1" and "01". 
   */
 
  Bp_GetBits(bp, 1, data);

  //printf("pic_structure_code %i, frame_pred_frame_flag %i\n", pic_structure_code, frame_pred_frame_flag);
  
  if (data) {
    Mpeg2MBInfoSetQuant(mbi,0);
    total = 1;
  } else {
    Bp_FlushBits(bp, 1);
    Mpeg2MBInfoSetQuant(mbi,1);
    total = 2;
  }

  /* These fields of the mbi are same for all
   * macroblocks in an I picture.
   */

  Mpeg2MBInfoSetMFwd(mbi,0);
  Mpeg2MBInfoSetMBwd(mbi,0);
  Mpeg2MBInfoSetPattern(mbi,0);
  Mpeg2MBInfoSetIntra(mbi,1);


  /* Parse rest of mb mode and set mbi fields
   * according to whether picture is frame or field.
   */

  if (pic_structure_code == STRUCTURE_CODE_FRAME) {
    if (frame_pred_frame_flag == 0) {
      Bp_GetBits(bp, 1, data);
      Mpeg2MBInfoSetDCTType(mbi, data);
      total += 1;
    } else {
      Mpeg2MBInfoSetDCTType(mbi, 0);
    }
    if (concealment_flag) {
      Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
      Mpeg2MBInfoSetMVCount(mbi, 1);
      Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
      Mpeg2MBInfoSetDMVFlag(mbi, 0);
    }
  } else {
    Mpeg2MBInfoSetDCTType(mbi, 0);
    if (concealment_flag) {
      Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_FIELD);
      Mpeg2MBInfoSetMVCount(mbi, 1);
      Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
      Mpeg2MBInfoSetDMVFlag(mbi, 0);
    }
  }      
  return total;
}

/*-----------------------------------------
 * Mpeg2DecodeMotionVectors
 *
 * Parameters:
 *   Mpeg2PicHdr *fh    -- picture header info
 *   BitParser *bp      -- source of bits to be parsed
 *   int back_flag      -- indicates whether forward or
 *                         backward motion vectors are being decoded
 *   Mpeg2MBInfo *mbi   -- structure in which to store vector values
 *
 * Return value:
 *   Number of bits parsed.
 *
 * Notes:
 *   VLC codes for motion vectors are decoded from bp and info
 *   is stored in appropriate fields of mbi. Field select flags
 *   also decoded and stored in mbi. Mbi should provide
 *   correct values for motion vector count, format, and the
 *   presence of a dmv (differential motion vector used with
 *   the Dual-Prime mode). The function Mpeg2DecodeMVector is
 *   used for the actual VLC decoding.
 *-----------------------------------------*/

static int
Mpeg2DecodeMotionVectors(Mpeg2PicHdr *fh,
			 BitParser *bp,
			 int back_flag,
			 Mpeg2MBInfo *mbi)
{
  unsigned int data;
  int total = 0;

  int mv_count = Mpeg2MBInfoGetMVCount(mbi);
  int mv_format = Mpeg2MBInfoGetMVFormat(mbi);
  int dmv_flag = Mpeg2MBInfoGetDMVFlag(mbi);

  if (mv_count == 1) {
    if ((mv_format == MV_FORMAT_FIELD) && (dmv_flag != 1)) {
      Bp_GetBits(bp, 1, data);
      total += 1;

      Mpeg2MBInfoSetMVFieldSelect(mbi, 0, back_flag, data);
    }
    total += Mpeg2DecodeMVector(fh, bp, 0, back_flag, mv_format, dmv_flag, mbi);
  } else {
    Bp_GetBits(bp, 1, data);
    total += 1;
    Mpeg2MBInfoSetMVFieldSelect(mbi, 0, back_flag, data);
    total += Mpeg2DecodeMVector(fh, bp, 0, back_flag, mv_format, dmv_flag, mbi);

    Bp_GetBits(bp, 1, data);
    total += 1;
    Mpeg2MBInfoSetMVFieldSelect(mbi, 1, back_flag, data);
    total += Mpeg2DecodeMVector(fh, bp, 1, back_flag, mv_format, dmv_flag, mbi);
  }
  return total;
}

/*-----------------------------------------
 * Mpeg2DecodeMVector
 * 
 * Parameters:
 *   Mpeg2PicHdr *fh     -- picture header info
 *   BitParser *bp       -- source of bits to parse/decode
 *   int vector_select   -- indicates whether this is first or second
 *                          vector encoded.
 *   int back_flag       -- indicates whether vector is for backward
 *                          or forward motion.
 *   int mv_format       -- indicates field or frame vector format
 *   int dmv_flag        -- indicates presence of dmv (i.e., Dual-Prime)
 *   Mpeg2MBInfo *mbi    -- structure where vector info is stored.
 *
 * Return value:
 *   Number of bits parsed.
 *
 * Notes:
 *   Motion vectors are parsed and decoded. Combined with motion vector
 *   predictors in fh and stored in mbi. Predictors in fh are updated.
 *   If dmv is present, stored in space for second vector (we can do
 *   this because Dual-Prime mode only encodes one vector along with
 *   dmv so space is free to use).
 *-----------------------------------------*/

static int
Mpeg2DecodeMVector(Mpeg2PicHdr *fh,
		   BitParser *bp,
		   int vector_select,
		   int back_flag,
		   int mv_format,
		   int dmv_flag,
		   Mpeg2MBInfo *mbi) 
{
  int total = 0;
  int off;
  int horiz_mc, horiz_m_residual, horiz_dmvector;
  int vert_mc, vert_m_residual, vert_dmvector;
  int horiz_fcode, vert_fcode;
  unsigned int data;
  int r_size, f, high, low, range;
  int prediction, h_comp, v_comp;
  int delta;

  /* First horizontal stuff */

  horiz_mc = Mpeg2DecodeMotionCode(bp, &off);
  total += off;

  horiz_fcode = Mpeg2PicHdrGetFCode(fh, back_flag, 0);

  if ((horiz_fcode != 1) && (horiz_mc != 0)) {
    Bp_GetBits(bp, horiz_fcode-1, data);
    horiz_m_residual = data;
    total += horiz_fcode-1;
  } else {
    horiz_m_residual = 0;
  }
    
  if (dmv_flag == 1) {
    Bp_GetBits(bp, 1, data);
    total += 1;
    if (!data) {
      horiz_dmvector = 0;
    } else {
      Bp_GetBits(bp, 1, data);
      total += 1;
      if (data) {
	horiz_dmvector = -1;
      } else {
	horiz_dmvector = 1;
      }
    }
    /* Store dmvector into vector 2 since dm vectors are only
     * used with Dual Prime and that will mean space for second
     * vector is free.
     */
    Mpeg2MBInfoSetMVectorHorizComp(mbi, 1, back_flag, horiz_dmvector);
    
  }

  /* Now vertical stuff */

  vert_mc = Mpeg2DecodeMotionCode(bp, &off);
  total += off;

  vert_fcode = Mpeg2PicHdrGetFCode(fh, back_flag, 1);

  if ((vert_fcode != 1) && (vert_mc != 0)) {
    Bp_GetBits(bp, vert_fcode-1, data);
    vert_m_residual = data;
    total += vert_fcode-1;
  } else {
    vert_m_residual = 0;
  }
    
  if (dmv_flag == 1) {
    Bp_GetBits(bp, 1, data);
    total += 1;
    if (!data) {
      vert_dmvector = 0;
    } else {
      Bp_GetBits(bp, 1, data);
      total += 1;
      if (data) {
	vert_dmvector = -1;
      } else {
	vert_dmvector = 1;
      }
    }
    /* Store dmvector into vector 2 (index = 1) since dm vectors are only
     * used with Dual Prime and that will mean space for second
     * vector is free.
     */
    Mpeg2MBInfoSetMVectorVertComp(mbi, 1, back_flag, vert_dmvector);

  }

  /* Calculate horizontal value */

  r_size = horiz_fcode - 1;
  f = 1 << r_size;
  high = (16 * f)-1;
  low = ((-16)*f);
  range = (32 * f);
  
  if ((f == 1) || (horiz_mc == 0)) {
    delta = horiz_mc;
  } else {
    delta = ((abs(horiz_mc)-1)*f)+horiz_m_residual+1;
    if (horiz_mc < 0) {
      delta *= -1;
    }
  }
  
  prediction = Mpeg2PicHdrGetPMVHorizComp(fh, vector_select, back_flag);
  
  h_comp = prediction + delta;
  if (h_comp < low) {
    h_comp += range;
  }
  if (h_comp > high) {
    h_comp -= range;
  }

  Mpeg2PicHdrSetPMVHorizComp(fh, vector_select, back_flag, h_comp);
  Mpeg2MBInfoSetMVectorHorizComp(mbi, vector_select, back_flag, h_comp);

  /* Calculate vertical value */

  r_size = vert_fcode - 1;
  f = 1 << r_size;
  high = (16 * f)-1;
  low = ((-16)*f);
  range = (32 * f);
  
  if ((f == 1) || (vert_mc == 0)) {
    delta = vert_mc;
  } else {
    delta = ((abs(vert_mc)-1)*f)+vert_m_residual+1;
    if (vert_mc < 0) {
      delta *= -1;
    }
  }
  
  prediction = Mpeg2PicHdrGetPMVVertComp(fh, vector_select, back_flag);

  if ((mv_format == MV_FORMAT_FIELD) && (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_FRAME)) {
    if (prediction < 0) {
      prediction = (prediction - 1) / 2;
    } else {
      prediction = prediction / 2;
    }
  }
  
  v_comp = prediction + delta;
  if (v_comp < low) {
    v_comp += range;
  }
  if (v_comp > high) {
    v_comp -= range;
  }

  if ((mv_format == MV_FORMAT_FIELD) && (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_FRAME)) {
    Mpeg2PicHdrSetPMVVertComp(fh, vector_select, back_flag, v_comp*2);
  } else {
    Mpeg2PicHdrSetPMVVertComp(fh, vector_select, back_flag, v_comp);
  }  

  Mpeg2MBInfoSetMVectorVertComp(mbi, vector_select, back_flag, v_comp);

  return total;
}


int
Mpeg2DecodeIntraBlock(BitParser *bp, 
		      int block_num, 
		      ScBlock *dest, 
		      int lum_flag, 
		      short *dc_pred, 
		      int dct_coeff_table_flag,
		      char *unzigzag,
		      int dc_quant_scale,
		      unsigned char *qmatrix,
		      int qscale)
{
  unsigned int data;
  int total = 0;
  char *val_table, *size_table;
  int dc_diff, dc_size, code_size;
  short *decoding_table;
  short **large_code_table;
  int num_ac_coeffs;
  int cur_idx;
  int odd_flag;
  int run;
  int level;
  int idx;
  int value;
  int len;
  
  if (lum_flag) {
    val_table = dct_dc_size_lum_value_table;
    size_table = dct_dc_size_lum_size_table;
    len = 9;
  } else {
    val_table = dct_dc_size_chr_value_table;
    size_table = dct_dc_size_chr_size_table;
    len = 10;
  }
  Bp_PeekBits(bp, len, data);
  dc_size = val_table[data];
  code_size = size_table[data];
  Bp_FlushBits(bp, code_size);
  total += code_size;

  if (dc_size) {
    Bp_GetBits(bp, dc_size, data);
    total += dc_size;

    dc_diff = data;

    if ((dc_diff & (1<<(dc_size-1))) == 0) {
      dc_diff -= (1<<dc_size)-1;
    }
  } else {
    dc_diff = 0;
  }

  dest->dc = (*dc_pred += dc_diff);

  dest->dc <<= dc_quant_scale;
  
  if (dct_coeff_table_flag == 0) {
    decoding_table = dct_coeff_table_zero;
    large_code_table = dct_coeff_large_code_table_zero;
  } else {
    decoding_table = dct_coeff_table_one;
    large_code_table = dct_coeff_large_code_table_one;
  }    
  
  num_ac_coeffs = 0;
  cur_idx = 1;

  odd_flag = dest->dc;

  while(1) {
    register short code;
    int mode;

    Bp_PeekBits(bp, 9, data);
    code = decoding_table[data];
    len = DCT_COEFF_TABLE_LEN(code);
    Bp_FlushBits(bp, len);
    total += len;
    mode = DCT_COEFF_TABLE_MODE(code);

    if (mode == DCT_COEFF_TABLE_EOB_MODE) {
      break; 
    } else if (mode == DCT_COEFF_TABLE_NORMAL_MODE) {
      run = DCT_COEFF_TABLE_RUN(code);
      level = DCT_COEFF_TABLE_LEVEL(code);
    } else if (mode == DCT_COEFF_TABLE_MOREBITS_MODE) {
      short *subtable;
      int idx = DCT_COEFF_TABLE_SUBTABLE_IDX(code);
      int num_bits = DCT_COEFF_TABLE_SUBTABLE_NBITS(code);

      subtable = large_code_table[idx];
      Bp_PeekBits(bp, num_bits, data);

      code = subtable[data];
      len = DCT_COEFF_SUBTABLE_LEN(code);
      run = DCT_COEFF_SUBTABLE_RUN(code);
      level = DCT_COEFF_SUBTABLE_LEVEL(code);

      Bp_FlushBits(bp, len);
      total += len;
    } else if (mode == DCT_COEFF_TABLE_ESC_MODE) {
      Bp_GetBits(bp, 6, data);
      run = data;
      Bp_GetBits(bp, 12, data);
      level = data;
      if (level & 0x800) {
	level |= 0xfffff000;
      }
    }

    cur_idx += run;
    idx = unzigzag[cur_idx];
      
    dest->index[num_ac_coeffs] = idx;

    value = (level * qmatrix[idx] * qscale)/16;

    value = (value < -2048) ? -2048 : (value > 2047) ? 2047 : value;
    dest->value[num_ac_coeffs] = value;
      
    odd_flag ^= value;
    cur_idx++;
    num_ac_coeffs++;
  }


  /* Mismatch control */
  if (!(odd_flag & 0x1)) {
    if (num_ac_coeffs) {
      if (dest->index[num_ac_coeffs-1] == 63) {
	dest->value[num_ac_coeffs-1] ^= 0x1;
      } else {
	dest->value[num_ac_coeffs] = 1;
	dest->index[num_ac_coeffs] = 63;
	num_ac_coeffs++;
      }
    } else {
      dest->value[0] = 1;
      dest->index[0] = 63;
      num_ac_coeffs = 1;
    }
  }
  
  dest->numOfAC = num_ac_coeffs;
  return total;
}


int
Mpeg2ScIFrameToByte(ScImage *sc_image,
		    Mpeg2MBInfoArray *mbi_array,
		    ByteImage *dest_image,
		    int lum_flag,
		    int non_420_flag)
{
  int mb_width, mb_height;
  int sc_width, sc_height;
  int mb_h_size, mb_v_size;
  int mb_addr;
  int mbx, mby;
  int ix, iy;
  Mpeg2MBInfo *mbi;
  int dest_w;
  ScBlock *scb;
  unsigned char *dest;
  short temp[64];
  int field_dct_flag;

	mb_width = Mpeg2MBInfoArrayGetWidth(mbi_array);
	mb_height = Mpeg2MBInfoArrayGetHeight(mbi_array);

	sc_width = sc_image->width;
	sc_height = sc_image->height;

	/* Using mb width and height and sc width and height,
	* infer chrominance formatting and set mb_h_size and
	* mb_v_size as appropriate.
	*/

	if (mb_width == sc_width) {
		mb_h_size = 1;
	} else if (mb_width*2 == sc_width) {
		mb_h_size = 2;
	} else {
		/*Should never get here.*/
		assert(0);
	}

	if (mb_height == sc_height) {
		mb_v_size = 1;
	} else if (mb_height*2 == sc_height) {
		mb_v_size = 2;
	} else {
		/*Should never get here.*/
		assert(0);
	}

	//printf("mb_width %i, mb_height %i, sc_width %i, sc_height %i, mb_h_size %i, mb_v_size %i\n",
	//	mb_width, mb_height, sc_width, sc_height, mb_h_size, mb_v_size);

	Initialize_Fast_IDCT();

	dest_w = dest_image->parentWidth;
	
	//printf("dest_w %i\n", dest_w);

	for (mby = 0; mby < mb_height; mby++) {
		for (mbx = 0; mbx < mb_width; mbx++) {

			/* Get macroblock info */

			mb_addr = mby * mb_width + mbx;
			mbi = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);

			/* Get pointer to appropriate block in ScImage */

			scb = &(sc_image->firstBlock[sc_image->parentWidth*(mby*mb_v_size)
					+(mbx*mb_h_size)]);

			/* Get pointer to appropriate dest byte */
			//printf("Dest pointer in array is %i\n", ((dest_w*(8*mb_v_size*mby))+(8*mb_h_size*mbx)));
			dest = &(dest_image->firstByte[((dest_w*
					(8*mb_v_size*mby))+
					(8*mb_h_size*mbx))]);
			//printf("DCT type %i, lum_flag %i, 420_flag %i\n", Mpeg2MBInfoGetDCTType(mbi), lum_flag, non_420_flag);

			if (Mpeg2MBInfoGetDCTType(mbi) && (lum_flag || non_420_flag)) {
				field_dct_flag = 1;
			} else {
				field_dct_flag = 0;
			}

			//printf("field_dct_flag %i\n", field_dct_flag);

			for (iy=0; iy<mb_v_size; iy++) {

				if (iy == 1) {
					scb += sc_image->parentWidth;

					if (field_dct_flag) {
						//printf("Dest + pWith\n");
						dest += dest_image->parentWidth;
					} else {
						//printf("Dest + pWith * 8\n");
						dest += dest_image->parentWidth*8;
					}
				}


				for (ix=0; ix<mb_h_size; ix++) {

					/* Adjust sc block pointer and dest pointers for
					* which block in macroblock we are doing.
					* This is a bit tricky due to different chrominance
					* formats.
					*/
					if (ix == 1) {
						scb += 1;
						dest += 8;
					}

					DO_IDCT(scb, temp);

					//if (field_dct_flag) {
						//printf("Doing W2\n");
						//THE BUG IS HERE WE ARE WALKING OUT OF BOUNDS
						//IN THE LAST CALL
						//BlockShortToByteCopy(temp, dest, dest_w*2);

						//} else {
						//printf("Doing W\n");

						BlockShortToByteCopy(temp, dest, dest_w);
					//}
				}

				if (ix == 2) {
					scb -= 1;
					dest -= 8;
				}
			}
    		}
  	}

  	return 1;
}



int
Mpeg2ScIFieldToByte(ScImage *sc_image,
		    ByteImage *dest_image)
{
  ScBlock *scb;
  int block_line_step;
  int byte_block_step;
  int dest_width;
  unsigned char *dest;
  short temp[64];
  int by, bx;

  scb = sc_image->firstBlock;
  block_line_step = sc_image->parentWidth - sc_image->width;

  
  dest_width = dest_image->parentWidth;
  byte_block_step = 8 * dest_width - dest_image->width;
  dest = dest_image->firstByte;

  for (by = 0; by < sc_image->height; by++) {
    for (bx = 0; bx < sc_image->width; bx++) {
      DO_IDCT(scb, temp);
      BlockShortToByteCopy(temp, dest, dest_width);
      scb++;
      dest += 8;
    }
    scb += block_line_step;
    dest += byte_block_step;
  }
  return 1;
}

int
Mpeg2ParseP(BitParser *bp,
	    Mpeg2SeqHdr *sh,
	    Mpeg2PicHdr *fh,
	    ScImage *y,
	    ScImage *u,
	    ScImage *v,
	    Mpeg2MBInfoArray *mbi_array,
	    int field_flag)
{
  unsigned int data;
  int intra_dct_coeff_table_flag;
  unsigned char *unzigzag;
  int block_count;
  unsigned char *lum_i_q_matrix, *chr_i_q_matrix;
  unsigned char *lum_non_i_q_matrix, *chr_non_i_q_matrix;
  int q_scale_type;
  int qscale_code;
  int qscale;
  int dc_pred_value;
  int total = 0;
  int dc_quant_scale;
  int off;
  int i;
  int last_mb_addr;
  unsigned int byte_start;
  unsigned int byte_end;
  unsigned int bit_count_start;
  unsigned int bit_count_end;

  byte_start = (unsigned int) bp->offsetPtr;
  bit_count_start = bp->bitCount;

  Bp_InByteAlign(bp);
  Bp_GetInt(bp, data);

  total = 32;
  
  /*
   * Choose appropriate Huffman code table for
   * intra blocks 
   */

  if (Mpeg2PicHdrGetIntraVLCFormat(fh)) {
    /* Set to table 1. */
    intra_dct_coeff_table_flag = 1;
  } else {
    intra_dct_coeff_table_flag = 0;
  }

  if (Mpeg2PicHdrGetAltScan(fh)) {
    unzigzag = alt_unzigzag_pattern;
  } else {
    unzigzag = unzigzag_pattern;
  }

  /* Determine number of blocks per macroblock */

  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
    block_count = 6;
  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
    block_count = 8;    
  } else {
    /* 4:4:4 is only other choice */
    block_count = 12;
  }

  /* Get pointers to quant matrices */

  lum_i_q_matrix = Mpeg2SeqHdrGetLumIQuantMatrix(sh);
  chr_i_q_matrix = Mpeg2SeqHdrGetChrIQuantMatrix(sh);
  lum_non_i_q_matrix = Mpeg2SeqHdrGetLumNonIQuantMatrix(sh);
  chr_non_i_q_matrix = Mpeg2SeqHdrGetChrNonIQuantMatrix(sh);

  /* Determine quant scale type */
  q_scale_type = Mpeg2PicHdrGetQScaleType(fh);

  /* Determine the dc prediction reset value
   * and dc quant scale
   */

  switch (Mpeg2PicHdrGetIntraDCPrecision(fh)) {
  case 0:
    dc_pred_value = 128;
    dc_quant_scale = 3;
    break;
  case 1:
    dc_pred_value = 256;
    dc_quant_scale = 2;
    break;
  case 2:
    dc_pred_value = 512;
    dc_quant_scale = 1;
    break;
  case 3:
    dc_pred_value = 1024;
    dc_quant_scale = 0;
    break;
  default:
    /* Totally bogus, but eliminates initialization warnings. */
    dc_pred_value = 0;
    dc_quant_scale = 0;
  }
  
  last_mb_addr = -1;
  
  while ((data >= MPEG2_SLICE_MIN_START_CODE) &&
	 (data <= MPEG2_SLICE_MAX_START_CODE)) {
    short dc_lum_pred;
    short dc_cr_pred;
    short dc_cb_pred;
    short *dc_pred;
    int slice_vertical_pos;
    int intra_slice;
    int prev_mb_addr;
    int mb_addr;
    int mb_row, mb_col;
    int block_pattern;

    /* Reset the motion vector predictors b/c
     * at beginning of slice;
     */

    Mpeg2PicHdrResetPMV(fh);

    /* Reset dc predictors */

    dc_lum_pred = dc_cr_pred = dc_cb_pred = dc_pred_value;
    
    slice_vertical_pos = data & 0xff;

    if (Mpeg2SeqHdrGetHeight(sh) > 2800) {
      /* Get slice vertical pos extension */
      Bp_GetBits(bp, 3, data);
      total += 3;
      slice_vertical_pos += (data << 7);
    }
  
    slice_vertical_pos--;
    
    if (Mpeg2SeqHdrScalableExtPresent(sh)) {
      /* XXX - scalable extensions not implemented */
    }
    
    /* Get and calculate quantizer */
    Bp_GetBits(bp, 5, data);
    total += 5;
    qscale_code = data;

    if (q_scale_type == MPEG2_NON_LINEAR_Q_SCALE) {
      qscale = nonlinear_q_scale_table[qscale_code];
    } else {
      qscale = qscale_code * 2;
    }

    /* Check for intra slice info */
    intra_slice = 0;
    Bp_GetBits(bp, 1, data);
    total += 1;
    if (data) {
      /* Intra slice info is there. */
      Bp_GetBits(bp, 1, data);
      total += 1;
      intra_slice = data;
      
      /* Parse off 7 reserved bits. */
      Bp_FlushBits(bp, 7);
      total += 7;

      /* Parse off extra slice info. */
      Bp_GetBits(bp, 1, data);
      total += 1;
      while(data) {
	Bp_FlushBits(bp, 8);
	total += 8;
	Bp_GetBits(bp, 1, data);
	total += 1;
      }
    }

    Bp_LargePeekBits(bp, 23, data);
    
    prev_mb_addr = (slice_vertical_pos * Mpeg2SeqHdrGetMBWidth(sh)) - 1;

    while (data != 0) {
      Mpeg2MBInfo *curr_mb_info;
      ScBlock *dest;

      mb_addr = prev_mb_addr + Mpeg2DecodeMBAddrIncrement(bp, &off);
      total += off;

      prev_mb_addr = mb_addr;

      /* Processes skipped macroblocks if any. */
      last_mb_addr++;
      if (mb_addr != last_mb_addr) {
	
	/* Reset predictors because of skipped macroblocks */

	dc_lum_pred = dc_cr_pred = dc_cb_pred = dc_pred_value;
	
	/* Reset motion predictors (this only occurs in P frames) */

	Mpeg2PicHdrResetPMV(fh);

	while (last_mb_addr != mb_addr) {
	  /* Do per skip macroblock processing here. */
	  mb_row = last_mb_addr / Mpeg2SeqHdrGetMBWidth(sh);
	  mb_col = last_mb_addr % Mpeg2SeqHdrGetMBWidth(sh);
      
	  curr_mb_info = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, last_mb_addr);

	  Mpeg2MBInfoSetQuant(curr_mb_info, 0);
	  Mpeg2MBInfoSetMFwd(curr_mb_info, 1);
	  Mpeg2MBInfoSetMBwd(curr_mb_info, 0);
	  Mpeg2MBInfoSetPattern(curr_mb_info, 0);
	  Mpeg2MBInfoSetIntra(curr_mb_info, 0);
	  Mpeg2MBInfoSetDCTType(curr_mb_info, 0);

	  if (field_flag) {
	    Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FIELD_MOTION_TYPE_FIELD);
	    Mpeg2MBInfoSetMVFormat(curr_mb_info, MV_FORMAT_FIELD);
	    
	    if (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_TOP) {
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 0);
	    } else {
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 1);
	    }
	  } else {
	    Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FRAME_MOTION_TYPE_FRAME);
	    Mpeg2MBInfoSetMVFormat(curr_mb_info, MV_FORMAT_FRAME);
	  }

	  Mpeg2MBInfoSetDMVFlag(curr_mb_info, 0);
	  Mpeg2MBInfoSetMVCount(curr_mb_info, 1);

	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 0, 0);
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 0, 0);

	  dest = ScGetBlock(y, (mb_col * 2), (mb_row * 2));
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  dest = ScGetBlock(y, (mb_col * 2)+1, (mb_row * 2));
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  dest = ScGetBlock(y, (mb_col * 2), (mb_row * 2)+1);
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  dest = ScGetBlock(y, (mb_col * 2)+1, (mb_row * 2)+1);
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
	    dest = ScGetBlock(u, mb_col, mb_row);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(v, mb_col, mb_row);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
	    dest = ScGetBlock(u, mb_col, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, mb_col, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, mb_col, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, mb_col, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	  } else {
	    dest = ScGetBlock(u, (mb_col*2), (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, (mb_col*2), (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, (mb_col*2)+1, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, (mb_col*2)+1, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, (mb_col*2), (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, (mb_col*2), (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(v, (mb_col*2)+1, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, (mb_col*2)+1, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	  }

	  last_mb_addr++;
	}
      }

       
      mb_row = mb_addr / Mpeg2SeqHdrGetMBWidth(sh);
      mb_col = mb_addr % Mpeg2SeqHdrGetMBWidth(sh);
      
      curr_mb_info = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);

      /* Decode macroblock mode */
      
      if (Mpeg2SeqHdrScalableExtPresent(sh)) {
	/* Decode macroblock mode based on scalability.
	 * Not yet supported. 
	 */
      } else {
	total += Mpeg2DecodePNonScalableMBMode(bp, curr_mb_info,
					       Mpeg2PicHdrGetConcealmentMVFlag(fh),
					       Mpeg2PicHdrGetStructureCode(fh),
					       Mpeg2PicHdrGetFramePredFrameDCT(fh)
					       );
      }	
	
      /* If quant code encoded, parse it */
      if (Mpeg2MBInfoGetQuant(curr_mb_info)) {
	Bp_GetBits(bp, 5, data);
	total += 5;
	qscale_code = data;
	
	if (q_scale_type == MPEG2_NON_LINEAR_Q_SCALE) {
	  qscale = nonlinear_q_scale_table[qscale_code];
	} else {
	  qscale = qscale_code * 2;
	}
      }

      /* Save qscale for posterity */
      Mpeg2MBInfoSetQuant(curr_mb_info, qscale);
    
      if (Mpeg2MBInfoGetMFwd(curr_mb_info) ||
	  (Mpeg2MBInfoGetIntra(curr_mb_info) && 
	   Mpeg2PicHdrGetConcealmentMVFlag(fh))) {
	total += Mpeg2DecodeMotionVectors(fh, bp, 0, curr_mb_info);
	
	if (Mpeg2MBInfoGetIntra(curr_mb_info) &&
	    Mpeg2PicHdrGetConcealmentMVFlag(fh)) {

	  /* Flush marker bit */
	  Bp_FlushBits(bp, 1);
	  total += 1;
	}
      } else {
	if (!Mpeg2MBInfoGetIntra(curr_mb_info)) {
	  /* No encoded motion vectors, but not intra either! */
	  Mpeg2MBInfoSetMVCount(curr_mb_info, 1);
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 0, 0);
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 0, 0);
	  Mpeg2PicHdrResetPMV(fh);
	  if (field_flag) {
	    Mpeg2MBInfoSetMotionType(curr_mb_info,
				     MPEG2_FIELD_MOTION_TYPE_FIELD);
	    Mpeg2MBInfoSetMVFormat(curr_mb_info, MV_FORMAT_FIELD);
	    if (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_TOP) {
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 0);
	    } else {
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 1);
	    }
	  } else {
	    Mpeg2MBInfoSetMotionType(curr_mb_info,
				     MPEG2_FRAME_MOTION_TYPE_FRAME);
	    Mpeg2MBInfoSetMVFormat(curr_mb_info, MV_FORMAT_FRAME);
	  }
	}
      }

      /* If Dual Prime, then do the additional dual prime arithmetic
       * and put the results into the mb vector space in the following
       * way:
       * 
       * If field picture: [0][0] = predictor for use with top ref. field.
       *                   [1][0] = predictor for use with bot ref. field.
       *
       * If frame picture: [0][0] = predictor for use with top ref. field
       *                            for top field of frame.
       *                   [1][0] = predictor for use with bot ref. field
       *                            for top field of frame.
       *                   [0][1], and [1][1] like above but for bot field
       *                           of frame.
       */

      if (Mpeg2MBInfoGetMotionType(curr_mb_info) == MPEG2_FIELD_MOTION_TYPE_DUAL_PRIME) {
	int efactor;
	short mv_h, dmv_h, mv_v, dmv_v, dual_right, dual_down;

	if (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_TOP) {
	  efactor = -1;
	} else {
	  efactor = 1;
	}

	mv_h = Mpeg2MBInfoGetMVectorHorizComp(curr_mb_info, 0, 0);
	dmv_h = Mpeg2MBInfoGetMVectorHorizComp(curr_mb_info, 1, 0);
	mv_v = Mpeg2MBInfoGetMVectorVertComp(curr_mb_info, 0, 0);
	dmv_v = Mpeg2MBInfoGetMVectorVertComp(curr_mb_info, 1, 0);
	
	dual_right = ((mv_h + (mv_h < 0 ? -1 : 1)) / 2) + dmv_h;
	dual_down = (((mv_v + (mv_v < 0 ? -1 : 1)) / 2) + 
		     dmv_v + efactor);

	if (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_TOP) {
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 0, mv_h);
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 0, mv_v);
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 1, 0, dual_right);
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 1, 0, dual_down);
	} else {
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 0, dual_right);
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 0, dual_down);
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 1, 0, mv_h);
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 1, 0, mv_v);
	}
      } else if (Mpeg2MBInfoGetMotionType(curr_mb_info) == MPEG2_FRAME_MOTION_TYPE_DUAL_PRIME) {	
	int efactor, mfactor;
	short mv_h, dmv_h, mv_v, dmv_v;
	short dual_right_for_top, dual_down_for_top;
	short dual_right_for_bot, dual_down_for_bot;

	if (Mpeg2PicHdrGetTopFieldFirstFlag(fh)) {
	  mfactor = 1;
	} else {
	  mfactor = 3;
	}
	efactor = -1;
	
	mv_h = Mpeg2MBInfoGetMVectorHorizComp(curr_mb_info, 0, 0);
	dmv_h = Mpeg2MBInfoGetMVectorHorizComp(curr_mb_info, 1, 0);
	mv_v = Mpeg2MBInfoGetMVectorVertComp(curr_mb_info, 0, 0);
	dmv_v = Mpeg2MBInfoGetMVectorVertComp(curr_mb_info, 1, 0);
	
	dual_right_for_top = (((mv_h * mfactor) + (mv_h < 0 ? -1 : 1)) / 2) 
	  + dmv_h;
	dual_down_for_top = ((((mv_v * mfactor) + (mv_v < 0 ? -1 : 1)) / 2) 
			     + dmv_v + efactor);
	
	if (Mpeg2PicHdrGetTopFieldFirstFlag(fh)) {
	  mfactor = 3;
	} else {
	  mfactor = 1;
	}
	efactor = 1;
	
	dual_right_for_bot = (((mv_h * mfactor) + (mv_h < 0 ? -1 : 1)) / 2) 
	  + dmv_h;
	dual_down_for_bot = ((((mv_v * mfactor) + (mv_v < 0 ? -1 : 1)) / 2) 
			     + dmv_v + efactor);
	

	Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 0, mv_h);
	Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 0, mv_v);
	Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 1, 0, dual_right_for_top);
	Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 1, 0, dual_down_for_top);
	Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 1, dual_right_for_bot);
	Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 1, dual_down_for_bot);
	Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 1, 1, mv_h);
	Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 1, 1, mv_v);
      }

      /* Update motion vector predictors */
      if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	if (!Mpeg2PicHdrGetConcealmentMVFlag(fh)) {
	  Mpeg2PicHdrResetPMV(fh);
	} else {
	  Mpeg2PicHdrUpdatePMV1Fwd(fh);
	}
      } else {
	if ((Mpeg2MBInfoGetMotionType(curr_mb_info) == 
	     MPEG2_FIELD_MOTION_TYPE_FIELD) || 
	    (Mpeg2MBInfoGetMotionType(curr_mb_info) ==
	     MPEG2_FRAME_MOTION_TYPE_FRAME)) {
	  if (Mpeg2MBInfoGetMFwd(curr_mb_info)) {
	    Mpeg2PicHdrUpdatePMV1Fwd(fh);
	  } else {
	    Mpeg2PicHdrResetPMV(fh);
	  }
	} else if ((Mpeg2MBInfoGetMotionType(curr_mb_info) == 
		    MPEG2_FIELD_MOTION_TYPE_DUAL_PRIME) ||
		   (Mpeg2MBInfoGetMotionType(curr_mb_info) ==
		    MPEG2_FRAME_MOTION_TYPE_DUAL_PRIME)) {
	  if (Mpeg2MBInfoGetMFwd(curr_mb_info)) {
	    Mpeg2PicHdrUpdatePMV1Fwd(fh);
	  }
	} 
      }
      
      if (Mpeg2MBInfoGetPatternFlag(curr_mb_info)) {
	block_pattern = Mpeg2DecodeBlockPattern(bp, &off, Mpeg2SeqHdrGetChromaFormatCode(sh));
	total += off;
      } else {
	if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	  block_pattern = 0xffff;
	} else {
	  block_pattern = 0;
	}
      }

      /* Save pattern for posterity */
      Mpeg2MBInfoSetPattern(curr_mb_info, block_pattern);

      for (i=0; i<block_count; i++) {
	int block_col, block_row;
	int lum_flag;
	unsigned char *qmatrix;

	if (i < 4) {
	  block_col = (mb_col * 2)+(i%2);
	  block_row = (mb_row * 2)+(i/2);
	  dest = ScGetBlock(y, block_col, block_row);
	  lum_flag = 1;
	  dc_pred = &(dc_lum_pred);
	  if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	    qmatrix = lum_i_q_matrix;
	  } else {
	    qmatrix = lum_non_i_q_matrix;
	  }
	} else {
	  if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	    qmatrix = chr_i_q_matrix;
	  } else {
	    qmatrix = chr_non_i_q_matrix;
	  }
	  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
	    if (i == 4) {
		dest = ScGetBlock(u, mb_col, mb_row);
		dc_pred =&(dc_cb_pred);
	    } else {
	      dest = ScGetBlock(v, mb_col, mb_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
	    if ((i % 2) == 0 ) {
	      block_row = (mb_row * 2) + ((i-4)/2);
	      dest = ScGetBlock(u, mb_col, block_row);
	      dc_pred = &(dc_cb_pred);
	    } else {
	      block_row = (mb_row * 2) + ((i-5)/2);
	      dest = ScGetBlock(v, mb_col, block_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  } else {
	    /* 4:4:4 case */
	    if ((i%2) == 0) {
	      block_row = (mb_row * 2) + (((i - 4)/2)%2);
	      block_col = (mb_col * 2) + (((i-4)/2)/2);
	      dest = ScGetBlock(u, block_col, block_row);
	      dc_pred =&(dc_cb_pred);
	    } else {
	      block_row = (mb_row * 2) + (((i - 5)/2)%2);
	      block_col = (mb_col * 2) + (((i-5)/2)/2);
	      dest = ScGetBlock(v, block_col, block_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  }
	  lum_flag = 0;
	}
	
	if (block_pattern & (1 << (block_count - i -1))) {
	  dest->intracoded = Mpeg2MBInfoGetIntra(curr_mb_info);
	  dest->skipMB = 0;
	  dest->skipBlock = 0;
	  
	  if (dest->intracoded) {
	    total += Mpeg2DecodeIntraBlock(bp, i, dest, lum_flag,
					   dc_pred, 
					   intra_dct_coeff_table_flag,
					   unzigzag, dc_quant_scale,
					   qmatrix, qscale);
	  } else {
	    total += Mpeg2DecodeNonIntraBlock(bp, i, dest,
					      unzigzag, 
					      qmatrix,
					      qscale);

	    /* Reset predictors */

	    dc_lum_pred = dc_cr_pred = dc_cb_pred = dc_pred_value;
	    
	  }
	} else {
	  /* Do skipped blocks here. */
	  dest->intracoded = 0;
	  dest->skipMB = 0;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	}
      }
      Bp_LargePeekBits(bp, 23, data);
    }

    Bp_InByteAlign(bp);
    data = NextStartCode(bp, &off);
    total += (off * 8);
  }
  Bp_RestoreInt(bp);
  total -= 32;

  byte_end = (unsigned int) bp->offsetPtr;
  bit_count_end = bp->bitCount;

  return (((byte_end - byte_start) * 8) + bit_count_start - bit_count_end)/8;
}
	  
static int
Mpeg2DecodePNonScalableMBMode(BitParser *bp,
			      Mpeg2MBInfo *mbi,
			      int concealment_flag,
			      int pic_structure_code,
			      int frame_pred_frame_flag)

{
  unsigned int data;
  int total = 0;

  Bp_PeekBits(bp, 6, data);

  Mpeg2MBInfoSetMBwd(mbi, 0);

  if (data & 0x20) {
    Mpeg2MBInfoSetQuant(mbi, 0);
    Mpeg2MBInfoSetMFwd(mbi, 1);
    Mpeg2MBInfoSetPattern(mbi, 1);
    Mpeg2MBInfoSetIntra(mbi, 0);
    Bp_FlushBits(bp, 1);
    total += 1;
  } else if (data & 0x10) {
    Mpeg2MBInfoSetQuant(mbi, 0);
    Mpeg2MBInfoSetMFwd(mbi, 0);
    Mpeg2MBInfoSetPattern(mbi, 1);
    Mpeg2MBInfoSetIntra(mbi, 0);
    Bp_FlushBits(bp, 2);
    total += 2;
  } else if (data & 0x8) {
    Mpeg2MBInfoSetQuant(mbi, 0);
    Mpeg2MBInfoSetMFwd(mbi, 1);
    Mpeg2MBInfoSetPattern(mbi, 0);
    Mpeg2MBInfoSetIntra(mbi, 0);
    Bp_FlushBits(bp, 3);
    total += 3;
  } else if (data & 0x4) {
    if (data & 0x2) {
      Mpeg2MBInfoSetQuant(mbi, 0);
      Mpeg2MBInfoSetMFwd(mbi, 0);
      Mpeg2MBInfoSetPattern(mbi, 0);
      Mpeg2MBInfoSetIntra(mbi, 1);
      Bp_FlushBits(bp, 5);
      total += 5;
    } else {
      Mpeg2MBInfoSetQuant(mbi, 1);
      Mpeg2MBInfoSetMFwd(mbi, 1);
      Mpeg2MBInfoSetPattern(mbi, 1);
      Mpeg2MBInfoSetIntra(mbi, 0);
      Bp_FlushBits(bp, 5);
      total += 5;
    }
  } else if (data & 0x2) {
    Mpeg2MBInfoSetQuant(mbi, 1);
    Mpeg2MBInfoSetMFwd(mbi, 0);
    Mpeg2MBInfoSetPattern(mbi, 1);
    Mpeg2MBInfoSetIntra(mbi, 0);
    Bp_FlushBits(bp, 5);
    total += 5;
  } else if (data & 0x1) {
    Mpeg2MBInfoSetQuant(mbi, 1);
    Mpeg2MBInfoSetMFwd(mbi, 0);
    Mpeg2MBInfoSetPattern(mbi, 0);
    Mpeg2MBInfoSetIntra(mbi, 1);
    Bp_FlushBits(bp, 6);
    total += 6;
  }

  if (Mpeg2MBInfoGetMFwd(mbi)) {
    if (pic_structure_code == STRUCTURE_CODE_FRAME) {
      if (!frame_pred_frame_flag) {
	Bp_GetBits(bp, 2, data);
	total += 2;
	if (data == 1) {
	  Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FIELD);
	  Mpeg2MBInfoSetMVCount(mbi, 2);
	  Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	  Mpeg2MBInfoSetDMVFlag(mbi, 0);
	} else if (data == 2) {
	  Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
	  Mpeg2MBInfoSetMVCount(mbi, 1);
	  Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
	  Mpeg2MBInfoSetDMVFlag(mbi, 0);
	} else if (data == 3) {
	  Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_DUAL_PRIME);
	  Mpeg2MBInfoSetMVCount(mbi, 1);
	  Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	  Mpeg2MBInfoSetDMVFlag(mbi, 1);
	}
      } else {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
	Mpeg2MBInfoSetMVCount(mbi, 1);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
	Mpeg2MBInfoSetDMVFlag(mbi, 0);
      }
    } else {
      Bp_GetBits(bp, 2, data);
      total += 2;
      if (data == 1) {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_FIELD);
	Mpeg2MBInfoSetMVCount(mbi, 1);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	Mpeg2MBInfoSetDMVFlag(mbi, 0);
      } else if (data == 2) {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_16_X_8_MC);
	Mpeg2MBInfoSetMVCount(mbi, 2);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	Mpeg2MBInfoSetDMVFlag(mbi, 0);
      } else if (data == 3) {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_DUAL_PRIME);
	Mpeg2MBInfoSetMVCount(mbi, 1);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	Mpeg2MBInfoSetDMVFlag(mbi, 1);
      }
    }
  }

  if (pic_structure_code == STRUCTURE_CODE_FRAME) {
    if ((frame_pred_frame_flag == 0) &&
	(Mpeg2MBInfoGetIntra(mbi) || Mpeg2MBInfoGetPattern(mbi)))
      {
	Bp_GetBits(bp, 1, data);
	Mpeg2MBInfoSetDCTType(mbi, data);
	total += 1;
      } else {
	Mpeg2MBInfoSetDCTType(mbi, 0);
      }

    if (Mpeg2MBInfoGetIntra(mbi) && concealment_flag) {
      Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
      Mpeg2MBInfoSetMVCount(mbi, 1);
      Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
      Mpeg2MBInfoSetDMVFlag(mbi, 0);
    }
  } else {
    Mpeg2MBInfoSetDCTType(mbi, 0);
    if (Mpeg2MBInfoGetIntra(mbi) && concealment_flag) {
      Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_FIELD);
      Mpeg2MBInfoSetMVCount(mbi, 1);
      Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
      Mpeg2MBInfoSetDMVFlag(mbi, 0);
    }
  }      

  return total;
}
  
static int
Mpeg2DecodeBlockPattern(BitParser *bp, int *off, int chroma_format_code)
{
  unsigned int data;
  int cbp;

  Bp_PeekBits(bp, 9, data);
  cbp = block_pattern_420_value_table[data];
  *off = block_pattern_420_size_table[data];
  Bp_FlushBits(bp, *off);

  if (chroma_format_code == CHROMA_CODE_4_2_2) {
    cbp <<= 2;
    Bp_GetBits(bp, 2, data);
    *off += 2;
    cbp |= data;
  } else if (chroma_format_code == CHROMA_CODE_4_4_4) {
    cbp <<= 6;
    Bp_GetBits(bp, 6, data);
    *off += 6;
    cbp |= data;
  }
  return cbp;
}
  
static int
Mpeg2DecodeNonIntraBlock(BitParser *bp,
			 int block_num,
			 ScBlock *dest,
			 char *unzigzag,
			 unsigned char *qmatrix,
			 int qscale)
{
  int total = 0;
  int odd_flag;
  int num_ac_coeffs;
  int cur_idx;
  int idx;
  int value;
  int run, len, level;
  unsigned int data;
  short code; 
  int mode;
  
  cur_idx = 0;
  num_ac_coeffs = 0;

  dest->dc = 0;

  Bp_PeekBits(bp, 9, data);
  code = dct_first_coeff_table_zero[data];

  mode = DCT_COEFF_TABLE_MODE(code);
  len = DCT_COEFF_TABLE_LEN(code);
  Bp_FlushBits(bp, len);
  total += len;

  if (mode == DCT_COEFF_TABLE_NORMAL_MODE) {
    level = DCT_COEFF_TABLE_LEVEL(code);
    run = DCT_COEFF_TABLE_RUN(code);
  } else if (mode == DCT_COEFF_TABLE_MOREBITS_MODE) {
    short *next_table;
    int nbits = DCT_COEFF_TABLE_SUBTABLE_NBITS(code);
    int idx = DCT_COEFF_TABLE_SUBTABLE_IDX(code);

    next_table = dct_coeff_large_code_table_zero[idx];

    Bp_PeekBits(bp, nbits, data);
    code = next_table[data];

    len = DCT_COEFF_SUBTABLE_LEN(code);
    run = DCT_COEFF_SUBTABLE_RUN(code);
    level = DCT_COEFF_SUBTABLE_LEVEL(code);
    Bp_FlushBits(bp, len);
    total += len;
  } else if (mode == DCT_COEFF_TABLE_ESC_MODE) {
    Bp_GetBits(bp, 6, data);
    run = data;
    Bp_GetBits(bp, 12, data);
    level = data;
    if (level & 0x800) {
      level |= 0xfffff000;
    }
  } 
  
  cur_idx += run;
  idx = unzigzag[cur_idx];

  if (level < 0) {
    value = (((level*2)-1)*qmatrix[idx]*qscale)/32;
  } else {
    value = (((level*2)+1)*qmatrix[idx]*qscale)/32;
  }
  
  value = (value < -2048) ? -2048 : (value > 2047) ? 2047 : value;
  
  if (idx == 0) {
    dest->dc = value;
  } else {
    dest->index[num_ac_coeffs] = idx;
    dest->value[num_ac_coeffs] = value;
    num_ac_coeffs++;
  }
  odd_flag = value;
  cur_idx++;
  
  while(1) {
    Bp_PeekBits(bp, 9, data);
    code = dct_coeff_table_zero[data];

    mode = DCT_COEFF_TABLE_MODE(code);
    len = DCT_COEFF_TABLE_LEN(code);
    Bp_FlushBits(bp, len);
    total += len;

    if (mode == DCT_COEFF_TABLE_EOB_MODE) {
      break;
    } else if (mode == DCT_COEFF_TABLE_NORMAL_MODE) {
      level = DCT_COEFF_TABLE_LEVEL(code);
      run = DCT_COEFF_TABLE_RUN(code);
    } else if (mode == DCT_COEFF_TABLE_MOREBITS_MODE) {
      short *next_table;
      int nbits = DCT_COEFF_TABLE_SUBTABLE_NBITS(code);
      int idx = DCT_COEFF_TABLE_SUBTABLE_IDX(code);
      
      next_table = dct_coeff_large_code_table_zero[idx];
      
      Bp_PeekBits(bp, nbits, data);
      code = next_table[data];
      
      len = DCT_COEFF_SUBTABLE_LEN(code);
      run = DCT_COEFF_SUBTABLE_RUN(code);
      level = DCT_COEFF_SUBTABLE_LEVEL(code);
      Bp_FlushBits(bp, len);
      total += len;
    } else if (mode == DCT_COEFF_TABLE_ESC_MODE) {
      Bp_GetBits(bp, 6, data);
      run = data;
      Bp_GetBits(bp, 12, data);
      level = data;
      if (level & 0x800) {
	level |= 0xfffff000;
      }
    } 

    cur_idx += run;
    idx = unzigzag[cur_idx];
    
    if (level < 0) {
      value = (((level*2)-1)*qmatrix[idx]*qscale)/32;
    } else {
      value = (((level*2)+1)*qmatrix[idx]*qscale)/32;
    }
    
    value = (value < -2048) ? -2048 : (value > 2047) ? 2047 : value;
    
    dest->index[num_ac_coeffs] = idx;
    dest->value[num_ac_coeffs] = value;
    
    odd_flag ^= value;
    cur_idx++;
    num_ac_coeffs++;
  }

  /* Mismatch control */
  if (!(odd_flag & 0x1)) {
    if (num_ac_coeffs) {
      if (dest->index[num_ac_coeffs-1] == 63) {
	dest->value[num_ac_coeffs-1] ^= 0x1;
      } else {
	dest->value[num_ac_coeffs] = 1;
	dest->index[num_ac_coeffs] = 63;
	num_ac_coeffs++;
      }
    } else {
      dest->value[0] = 1;
      dest->index[0] = 63;
      num_ac_coeffs = 1;
    }
  }
  
  dest->numOfAC = num_ac_coeffs;
  return total;
}

static void
DeterminePredictionSources(short h, short v, int *np,
			   unsigned char **s1, 
			   unsigned char **s2, 
			   unsigned char **s3, 
			   unsigned char **s4, 
			   unsigned char *src,
			   int src_w)
{
  short ih, iv;
  
  ih = h >> 1;
  iv = v >> 1;
  
  if (h & 0x1) {
    if (v & 0x1) {
      /* Both on half pixel */
      *np = 4;
      *s1 = src+ih+(iv*src_w);
      *s2 = *s1 + 1;
      *s3 = *s1+src_w;
      *s4 = *s3 + 1;
    } else {
      /* Horiz. half pixel only */
      *np = 2;
      *s1 = src+ih+(iv*src_w);
      *s2 = *s1 + 1;
    }
  } else {
    if (v & 0x1) {
      /* Vert. half pixel only */
      *np = 2;
      *s1 = src+ih+(iv*src_w);
      *s2 = *s1+src_w;
    } else {
      /* Neither on half pixel */
      *np = 1;
      *s1 = src+ih+(iv*src_w);
    }
  }
}

static void
Block_Copy_16_X_8(unsigned char *s, int sw, unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    *d++ = *s++;     *d++ = *s++; 
    d += dw - 16;
    s += sw - 16;
  }
}

static void
Block_Copy_8_X_8(unsigned char *s, int sw, unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    d += dw - 8;
    s += sw - 8;
  }
}

static void
Block_Copy_8_X_4(unsigned char *s, int sw, unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<4; i++) {
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    *d++ = *s++;
    d += dw - 8;
    s += sw - 8;
  }
}

static void
Block_Avg2_16_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *d, int dw) 
{
  int i;
  
  for (i=0; i<8; i++) {
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;   *d++ = ((*s1++) + (*s2++) + 1) / 2;
    d += dw - 16;
    s1 += s1w - 16;
    s2 += s2w - 16;
  }
}

static void
Block_Avg3_16_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    d += dw - 16;
    s1 += s1w - 16;
    s2 += s2w - 16;
    s3 += s3w - 16;
  }
}

static void
Block_Avg4_16_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *d, int dw) 
{
  int i;
  
  for (i=0; i<8; i++) {
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    d += dw - 16;
    s1 += s1w - 16;
    s2 += s2w - 16;
    s3 += s3w - 16;
    s4 += s4w - 16;
  }
}
  
static void
Block_Avg5_16_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    d += dw - 16;
    s1 += s1w - 16;
    s2 += s2w - 16;
    s3 += s3w - 16;
    s4 += s4w - 16;
    s5 += s5w - 16;
  }
}

static void
Block_Avg6_16_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *s6, int s6w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    d += dw - 16;
    s1 += s1w - 16;
    s2 += s2w - 16;
    s3 += s3w - 16;
    s4 += s4w - 16;
    s5 += s5w - 16;
    s6 += s6w - 16;
  }
}

static void
Block_Avg8_16_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *s6, int s6w,
		  unsigned char *s7, int s7w,
		  unsigned char *s8, int s8w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    d += dw - 16;
    s1 += s1w - 16;
    s2 += s2w - 16;
    s3 += s3w - 16;
    s4 += s4w - 16;
    s5 += s5w - 16;
    s6 += s6w - 16;
    s7 += s7w - 16;
    s8 += s8w - 16;
  }
}

static void
Block_Avg2_8_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *d, int dw) 
{
  int i;
  
  for (i=0; i<8; i++) {
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
  }
}

static void
Block_Avg2_8_X_4(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *d, int dw) 
{
  int i;
  
  for (i=0; i<4; i++) {
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    *d++ = ((*s1++) + (*s2++) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
  }
}

static void
Block_Avg3_8_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
  }
}

static void
Block_Avg3_8_X_4(unsigned char *s1, int s1w, 
		 unsigned char *s2, int s2w,
		 unsigned char *s3, int s3w,
		 unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<4; i++) {
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + 1)/2) + (*s3++) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
  }
}

static void
Block_Avg4_8_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *d, int dw) 
{
  int i;
  
  for (i=0; i<8; i++) {
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
  }
}

static void
Block_Avg4_8_X_4(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *d, int dw) 
{
  int i;
  
  for (i=0; i<4; i++) {
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) + 2) / 4;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
  }
}
  
static void
Block_Avg5_8_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
    s5 += s5w - 8;
  }
}

static void
Block_Avg5_8_X_4(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<4; i++) {
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (*s5++) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
    s5 += s5w - 8;
  }
}

static void
Block_Avg6_8_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *s6, int s6w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
    s5 += s5w - 8;
    s6 += s6w - 8;
  }
}

static void
Block_Avg6_8_X_4(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *s6, int s6w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<4; i++) {
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    *d++ = ((((*s1++) + (*s2++) + (*s3++) + (*s4++)+2)/4) + (((*s5++) + (*s6++) + 1)/2) + 1) / 2;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
    s5 += s5w - 8;
    s6 += s6w - 8;
  }
}

static void
Block_Avg8_8_X_8(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *s6, int s6w,
		  unsigned char *s7, int s7w,
		  unsigned char *s8, int s8w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
    s5 += s5w - 8;
    s6 += s6w - 8;
    s7 += s7w - 8;
    s8 += s8w - 8;
  }
}

static void
Block_Avg8_8_X_4(unsigned char *s1, int s1w, 
		  unsigned char *s2, int s2w,
		  unsigned char *s3, int s3w,
		  unsigned char *s4, int s4w,
		  unsigned char *s5, int s5w,
		  unsigned char *s6, int s6w,
		  unsigned char *s7, int s7w,
		  unsigned char *s8, int s8w,
		  unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<4; i++) {
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    *d++ = ((*s1++) + (*s2++) + (*s3++) + (*s4++) +
	    (*s5++) + (*s6++) + (*s7++) + (*s8++) + 4) / 8;
    d += dw - 8;
    s1 += s1w - 8;
    s2 += s2w - 8;
    s3 += s3w - 8;
    s4 += s4w - 8;
    s5 += s5w - 8;
    s6 += s6w - 8;
    s7 += s7w - 8;
    s8 += s8w - 8;
  }
}

static void 
BlockShortToByteCopy(short *temp, unsigned char *dest, int dest_w)
{
  int i;

  for (i=0; i<8; i++) {
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    *dest++ = CROP(*temp++);
    dest += dest_w - 8;
  }
}

static void
BlockShortToByteAdd(short *t, unsigned char *d, int dw) 
{
  int i;

  for (i=0; i<8; i++) {
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    *d = CROP(((*d) + (*t++)));    d++;
    d += dw - 8;
  }
}


int
Mpeg2ScPFieldToByte(ScImage *sc_image,
		    Mpeg2MBInfoArray *mbi_array,
		    ByteImage *prev_top,
		    ByteImage *prev_bot,
		    ByteImage *dest_image)
{
  int mb_width, mb_height;
  int mby, mbx;
  Mpeg2MBInfo *mbi;
  ScBlock *scb;
  short temp[64];
  unsigned char *top_pred;
  unsigned char *bot_pred;
  int dest_w;
  int mb_addr;
  unsigned char *dest;
  short v0_h, v0_v;
  short dual_top_h, dual_top_v, dual_bot_h, dual_bot_v;
  int num_preds,  num_dual_preds;
  unsigned char *s1, *s2, *s3, *s4, *s5, *s6, *s7, *s8;
  int s1_w, s2_w, s3_w, s4_w, s5_w, s6_w, s7_w, s8_w;
  int ix, iy;
  int mb_h_size, mb_v_size;
  int sc_width, sc_height;

  mb_width = Mpeg2MBInfoArrayGetWidth(mbi_array);
  mb_height = Mpeg2MBInfoArrayGetHeight(mbi_array);

  sc_width = sc_image->width;
  sc_height = sc_image->height;

  /* Using mb width and height and sc width and height, 
   * infer chrominance formatting and set mb_h_size and
   * mb_v_size as appropriate.
   */
  if (mb_width == sc_width) {
    mb_h_size = 1;
  } else if (mb_width*2 == sc_width) {
    mb_h_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }
  if (mb_height == sc_height) {
    mb_v_size = 1;
  } else if (mb_height*2 == sc_height) {
    mb_v_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }

  dest_w = dest_image->parentWidth;

  for (mby = 0; mby < mb_height; mby++) {
    for (mbx = 0; mbx < mb_width; mbx++) {
      /* Get macroblock info */

      mb_addr = mby * mb_width + mbx;
      mbi = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);
      
      /* Get pointer to appropriate block in ScImage */
      
      scb = &(sc_image->firstBlock[sc_image->parentWidth*(mby*mb_v_size)
				  +(mbx*mb_h_size)]);

      /* Get pointer to appropriate dest byte */
      dest = &(dest_image->firstByte[((dest_image->parentWidth*
				       (8*mb_v_size*mby))+
				      (8*mb_h_size*mbx))]);

      if (!scb->intracoded) {

	/* Get pointer to corresponding point in prediction fields */
	top_pred = &(prev_top->firstByte[((prev_top->parentWidth*
					     (8*mb_v_size*mby))+
					    (8*mb_h_size*mbx))]);
	bot_pred = &(prev_bot->firstByte[((prev_bot->parentWidth*
					     (8*mb_v_size*mby))+
					    (8*mb_h_size*mbx))]);

	/* Calculate all of the required prediction pointers and how many
	 * there are (could be up to 8!!!)
	 */

	if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FIELD_MOTION_TYPE_FIELD) {
	  /* Simple field prediction. Could be 1, 2, or 4, prediction pointers
	   * depending on half-pixel granularity. 
	   */
	  v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	  v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);

	  if (mb_h_size == 1) {
	    v0_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    v0_v >>= 1;
	  }

	  if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 0)) {
	    DeterminePredictionSources(v0_h, v0_v, &num_preds, 
				       &s1, &s2, &s3, &s4,
				       top_pred,
				       prev_top->parentWidth);
	    s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	  } else {
	    DeterminePredictionSources(v0_h, v0_v, &num_preds, 
				       &s1, &s2, &s3, &s4,
				       bot_pred,
				       prev_bot->parentWidth);
	    s1_w = s2_w = s3_w = s4_w = prev_bot->parentWidth;
	  }

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w);
	    } else {
	      Do_8_X_16_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				   dest, dest_w);
	    }
	  } else {
	    Do_16_X_16_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w);
	  }
	} else if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FIELD_MOTION_TYPE_16_X_8_MC) {
	  v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	  v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);

	  if (mb_h_size == 1) {
	    v0_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    v0_v >>= 1;
	  }
	
	  if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 0)) {
	    DeterminePredictionSources(v0_h, v0_v, &num_preds,
				       &s1, &s2, &s3, &s4,
				       top_pred,
				       prev_top->parentWidth);
	    s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	  } else {
	    DeterminePredictionSources(v0_h, v0_v, &num_preds,
				       &s1, &s2, &s3, &s4,
				       bot_pred,
				       prev_bot->parentWidth);
	    s1_w = s2_w = s3_w = s4_w = prev_bot->parentWidth;
	  }

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				 s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				 NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				 dest, dest_w);
	  }
	  
	  v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 0);
	  v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 0);
	
	  if (mb_h_size == 1) {
	    v0_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    v0_v >>= 1;
	  }

	  if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 1, 0)) {
	    DeterminePredictionSources(v0_h, v0_v, &num_preds,
				       &s1, &s2, &s3, &s4,
				       top_pred,
				       prev_top->parentWidth);
	    s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	  } else {
	    DeterminePredictionSources(v0_h, v0_v, &num_preds,
				       &s1, &s2, &s3, &s4,
				       bot_pred,
 				       prev_bot->parentWidth);
	    s1_w = s2_w = s3_w = s4_w = prev_bot->parentWidth;
	  }

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1+(s1_w*4), s1_w, s2+(s2_w*4), s2_w, 
				  s3+(s3_w*4), s3_w, s4+(s4_w*4), s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest+(dest_w*4), dest_w);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				  s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, 
				  s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest+(dest_w*8), dest_w);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				 s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, 
				 s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w,
				 NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				 dest+(dest_w*8), dest_w);
	  }
	} else if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FIELD_MOTION_TYPE_DUAL_PRIME) {
	  dual_top_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	  dual_top_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);
	  dual_bot_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 0);
	  dual_bot_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 0);
	  
	  if (mb_h_size == 1) {
	    dual_top_h >>= 1;
	    dual_bot_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    dual_top_v >>= 1;
	    dual_bot_v >>= 1;
	  }
	  
	  DeterminePredictionSources(dual_top_h, dual_top_v, &num_preds, 
				     &s1, &s2, &s3, &s4,
				     top_pred,
				     prev_top->parentWidth);
	  s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	  DeterminePredictionSources(dual_bot_h, dual_bot_v, &num_dual_preds,
				     &s5, &s6, &s7, &s8,
				     bot_pred,
				     prev_bot->parentWidth);
	  s5_w = s6_w = s7_w = s8_w = prev_bot->parentWidth;
	  
	  if (num_preds == 1) {
	    if (num_dual_preds == 2) {
	      s3 = s1;
	      s1 = s5;
	      s2 = s6;
	      s3_w = s1_w;
	      s1_w = s5_w;
	      s2_w = s6_w;
	    } else if (num_dual_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1;
	      t1_w = s1_w;
	      s1 = s5;
	      s2 = s6;
	      s3 = s7;
	      s4 = s8;
	      s5 = t1;
	      s1_w = s5_w;
	      s2_w = s6_w;
	      s3_w = s7_w;
	      s4_w = s8_w;
	      s5_w = t1_w;
	    } else {
	      s2 = s5;
	      s2_w = s5_w;
	    }
	  } else if (num_preds == 2) {
	    if (num_dual_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;

	      t1 = s1;
	      t2 = s2;
	      t1_w = s1_w;
	      t2_w = s2_w;

	      s1 = s5;
	      s2 = s6;
	      s3 = s7;
	      s4 = s8;
	      s5 = t1;
	      s6 = t2;
	      s1_w = s5_w;
	      s2_w = s6_w;
	      s3_w = s7_w;
	      s4_w = s8_w;
	      s5_w = t1_w;
	      s6_w = t2_w;
	    } else {
	      s3 = s5;
	      s4 = s6;
	      s5 = s7;
	      s6 = s8;
	      s3_w = s5_w;
	      s4_w = s6_w;
	      s5_w = s7_w;
	      s6_w = s8_w;
	    }
	  }
	  num_preds += num_dual_preds;
	  
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	    } else {
	      Do_8_X_16_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				   dest, dest_w);
	    }
	  } else {
	    Do_16_X_16_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	  }
	}
      }

      if (!scb->skipMB) {

	for (iy=0; iy<mb_v_size; iy++) {
	  if (iy == 1) {
	    scb += sc_image->parentWidth;
	    dest += dest_image->parentWidth*8;
	  }
	  for (ix=0; ix<mb_h_size; ix++) {
	    /* Adjust sc block pointer and dest pointers for
	     * which block in macroblock we are doing.
	     * This is a bit tricky due to different chrominance
	     * formats.
	     */
	    if (ix == 1) {
	      scb += 1;
	      dest += 8;
	    }
	    if (!scb->skipBlock) {
	      if (scb->intracoded) {
		/* No prediction, just DCT */
		DO_IDCT(scb, temp);
		BlockShortToByteCopy(temp, dest, dest_w);
	      } else {
		/* Both prediction and DCT */
		DO_IDCT(scb, temp);
		BlockShortToByteAdd(temp, dest, dest_w);
	      }
	    }
	  }
	  if (ix == 2) {
	    scb -= 1;
	    dest -= 8;
	  }
	}
      }
    }
  }
  return 1;
}  
  
int
Mpeg2ParseB(BitParser *bp,
	    Mpeg2SeqHdr *sh,
	    Mpeg2PicHdr *fh,
	    ScImage *y,
	    ScImage *u,
	    ScImage *v,
	    Mpeg2MBInfoArray *mbi_array,
	    int field_flag)
{
  unsigned int data;
  int intra_dct_coeff_table_flag;
  unsigned char *unzigzag;
  int block_count;
  unsigned char *lum_i_q_matrix, *chr_i_q_matrix;
  unsigned char *lum_non_i_q_matrix, *chr_non_i_q_matrix;
  int q_scale_type;
  int qscale_code;
  int qscale;
  int dc_pred_value;
  int total = 0;
  int dc_quant_scale;
  int off;
  int i;
  int last_mb_addr;
  unsigned int byte_start;
  unsigned int byte_end;
  unsigned int bit_count_start;
  unsigned int bit_count_end;

  byte_start = (unsigned int) bp->offsetPtr;
  bit_count_start = bp->bitCount;

  Bp_InByteAlign(bp);
  Bp_GetInt(bp, data);
  
  total = 32;
  
  /*
   * Choose appropriate Huffman code table for
   * intra blocks 
   */

  if (Mpeg2PicHdrGetIntraVLCFormat(fh)) {
    /* Set to table 1. */
    intra_dct_coeff_table_flag = 1;
  } else {
    intra_dct_coeff_table_flag = 0;
  }

  if (Mpeg2PicHdrGetAltScan(fh)) {
    unzigzag = alt_unzigzag_pattern;
  } else {
    unzigzag = unzigzag_pattern;
  }

  /* Determine number of blocks per macroblock */
  
  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
    block_count = 6;
  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
    block_count = 8;    
  } else {
    /* 4:4:4 is only other choice */
    block_count = 12;
  }

  /* Get pointers to quant matrices */
  
  lum_i_q_matrix = Mpeg2SeqHdrGetLumIQuantMatrix(sh);
  chr_i_q_matrix = Mpeg2SeqHdrGetChrIQuantMatrix(sh);
  lum_non_i_q_matrix = Mpeg2SeqHdrGetLumNonIQuantMatrix(sh);
  chr_non_i_q_matrix = Mpeg2SeqHdrGetChrNonIQuantMatrix(sh);

  /* Determine quant scale type */
  q_scale_type = Mpeg2PicHdrGetQScaleType(fh);

  /* Determine the dc prediction reset value
   * and dc quant scale
   */
  
  switch (Mpeg2PicHdrGetIntraDCPrecision(fh)) {
  case 0:
    dc_pred_value = 128;
    dc_quant_scale = 3;
    break;
  case 1:
    dc_pred_value = 256;
    dc_quant_scale = 2;
    break;
  case 2:
    dc_pred_value = 512;
    dc_quant_scale = 1;
    break;
  case 3:
    dc_pred_value = 1024;
    dc_quant_scale = 0;
    break;
  default:
    /* Totally bogus, but eliminates initialization warnings. */
    dc_pred_value = 0;
    dc_quant_scale = 0;
  }
  
  last_mb_addr = -1;
  
  while ((data >= MPEG2_SLICE_MIN_START_CODE) &&
	 (data <= MPEG2_SLICE_MAX_START_CODE)) {
    short dc_lum_pred;
    short dc_cr_pred;
    short dc_cb_pred;
    short *dc_pred;
    int slice_vertical_pos;
    int intra_slice;
    int prev_mb_addr;
    int mb_addr;
    int mb_row, mb_col;
    int block_pattern;
    Mpeg2MBInfo *last_mb_info = NULL;

    /* Reset the motion vector predictors b/c
     * at beginning of slice;
     */

    Mpeg2PicHdrResetPMV(fh);

    /* Reset dc predictors */

    dc_lum_pred = dc_cr_pred = dc_cb_pred = dc_pred_value;
    
    slice_vertical_pos = data & 0xff;

    if (Mpeg2SeqHdrGetHeight(sh) > 2800) {
      /* Get slice vertical pos extension */
      Bp_GetBits(bp, 3, data);
      total += 3;
      slice_vertical_pos += (data << 7);
    }
  
    slice_vertical_pos--;
    
    if (Mpeg2SeqHdrScalableExtPresent(sh)) {
      /* XXX - scalable extensions not implemented */
    }
    
    /* Get and calculate quantizer */
    Bp_GetBits(bp, 5, data);
    total += 5;
    qscale_code = data;

    if (q_scale_type == MPEG2_NON_LINEAR_Q_SCALE) {
      qscale = nonlinear_q_scale_table[qscale_code];
    } else {
      qscale = qscale_code * 2;
    }
    
    /* Check for intra slice info */
    intra_slice = 0;
    Bp_GetBits(bp, 1, data);
    total += 1;
    if (data) {
      /* Intra slice info is there. */
      Bp_GetBits(bp, 1, data);
      total += 1;
      intra_slice = data;
      
      /* Parse off 7 reserved bits. */
      Bp_FlushBits(bp, 7);
      total += 7;

      /* Parse off extra slice info. */
      Bp_GetBits(bp, 1, data);
      total += 1;
      while(data) {
	Bp_FlushBits(bp, 8);
	total += 8;
	Bp_GetBits(bp, 1, data);
	total += 1;
      }
    }

    Bp_LargePeekBits(bp, 23, data);
    
    prev_mb_addr = (slice_vertical_pos * Mpeg2SeqHdrGetMBWidth(sh)) - 1;

    while (data != 0) {
      Mpeg2MBInfo *curr_mb_info;
      ScBlock *dest;

      mb_addr = prev_mb_addr + Mpeg2DecodeMBAddrIncrement(bp, &off);
      total += off;

      prev_mb_addr = mb_addr;

      /* Processes skipped macroblocks if any. */
      last_mb_addr++;
      if (mb_addr != last_mb_addr) {
	
	/* Reset predictors because of skipped macroblocks.
	 * Note: PMV is not reset because this is a B picture
	 */

	dc_lum_pred = dc_cr_pred = dc_cb_pred = dc_pred_value;

	while (last_mb_addr != mb_addr) {
	  /* Do per skip macroblock processing here. */
	  mb_row = last_mb_addr / Mpeg2SeqHdrGetMBWidth(sh);
	  mb_col = last_mb_addr % Mpeg2SeqHdrGetMBWidth(sh);
      
	  curr_mb_info = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, last_mb_addr);


	  Mpeg2MBInfoSetQuant(curr_mb_info, 0);
	  Mpeg2MBInfoSetMFwd(curr_mb_info, Mpeg2MBInfoGetMFwd(last_mb_info));
	  Mpeg2MBInfoSetMBwd(curr_mb_info, Mpeg2MBInfoGetMBwd(last_mb_info));
	  Mpeg2MBInfoSetPattern(curr_mb_info, 0);
	  Mpeg2MBInfoSetIntra(curr_mb_info, 0);
	  Mpeg2MBInfoSetDCTType(curr_mb_info, 0);

	  if (field_flag) {
	    Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FIELD_MOTION_TYPE_FIELD);
	    Mpeg2MBInfoSetMVFormat(curr_mb_info, MV_FORMAT_FIELD);
	    if (Mpeg2PicHdrGetStructureCode(fh) == STRUCTURE_CODE_TOP) {
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 0);
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 1, 0);
	    } else {
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 1);
	      Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 1, 1);
	    }
	  } else {
	    Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FRAME_MOTION_TYPE_FRAME);
	    Mpeg2MBInfoSetMVFormat(curr_mb_info, MV_FORMAT_FRAME);

	    Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 0, 0);
	    Mpeg2MBInfoSetMVFieldSelect(curr_mb_info, 0, 1, 0);
	  }

	  Mpeg2MBInfoSetDMVFlag(curr_mb_info, 0);
	  Mpeg2MBInfoSetMVCount(curr_mb_info, 1);	  
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 0, Mpeg2MBInfoGetMVectorHorizComp(last_mb_info, 0, 0));
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 0, Mpeg2MBInfoGetMVectorVertComp(last_mb_info, 0, 0));
	  Mpeg2MBInfoSetMVectorHorizComp(curr_mb_info, 0, 1, Mpeg2MBInfoGetMVectorHorizComp(last_mb_info, 0, 1));
	  Mpeg2MBInfoSetMVectorVertComp(curr_mb_info, 0, 1, Mpeg2MBInfoGetMVectorVertComp(last_mb_info, 0, 1));

	  dest = ScGetBlock(y, (mb_col * 2), (mb_row * 2));
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  dest = ScGetBlock(y, (mb_col * 2)+1, (mb_row * 2));
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  dest = ScGetBlock(y, (mb_col * 2), (mb_row * 2)+1);
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  dest = ScGetBlock(y, (mb_col * 2)+1, (mb_row * 2)+1);
	  dest->skipMB = 1;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	  dest->intracoded = 0;

	  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
	    dest = ScGetBlock(u, mb_col, mb_row);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(v, mb_col, mb_row);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
	    dest = ScGetBlock(u, mb_col, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, mb_col, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, mb_col, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, mb_col, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	  } else {
	    dest = ScGetBlock(u, (mb_col*2), (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, (mb_col*2), (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, (mb_col*2)+1, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(u, (mb_col*2)+1, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, (mb_col*2), (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, (mb_col*2), (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;

	    dest = ScGetBlock(v, (mb_col*2)+1, (mb_row*2));
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	    
	    dest = ScGetBlock(v, (mb_col*2)+1, (mb_row*2)+1);
	    dest->skipMB = 1;
	    dest->skipBlock = 1;
	    dest->dc = 0;
	    dest->numOfAC = 0;
	    dest->intracoded = 0;
	  }

	  last_mb_addr++;
	}
      }

       
      mb_row = mb_addr / Mpeg2SeqHdrGetMBWidth(sh);
      mb_col = mb_addr % Mpeg2SeqHdrGetMBWidth(sh);
      
      curr_mb_info = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);

      /* Decode macroblock mode */
      
      if (Mpeg2SeqHdrScalableExtPresent(sh)) {
	/* Decode macroblock mode based on scalability.
	 * Not yet supported. 
	 */
      } else {
	total += Mpeg2DecodeBNonScalableMBMode(bp, curr_mb_info,
					       Mpeg2PicHdrGetConcealmentMVFlag(fh),
					       Mpeg2PicHdrGetStructureCode(fh),
					       Mpeg2PicHdrGetFramePredFrameDCT(fh)
					       );
      }	

      /* If quant code encoded, parse it */
      if (Mpeg2MBInfoGetQuant(curr_mb_info)) {
	Bp_GetBits(bp, 5, data);
	total += 5;
	qscale_code = data;
	
	if (q_scale_type == MPEG2_NON_LINEAR_Q_SCALE) {
	  qscale = nonlinear_q_scale_table[qscale_code];
	} else {
	  qscale = qscale_code * 2;
	}
      }

      /* Save qscale for posterity */
      Mpeg2MBInfoSetQuant(curr_mb_info, qscale);
      
      if (Mpeg2MBInfoGetMFwd(curr_mb_info) ||
	  (Mpeg2MBInfoGetIntra(curr_mb_info) && 
	   Mpeg2PicHdrGetConcealmentMVFlag(fh))) {
	total += Mpeg2DecodeMotionVectors(fh, bp, 0, curr_mb_info);
	
	if (Mpeg2MBInfoGetIntra(curr_mb_info) &&
	    Mpeg2PicHdrGetConcealmentMVFlag(fh)) {
	  
	  if (field_flag) {
	    Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FIELD_MOTION_TYPE_FIELD);
	  } else {
	    Mpeg2MBInfoSetMotionType(curr_mb_info, MPEG2_FRAME_MOTION_TYPE_FRAME);
	  }

	  /* Flush marker bit */
	  Bp_FlushBits(bp, 1);
	  total += 1;
	}
      }
      if (Mpeg2MBInfoGetMBwd(curr_mb_info)) {
	total += Mpeg2DecodeMotionVectors(fh, bp, 1, curr_mb_info);
      }


      /* Update motion vector predictors */
      if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	if (!Mpeg2PicHdrGetConcealmentMVFlag(fh)) {
	  Mpeg2PicHdrResetPMV(fh);
	} else {
	  Mpeg2PicHdrUpdatePMV1Fwd(fh);
	}
      } else {
	/* Reset predictors because non-intra block coded. */
	dc_lum_pred = dc_cr_pred = dc_cb_pred = dc_pred_value;	    

	if ((Mpeg2MBInfoGetMotionType(curr_mb_info) == 
	     MPEG2_FIELD_MOTION_TYPE_FIELD) || 
	    (Mpeg2MBInfoGetMotionType(curr_mb_info) ==
	     MPEG2_FRAME_MOTION_TYPE_FRAME)) {
	  if (Mpeg2MBInfoGetMFwd(curr_mb_info)) {
	    Mpeg2PicHdrUpdatePMV1Fwd(fh);
	  }
	  if (Mpeg2MBInfoGetMBwd(curr_mb_info)) {
	    Mpeg2PicHdrUpdatePMV1Bwd(fh);
	  }
	} else if ((Mpeg2MBInfoGetMotionType(curr_mb_info) == 
		    MPEG2_FIELD_MOTION_TYPE_DUAL_PRIME) ||
		   (Mpeg2MBInfoGetMotionType(curr_mb_info) ==
		    MPEG2_FRAME_MOTION_TYPE_DUAL_PRIME)) {
	  if (Mpeg2MBInfoGetMFwd(curr_mb_info)) {
	    Mpeg2PicHdrUpdatePMV1Fwd(fh);
	  }
	} 
      }

      if (Mpeg2MBInfoGetPatternFlag(curr_mb_info)) {
	block_pattern = Mpeg2DecodeBlockPattern(bp, &off, Mpeg2SeqHdrGetChromaFormatCode(sh));
	total += off;
      } else {
	if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	  block_pattern = 0xffff;
	} else {
	  block_pattern = 0;
	}
      }

      /* Save pattern for posterity */
      Mpeg2MBInfoSetPattern(curr_mb_info, block_pattern);

      for (i=0; i<block_count; i++) {
	int block_col, block_row;
	int lum_flag;
	unsigned char *qmatrix;

	if (i < 4) {
	  block_col = (mb_col * 2)+(i%2);
	  block_row = (mb_row * 2)+(i/2);
	  dest = ScGetBlock(y, block_col, block_row);
	  lum_flag = 1;
	  dc_pred = &(dc_lum_pred);
	  if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	    qmatrix = lum_i_q_matrix;
	  } else {
	    qmatrix = lum_non_i_q_matrix;
	  }
	} else {
	  if (Mpeg2MBInfoGetIntra(curr_mb_info)) {
	    qmatrix = chr_i_q_matrix;
	  } else {
	    qmatrix = chr_non_i_q_matrix;
	  }
	  if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_0) {
	    if (i == 4) {
		dest = ScGetBlock(u, mb_col, mb_row);
		dc_pred =&(dc_cb_pred);
	    } else {
	      dest = ScGetBlock(v, mb_col, mb_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  } else if (Mpeg2SeqHdrGetChromaFormatCode(sh) == CHROMA_CODE_4_2_2) {
	    if ((i % 2) == 0 ) {
	      block_row = (mb_row * 2) + ((i-4)/2);
	      dest = ScGetBlock(u, mb_col, block_row);
	      dc_pred = &(dc_cb_pred);
	    } else {
	      block_row = (mb_row * 2) + ((i-5)/2);
	      dest = ScGetBlock(v, mb_col, block_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  } else {
	    /* 4:4:4 case */
	    if ((i%2) == 0) {
	      block_row = (mb_row * 2) + (((i - 4)/2)%2);
	      block_col = (mb_col * 2) + (((i-4)/2)/2);
	      dest = ScGetBlock(u, block_col, block_row);
	      dc_pred =&(dc_cb_pred);
	    } else {
	      block_row = (mb_row * 2) + (((i - 5)/2)%2);
	      block_col = (mb_col * 2) + (((i-5)/2)/2);
	      dest = ScGetBlock(v, block_col, block_row);
	      dc_pred =&(dc_cr_pred);
	    }
	  }
	  lum_flag = 0;
	}
	
	if (block_pattern & (1 << (block_count - i -1))) {
	  dest->intracoded = Mpeg2MBInfoGetIntra(curr_mb_info);
	  dest->skipMB = 0;
	  dest->skipBlock = 0;
	  
	  if (dest->intracoded) {
	    total += Mpeg2DecodeIntraBlock(bp, i, dest, lum_flag,
					   dc_pred, 
					   intra_dct_coeff_table_flag,
					   unzigzag, dc_quant_scale,
					   qmatrix, qscale);
	  } else {
	    total += Mpeg2DecodeNonIntraBlock(bp, i, dest,
					      unzigzag, 
					      qmatrix,
					      qscale);

	  }
	} else {
	  /* Do skipped blocks here. */
	  dest->intracoded = 0;
	  dest->skipMB = 0;
	  dest->skipBlock = 1;
	  dest->dc = 0;
	  dest->numOfAC = 0;
	}
      }
      last_mb_info = curr_mb_info;

      Bp_LargePeekBits(bp, 23, data);
    }

    Bp_InByteAlign(bp);
    data = NextStartCode(bp, &off);
    total += (off * 8);
  }
  Bp_RestoreInt(bp);
  total -= 32;

  byte_end = (unsigned int) bp->offsetPtr;
  bit_count_end = bp->bitCount;

  return (((byte_end - byte_start) * 8) + bit_count_start - bit_count_end)/8;
  
}
  
static int
Mpeg2DecodeBNonScalableMBMode(BitParser *bp,
			      Mpeg2MBInfo *mbi,
			      int concealment_flag,
			      int pic_structure_code,
			      int frame_pred_frame_flag)
{
  unsigned int data;
  int total;
  unsigned char mb_mode_code;
  int mb_mode_code_size;

  Bp_PeekBits(bp, 6, data);

  mb_mode_code = b_non_scalable_mode_table[data];
  mb_mode_code_size = b_non_scalable_mode_size_table[data];
  Bp_FlushBits(bp, mb_mode_code_size);
  total = mb_mode_code_size;

  if (mb_mode_code & MB_MODE_BIT_QUANT) {
    Mpeg2MBInfoSetQuant(mbi, 1);
  } else {
    Mpeg2MBInfoSetQuant(mbi, 0);
  }

  if (mb_mode_code & MB_MODE_BIT_MFWD) {
    Mpeg2MBInfoSetMFwd(mbi, 1);
  } else {
    Mpeg2MBInfoSetMFwd(mbi, 0);
  }

  if (mb_mode_code & MB_MODE_BIT_MBWD) {
    Mpeg2MBInfoSetMBwd(mbi, 1);
  } else {
    Mpeg2MBInfoSetMBwd(mbi, 0);
  }

  if (mb_mode_code & MB_MODE_BIT_PATTERN) {
    Mpeg2MBInfoSetPattern(mbi, 1);
  } else {
    Mpeg2MBInfoSetPattern(mbi, 0);
  }

  if (mb_mode_code & MB_MODE_BIT_INTRA) {
    Mpeg2MBInfoSetIntra(mbi, 1);
  } else {
    Mpeg2MBInfoSetIntra(mbi, 0);
  }

  if (Mpeg2MBInfoGetMFwd(mbi) || Mpeg2MBInfoGetMBwd(mbi)) {
    if (pic_structure_code == STRUCTURE_CODE_FRAME) {
      if (!frame_pred_frame_flag) {
	Bp_GetBits(bp, 2, data);
	total += 2;
	if (data == 1) {
	  Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FIELD);
	  Mpeg2MBInfoSetMVCount(mbi, 2);
	  Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	  Mpeg2MBInfoSetDMVFlag(mbi, 0);
	} else if (data == 2) {
	  Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
	  Mpeg2MBInfoSetMVCount(mbi, 1);
	  Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
	  Mpeg2MBInfoSetDMVFlag(mbi, 0);
	} else if (data == 3) {
	  Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_DUAL_PRIME);
	  Mpeg2MBInfoSetMVCount(mbi, 1);
	  Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	  Mpeg2MBInfoSetDMVFlag(mbi, 1);
	}
      } else {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
	Mpeg2MBInfoSetMVCount(mbi, 1);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
	Mpeg2MBInfoSetDMVFlag(mbi, 0);
      }
    } else {
      Bp_GetBits(bp, 2, data);
      total += 2;
      if (data == 1) {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_FIELD);
	Mpeg2MBInfoSetMVCount(mbi, 1);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	Mpeg2MBInfoSetDMVFlag(mbi, 0);
      } else if (data == 2) {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_16_X_8_MC);
	Mpeg2MBInfoSetMVCount(mbi, 2);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	Mpeg2MBInfoSetDMVFlag(mbi, 0);
      } else if (data == 3) {
	Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_DUAL_PRIME);
	Mpeg2MBInfoSetMVCount(mbi, 1);
	Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
	Mpeg2MBInfoSetDMVFlag(mbi, 1);
      }
    }
  }
  
  if (pic_structure_code == STRUCTURE_CODE_FRAME) {
    if ((frame_pred_frame_flag == 0) &&
	(Mpeg2MBInfoGetIntra(mbi) || Mpeg2MBInfoGetPattern(mbi)))
      {
	Bp_GetBits(bp, 1, data);
	Mpeg2MBInfoSetDCTType(mbi, data);
	total += 1;
      } else {
	Mpeg2MBInfoSetDCTType(mbi, 0);
      }
    if (Mpeg2MBInfoGetIntra(mbi) && concealment_flag) {
      Mpeg2MBInfoSetMotionType(mbi, MPEG2_FRAME_MOTION_TYPE_FRAME);
      Mpeg2MBInfoSetMVCount(mbi, 1);
      Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FRAME);
      Mpeg2MBInfoSetDMVFlag(mbi, 0);
    }
  } else {
    Mpeg2MBInfoSetDCTType(mbi, 0);
    if (Mpeg2MBInfoGetIntra(mbi) && concealment_flag) {
      Mpeg2MBInfoSetMotionType(mbi, MPEG2_FIELD_MOTION_TYPE_FIELD);
      Mpeg2MBInfoSetMVCount(mbi, 1);
      Mpeg2MBInfoSetMVFormat(mbi, MV_FORMAT_FIELD);
      Mpeg2MBInfoSetDMVFlag(mbi, 0);
    }
  }      
  
  return total;
}

int
Mpeg2ScBFieldToByte(ScImage *sc_image,
		    Mpeg2MBInfoArray *mbi_array,
		    ByteImage *prev_top,
		    ByteImage *prev_bot,
		    ByteImage *future_top,
		    ByteImage *future_bot,
		    ByteImage *dest_image)
{
  int mb_width, mb_height;
  int mby, mbx;
  Mpeg2MBInfo *mbi;
  ScBlock *scb;
  short temp[64];
  unsigned char *prev_top_ptr;
  unsigned char *prev_bot_ptr;
  unsigned char *future_top_ptr;
  unsigned char *future_bot_ptr;
  int dest_w;
  int mb_addr;
  unsigned char *dest;
  short v0_h, v0_v;
  int num_preds, num_fwd_preds, num_bwd_preds;
  unsigned char *s1, *s2, *s3, *s4, *s5, *s6, *s7, *s8;
  int s1_w, s2_w, s3_w, s4_w, s5_w, s6_w, s7_w, s8_w;
  int ix, iy;
  int sc_width, sc_height, mb_h_size, mb_v_size;


  mb_width = Mpeg2MBInfoArrayGetWidth(mbi_array);
  mb_height = Mpeg2MBInfoArrayGetHeight(mbi_array);

  sc_width = sc_image->width;
  sc_height = sc_image->height;

  /* Using mb width and height and sc width and height, 
   * infer chrominance formatting and set mb_h_size and
   * mb_v_size as appropriate.
   */
  if (mb_width == sc_width) {
    mb_h_size = 1;
  } else if (mb_width*2 == sc_width) {
    mb_h_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }
  if (mb_height == sc_height) {
    mb_v_size = 1;
  } else if (mb_height*2 == sc_height) {
    mb_v_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }

  dest_w = dest_image->parentWidth;

  for (mby = 0; mby < mb_height; mby++) {
    for (mbx = 0; mbx < mb_width; mbx++) {
      /* Get macroblock info */

      mb_addr = mby * mb_width + mbx;
      mbi = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);
      
      /* Get pointer to appropriate block in ScImage */
      
      scb = &(sc_image->firstBlock[sc_image->parentWidth*(mb_v_size*mby)+
				  (mb_h_size*mbx)]);

      /* Get pointer to appropriate dest byte */
      dest = &(dest_image->firstByte[((dest_image->parentWidth*
				       (8*mb_v_size*mby))+
				      (8*mb_h_size*mbx))]);

      if (!scb->intracoded) {
	/* Calculate all of the required prediction pointers and how many
	 * there are (could be up to 8!!!). Then make the prediction and
	 * put result in destination.
	 */

	/* Get pointer to corresponding point in prediction fields */
	prev_top_ptr = &(prev_top->firstByte[((prev_top->parentWidth*
					     (8*mb_v_size*mby))+
					    (8*mb_h_size*mbx))]);
	prev_bot_ptr = &(prev_bot->firstByte[((prev_bot->parentWidth*
					     (8*mb_v_size*mby))+
					    (8*mb_h_size*mbx))]);

	future_top_ptr = &(future_top->firstByte[((future_top->parentWidth*
						 (8*mb_v_size*mby))+
						(8*mb_h_size*mbx))]);
	future_bot_ptr = &(future_bot->firstByte[((future_bot->parentWidth*
						 (8*mb_v_size*mby))+
						(8*mb_h_size*mbx))]);


	if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FIELD_MOTION_TYPE_FIELD) {
	  /* Simple field prediction. Could be 1, 2, 3, 4, 6, or 8, 
	   * prediction pointers depending on half-pixel granularity
	   * and forward or backward motion.
	   */
	  
	  if (Mpeg2MBInfoGetMFwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);

	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }

	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 0)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_top_ptr,
					 prev_top->parentWidth);
	      s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_bot_ptr,
					 prev_bot->parentWidth);
	      s1_w = s2_w = s3_w = s4_w = prev_bot->parentWidth;
	    }
	  } else {
	    num_fwd_preds = 0;
	  }
	  
	  if (Mpeg2MBInfoGetMBwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 1);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 1);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 1)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_top_ptr,
					 future_top->parentWidth);
	      s5_w = s6_w = s7_w = s8_w = future_top->parentWidth;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_bot_ptr,
					 future_bot->parentWidth);
	      s5_w = s6_w = s7_w = s8_w = future_bot->parentWidth;
	    }
	  } else {
	    num_bwd_preds = 0;
	  }

	  switch (num_fwd_preds) {
	  case 0:
	    s1 = s5; s1_w = s5_w;
	    s2 = s6; s2_w = s6_w;
	    s3 = s7; s3_w = s7_w;
	    s4 = s8; s4_w = s8_w;
	    break;
	  case 1:
	    if (num_bwd_preds == 2) {
	      s3 = s1; s3_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	    } else if (num_bwd_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1; t1_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	    } else {
	      s2 = s5; s2_w = s5_w;
	    }
	    break;
	  case 2:
	    if (num_bwd_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;
	      
	      t1 = s1; t1_w = s1_w;
	      t2 = s2; t2_w = s2_w;

	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	      s6 = t2; s6_w = t2_w;
	    } else {
	      s3 = s5; s3_w = s5_w;
	      s4 = s6; s4_w = s6_w;
	      s5 = s7; s5_w = s7_w;
	      s6 = s8; s6_w = s8_w;
	    }
	    break;
	  }
	  num_preds = num_fwd_preds + num_bwd_preds;
	  
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_8_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	    } else {
	      Do_8_X_16_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				   dest, dest_w);
	    }
	  } else {
	    Do_16_X_16_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	  }
	} else if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FIELD_MOTION_TYPE_16_X_8_MC) {
	  if (Mpeg2MBInfoGetMFwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 0)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_top_ptr,
					 prev_top->parentWidth);
	      s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_bot_ptr,
					 prev_bot->parentWidth);
	      s1_w = s2_w = s3_w = s4_w = prev_bot->parentWidth;
	    }
	  } else {
	    num_fwd_preds = 0;
	  }

	  if (Mpeg2MBInfoGetMBwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 1);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 1);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 1)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_top_ptr,
					 future_top->parentWidth);
	      s5_w = s6_w = s7_w = s8_w = future_top->parentWidth;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_bot_ptr,
					 future_bot->parentWidth);
	      s5_w = s6_w = s7_w = s8_w = future_bot->parentWidth;
	    }
	  } else {
	    num_bwd_preds = 0;
	  }

	  switch (num_fwd_preds) {
	  case 0:
	    s1 = s5; s1_w = s5_w;
	    s2 = s6; s2_w = s6_w;
	    s3 = s7; s3_w = s7_w;
	    s4 = s8; s4_w = s8_w;
	    break;
	  case 1:
	    if (num_bwd_preds == 2) {
	      s3 = s1; s3_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	    } else if (num_bwd_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1; t1_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	    } else {
	      s2 = s5; s2_w = s5_w;
	    }
	    break;
	  case 2:
	    if (num_bwd_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;
	      
	      t1 = s1; t1_w = s1_w;
	      t2 = s2; t2_w = s2_w;

	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	      s6 = t2; s6_w = t2_w;
	    } else {
	      s3 = s5; s3_w = s5_w;
	      s4 = s6; s4_w = s6_w;
	      s5 = s7; s5_w = s7_w;
	      s6 = s8; s6_w = s8_w;
	    }
	    break;
	  }
	  num_preds = num_fwd_preds + num_bwd_preds;
	  
	  
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,    
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	    } else {
	      Do_8_X_8_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,    
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds, 
				 s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				 s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				 dest, dest_w);
	  }
	  
	  if (Mpeg2MBInfoGetMFwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 0);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 0);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 1, 0)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_top_ptr,
					 prev_top->parentWidth);
	      s1_w = s2_w = s3_w = s4_w = prev_top->parentWidth;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_bot_ptr,
					 prev_bot->parentWidth);
	      s1_w = s2_w = s3_w = s4_w = prev_bot->parentWidth;
	    }
	  } else {
	    num_fwd_preds = 0;
	  }
	  
	  if (Mpeg2MBInfoGetMBwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 1);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 1);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 1, 1)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_top_ptr,
					 future_top->parentWidth);
	      s5_w = s6_w = s7_w = s8_w = future_top->parentWidth;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_bot_ptr,
					 future_bot->parentWidth);
	      s5_w = s6_w = s7_w = s8_w = future_bot->parentWidth;
	    }
	  } else {
	    num_bwd_preds = 0;
	  }

	  switch (num_fwd_preds) {
	  case 0:
	    s1 = s5; s1_w = s5_w;
	    s2 = s6; s2_w = s6_w;
	    s3 = s7; s3_w = s7_w;
	    s4 = s8; s4_w = s8_w;
	    break;
	  case 1:
	    if (num_bwd_preds == 2) {
	      s3 = s1; s3_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	    } else if (num_bwd_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1; t1_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	    } else {
	      s2 = s5; s2_w = s5_w;
	    }
	    break;
	  case 2:
	    if (num_bwd_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;
	      
	      t1 = s1; t1_w = s1_w;
	      t2 = s2; t2_w = s2_w;

	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	      s6 = t2; s6_w = t2_w;
	    } else {
	      s3 = s5; s3_w = s5_w;
	      s4 = s6; s4_w = s6_w;
	      s5 = s7; s5_w = s7_w;
	      s6 = s8; s6_w = s8_w;
	    }
	    break;
	  }
	  num_preds = num_fwd_preds + num_bwd_preds;
	  
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1+(s1_w*4), s1_w, s2+(s2_w*4), s2_w, 
				  s3+(s3_w*4), s3_w, s4+(s4_w*4), s4_w,
				  s5+(s5_w*4), s5_w, s6+(s6_w*4), s6_w, 
				  s7+(s7_w*4), s7_w, s8+(s8_w*4), s8_w,
				  dest+(dest_w*4), dest_w);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				  s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, 
				  s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w,
				  s5+(s5_w*8), s5_w, s6+(s6_w*8), s6_w, 
				  s7+(s7_w*8), s7_w, s8+(s8_w*8), s8_w,
				  dest+(dest_w*8), dest_w);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				 s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, 
				 s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w,
				 s5+(s5_w*8), s5_w, s6+(s6_w*8), s6_w, 
				 s7+(s7_w*8), s7_w, s8+(s8_w*8), s8_w,
				 dest+(dest_w*8), dest_w);
	  }
	}
      }

      if (!scb->skipMB) {
	for (iy=0; iy<mb_v_size; iy++) {
	  if (iy == 1) {
	    scb += sc_image->parentWidth;
	    dest += dest_image->parentWidth*8;
	  }
	  for (ix=0; ix<mb_h_size; ix++) {
	    /* Adjust sc block pointer and dest pointers for
	     * which block in macroblock we are doing.
	     * This is a bit tricky due to different chrominance
	     * formats.
	     */
	    if (ix == 1) {
	      scb += 1;
	      dest += 8;
	    }
	    if (!scb->skipBlock) {
	      if (scb->intracoded) {
		/* No prediction, just DCT */
		DO_IDCT(scb, temp);
		BlockShortToByteCopy(temp, dest, dest_w);
	      } else {
		/* Both prediction and DCT */
		DO_IDCT(scb, temp);
		BlockShortToByteAdd(temp, dest, dest_w);
	      }
	    }
	  }
	  if (ix == 2) {
	    scb -= 1;
	    dest -= 8;
	  }
	}
      }
    }
  }
  return 1;
}  
  


int
Mpeg2ScBFrameToByte(ScImage *sc_image,
		    Mpeg2MBInfoArray *mbi_array,
		    ByteImage *prev,
		    ByteImage *future,
		    ByteImage *dest_image,
		    int lum_flag,
		    int non_420_flag)
{
  int mb_width, mb_height;
  int mby, mbx;
  Mpeg2MBInfo *mbi;
  ScBlock *scb;
  short temp[64];
  unsigned char *prev_ptr;
  unsigned char *future_ptr;
  int prev_w, future_w;
  int dest_w;
  int mb_addr;
  unsigned char *dest;
  short v0_h, v0_v;
  int num_preds, num_fwd_preds, num_bwd_preds;
  unsigned char *s1, *s2, *s3, *s4, *s5, *s6, *s7, *s8;
  int s1_w, s2_w, s3_w, s4_w, s5_w, s6_w, s7_w, s8_w;
  int ix, iy;
  int sc_width, sc_height, mb_h_size, mb_v_size;
  int field_dct_flag;

  mb_width = Mpeg2MBInfoArrayGetWidth(mbi_array);
  mb_height = Mpeg2MBInfoArrayGetHeight(mbi_array);

  sc_width = sc_image->width;
  sc_height = sc_image->height;

  /* Using mb width and height and sc width and height, 
   * infer chrominance formatting and set mb_h_size and
   * mb_v_size as appropriate.
   */
  if (mb_width == sc_width) {
    mb_h_size = 1;
  } else if (mb_width*2 == sc_width) {
    mb_h_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }
  if (mb_height == sc_height) {
    mb_v_size = 1;
  } else if (mb_height*2 == sc_height) {
    mb_v_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }

  dest_w = dest_image->parentWidth;
  prev_w = prev->parentWidth;
  future_w = future->parentWidth;

  for (mby = 0; mby < mb_height; mby++) {
    for (mbx = 0; mbx < mb_width; mbx++) {
      /* Get macroblock info */

      mb_addr = mby * mb_width + mbx;
      mbi = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);
      
      /* Get pointer to appropriate block in ScImage */
      
      scb = &(sc_image->firstBlock[sc_image->parentWidth*(mb_v_size*mby)+
				  (mb_h_size*mbx)]);

      /* Get pointer to appropriate dest byte */
      dest = &(dest_image->firstByte[((dest_w*(8*mb_v_size*mby))+
				      (8*mb_h_size*mbx))]);

      if (!scb->intracoded) {
	/* Calculate all of the required prediction pointers and how many
	 * there are (could be up to 8!!!). Then make the prediction and
	 * put result in destination.
	 */

	/* Get pointer to corresponding point in prediction fields */
	prev_ptr = &(prev->firstByte[((prev_w*(8*mb_v_size*mby))+
				      (8*mb_h_size*mbx))]);
	future_ptr = &(future->firstByte[((future_w*(8*mb_v_size*mby))+
					  (8*mb_h_size*mbx))]);
	
	if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FRAME_MOTION_TYPE_FIELD) {
	  /* Field prediction in a frame. */
	  /* First do top field. */
	  if (Mpeg2MBInfoGetMFwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 0)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_ptr,
					 prev_w*2);
	      s1_w = s2_w = s3_w = s4_w = prev_w*2;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_ptr+prev_w,
					 prev_w*2);
	      s1_w = s2_w = s3_w = s4_w = prev_w*2;
	    }
	  } else {
	    num_fwd_preds = 0;
	  }
	  if (Mpeg2MBInfoGetMBwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 1);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 1);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }

	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 0, 1)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_ptr,
					 future_w*2);
	      s5_w = s6_w = s7_w = s8_w = future_w*2;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_ptr+future_w,
					 future_w*2);
	      s5_w = s6_w = s7_w = s8_w = future_w*2;
	    }
	  } else {
	    num_bwd_preds = 0;
	  }

	  switch (num_fwd_preds) {
	  case 0:
	    s1 = s5; s1_w = s5_w;
	    s2 = s6; s2_w = s6_w;
	    s3 = s7; s3_w = s7_w;
	    s4 = s8; s4_w = s8_w;
	    break;
	  case 1:
	    if (num_bwd_preds == 2) {
	      s3 = s1; s3_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	    } else if (num_bwd_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1; t1_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	    } else {
	      s2 = s5; s2_w = s5_w;
	    }
	    break;
	  case 2:
	    if (num_bwd_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;
	      
	      t1 = s1; t1_w = s1_w;
	      t2 = s2; t2_w = s2_w;

	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	      s6 = t2; s6_w = t2_w;
	    } else {
	      s3 = s5; s3_w = s5_w;
	      s4 = s6; s4_w = s6_w;
	      s5 = s7; s5_w = s7_w;
	      s6 = s8; s6_w = s8_w;
	    }
	    break;
	  }
	  num_preds = num_fwd_preds + num_bwd_preds;

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w*2);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w*2);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w*2);
	  }
	  /* Now do bottom field. */
	  if (Mpeg2MBInfoGetMFwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 0);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 0);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 1, 0)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_ptr,
					 prev_w*2);
	      s1_w = s2_w = s3_w = s4_w = prev_w*2;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
					 &s1, &s2, &s3, &s4,
					 prev_ptr+prev_w,
					 prev_w*2);
	      s1_w = s2_w = s3_w = s4_w = prev_w*2;
	    }
	  } else {
	    num_fwd_preds = 0;
	  }
	  if (Mpeg2MBInfoGetMBwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 1);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 1);
	    
	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }
	    
	    if (!Mpeg2MBInfoGetMVFieldSelect(mbi, 1, 1)) {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_ptr,
					 future_w*2);
	      s5_w = s6_w = s7_w = s8_w = future_w*2;
	    } else {
	      DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
					 &s5, &s6, &s7, &s8,
					 future_ptr+future_w,
					 future_w*2);
	      s5_w = s6_w = s7_w = s8_w = future_w*2;
	    }
	  } else {
	    num_bwd_preds = 0;
	  }

	  switch (num_fwd_preds) {
	  case 0:
	    s1 = s5; s1_w = s5_w;
	    s2 = s6; s2_w = s6_w;
	    s3 = s7; s3_w = s7_w;
	    s4 = s8; s4_w = s8_w;
	    break;
	  case 1:
	    if (num_bwd_preds == 2) {
	      s3 = s1; s3_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	    } else if (num_bwd_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1; t1_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	    } else {
	      s2 = s5; s2_w = s5_w;
	    }
	    break;
	  case 2:
	    if (num_bwd_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;
	      
	      t1 = s1; t1_w = s1_w;
	      t2 = s2; t2_w = s2_w;

	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	      s6 = t2; s6_w = t2_w;
	    } else {
	      s3 = s5; s3_w = s5_w;
	      s4 = s6; s4_w = s6_w;
	      s5 = s7; s5_w = s7_w;
	      s6 = s8; s6_w = s8_w;
	    }
	    break;
	  }
	  num_preds = num_fwd_preds + num_bwd_preds;

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest+dest_w, dest_w*2);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest+dest_w, dest_w*2);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds, 
				 s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				 s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				 dest+dest_w, dest_w*2);
	  }
	} else if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FRAME_MOTION_TYPE_FRAME) {
	  /* Simple frame prediction. Could be 1, 2, 3, 4, 5, 6, or 8, 
	   * prediction pointers depending on half-pixel granularity
	   * and forward or backward motion.
	   */
	  
	  if (Mpeg2MBInfoGetMFwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);

	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }

	    DeterminePredictionSources(v0_h, v0_v, &num_fwd_preds, 
				       &s1, &s2, &s3, &s4,
				       prev_ptr,
				       prev_w);
	    s1_w = s2_w = s3_w = s4_w = prev_w;
	  } else {
	    num_fwd_preds = 0;
	  }

	  if (Mpeg2MBInfoGetMBwd(mbi)) {
	    v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 1);
	    v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 1);

	    if (mb_h_size == 1) {
	      v0_h >>= 1;
	    }
	    if (mb_v_size == 1) {
	      v0_v >>= 1;
	    }

	    DeterminePredictionSources(v0_h, v0_v, &num_bwd_preds, 
				       &s5, &s6, &s7, &s8,
				       future_ptr,
				       future_w);
	    s5_w = s6_w = s7_w = s8_w = future_w;
	  } else {
	    num_bwd_preds = 0;
	  }

	  switch (num_fwd_preds) {
	  case 0:
	    s1 = s5; s1_w = s5_w;
	    s2 = s6; s2_w = s6_w;
	    s3 = s7; s3_w = s7_w;
	    s4 = s8; s4_w = s8_w;
	    break;
	  case 1:
	    if (num_bwd_preds == 2) {
	      s3 = s1; s3_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	    } else if (num_bwd_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1; t1_w = s1_w;
	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	    } else {
	      s2 = s5; s2_w = s5_w;
	    }
	    break;
	  case 2:
	    if (num_bwd_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;
	      
	      t1 = s1; t1_w = s1_w;
	      t2 = s2; t2_w = s2_w;

	      s1 = s5; s1_w = s5_w;
	      s2 = s6; s2_w = s6_w;
	      s3 = s7; s3_w = s7_w;
	      s4 = s8; s4_w = s8_w;
	      s5 = t1; s5_w = t1_w;
	      s6 = t2; s6_w = t2_w;
	    } else {
	      s3 = s5; s3_w = s5_w;
	      s4 = s6; s4_w = s6_w;
	      s5 = s7; s5_w = s7_w;
	      s6 = s8; s6_w = s8_w;
	    }
	    break;
	  }
	  num_preds = num_fwd_preds + num_bwd_preds;

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_8_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	    } else {
	      Do_8_X_16_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	    }
	  } else {
	    Do_16_X_16_Prediction(num_preds, 
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w);
	  }
	}
      }

      if (!scb->skipMB) {

	if (Mpeg2MBInfoGetDCTType(mbi) && (lum_flag || non_420_flag)) {
	  field_dct_flag = 1;
	} else {
	  field_dct_flag = 0;
	}

	for (iy=0; iy<mb_v_size; iy++) {
	  if (iy == 1) {
	    scb += sc_image->parentWidth;
	    if (field_dct_flag) {
	      dest += dest_image->parentWidth;
	    } else {
	      dest += dest_image->parentWidth*8;
	    }
	  }
	  for (ix=0; ix<mb_h_size; ix++) {
	    /* Adjust sc block pointer and dest pointers for
	     * which block in macroblock we are doing.
	     * This is a bit tricky due to different chrominance
	     * formats.
	     */
	    if (ix == 1) {
	      scb += 1;
	      dest += 8;
	    }
	    if (!scb->skipBlock) {
	      if (scb->intracoded) {
		/* No prediction, just DCT */
		DO_IDCT(scb, temp);
		if (field_dct_flag) {
		  BlockShortToByteCopy(temp, dest, dest_w*2);
		} else {
		  BlockShortToByteCopy(temp, dest, dest_w);
		}
	      } else {
		/* Both prediction and DCT */
		DO_IDCT(scb, temp);
		if (field_dct_flag) {
		  BlockShortToByteAdd(temp, dest, dest_w*2);
		} else {
		  BlockShortToByteAdd(temp, dest, dest_w);
		}
	      }
	    }
	  }
	  if (ix == 2) {
	    scb -= 1;
	    dest -= 8;
	  }
	}
      }
    }
  }
  return 1;
}  
      

static void
Do_16_X_16_Prediction(int num_preds,
		      unsigned char *s1, int s1_w,
		      unsigned char *s2, int s2_w,
		      unsigned char *s3, int s3_w,
		      unsigned char *s4, int s4_w,
		      unsigned char *s5, int s5_w,
		      unsigned char *s6, int s6_w,
		      unsigned char *s7, int s7_w,
		      unsigned char *s8, int s8_w,
		      unsigned char *dest, int dest_w)
{
  switch (num_preds) {
  case 1:
    Block_Copy_16_X_8(s1, s1_w, dest, dest_w);
    Block_Copy_16_X_8(s1+(s1_w*8), s1_w, dest+(dest_w*8), dest_w);
    break;
  case 2:
    Block_Avg2_16_X_8(s1, s1_w, s2, s2_w, dest, dest_w);
    Block_Avg2_16_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, dest+(dest_w*8), dest_w);
    break;
  case 3:
    Block_Avg3_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, dest, dest_w);
    Block_Avg3_16_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, dest+(dest_w*8), dest_w);
    break;
  case 4:
    Block_Avg4_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, dest, dest_w);
    Block_Avg4_16_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, dest+(dest_w*8), dest_w);
    break;
  case 5:
    Block_Avg5_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, dest, dest_w);
    Block_Avg5_16_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, s5+(s5_w*8), s5_w, dest+(dest_w*8), dest_w);
    break;
  case 6:
    Block_Avg6_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, dest, dest_w);
    Block_Avg6_16_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, s5+(s5_w*8), s5_w, s6+(s6_w*8), s6_w, dest+(dest_w*8), dest_w);
    break;
  case 8:
    Block_Avg8_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w, dest, dest_w);
    Block_Avg8_16_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, s5+(s5_w*8), s5_w, s6+(s6_w*8), s6_w, s7+(s7_w*8), s7_w, s8+(s8_w*8), s8_w, dest+(dest_w*8), dest_w);
    break;
  }
}

static void
Do_16_X_8_Prediction(int num_preds,
		     unsigned char *s1, int s1_w,
		     unsigned char *s2, int s2_w,
		     unsigned char *s3, int s3_w,
		     unsigned char *s4, int s4_w,
		     unsigned char *s5, int s5_w,
		     unsigned char *s6, int s6_w,
		     unsigned char *s7, int s7_w,
		     unsigned char *s8, int s8_w,
		     unsigned char *dest, int dest_w)
{
  switch (num_preds) {
  case 1:
    Block_Copy_16_X_8(s1, s1_w, dest, dest_w);
    break;
  case 2:
    Block_Avg2_16_X_8(s1, s1_w, s2, s2_w, dest, dest_w);
    break;
  case 3:
    Block_Avg3_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, dest, dest_w);
    break;
  case 4:
    Block_Avg4_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, dest, dest_w);
    break;
  case 5:
    Block_Avg5_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, dest, dest_w);
    break;
  case 6:
    Block_Avg6_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, dest, dest_w);
    break;
  case 8:
    Block_Avg8_16_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w, dest, dest_w);
    break;
  }
}

static void
Do_8_X_16_Prediction(int num_preds,
		     unsigned char *s1, int s1_w,
		     unsigned char *s2, int s2_w,
		     unsigned char *s3, int s3_w,
		     unsigned char *s4, int s4_w,
		     unsigned char *s5, int s5_w,
		     unsigned char *s6, int s6_w,
		     unsigned char *s7, int s7_w,
		     unsigned char *s8, int s8_w,
		     unsigned char *dest, int dest_w)
{
  switch (num_preds) {
  case 1:
    Block_Copy_8_X_8(s1, s1_w, dest, dest_w);
    Block_Copy_8_X_8(s1+(s1_w*8), s1_w, dest+(dest_w*8), dest_w);
    break;
  case 2:
    Block_Avg2_8_X_8(s1, s1_w, s2, s2_w, dest, dest_w);
    Block_Avg2_8_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, dest+(dest_w*8), dest_w);
    break;
  case 3:
    Block_Avg3_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, dest, dest_w);
    Block_Avg3_8_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, dest+(dest_w*8), dest_w);
    break;
  case 4:
    Block_Avg4_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, dest, dest_w);
    Block_Avg4_8_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, dest+(dest_w*8), dest_w);
    break;
  case 5:
    Block_Avg5_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, dest, dest_w);
    Block_Avg5_8_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, s5+(s5_w*8), s5_w, dest+(dest_w*8), dest_w);
    break;
  case 6:
    Block_Avg6_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, dest, dest_w);
    Block_Avg6_8_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, s5+(s5_w*8), s5_w, s6+(s6_w*8), s6_w, dest+(dest_w*8), dest_w);
    break;
  case 8:
    Block_Avg8_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w, dest, dest_w);
    Block_Avg8_8_X_8(s1+(s1_w*8), s1_w, s2+(s2_w*8), s2_w, s3+(s3_w*8), s3_w, s4+(s4_w*8), s4_w, s5+(s5_w*8), s5_w, s6+(s6_w*8), s6_w, s7+(s7_w*8), s7_w, s8+(s8_w*8), s8_w, dest+(dest_w*8), dest_w);
    break;
  }
}

static void
Do_8_X_8_Prediction(int num_preds,
		    unsigned char *s1, int s1_w,
		    unsigned char *s2, int s2_w,
		    unsigned char *s3, int s3_w,
		    unsigned char *s4, int s4_w,
		    unsigned char *s5, int s5_w,
		    unsigned char *s6, int s6_w,
		    unsigned char *s7, int s7_w,
		    unsigned char *s8, int s8_w,
		    unsigned char *dest, int dest_w)
{
  switch (num_preds) {
  case 1:
    Block_Copy_8_X_8(s1, s1_w, dest, dest_w);
    break;
  case 2:
    Block_Avg2_8_X_8(s1, s1_w, s2, s2_w, dest, dest_w);
    break;
  case 3:
    Block_Avg3_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, dest, dest_w);
    break;
  case 4:
    Block_Avg4_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, dest, dest_w);
    break;
  case 5:
    Block_Avg5_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, dest, dest_w);
    break;
  case 6:
    Block_Avg6_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, dest, dest_w);
    break;
  case 8:
    Block_Avg8_8_X_8(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w, dest, dest_w);
    break;
  }
}

static void
Do_8_X_4_Prediction(int num_preds,
		    unsigned char *s1, int s1_w,
		    unsigned char *s2, int s2_w,
		    unsigned char *s3, int s3_w,
		    unsigned char *s4, int s4_w,
		    unsigned char *s5, int s5_w,
		    unsigned char *s6, int s6_w,
		    unsigned char *s7, int s7_w,
		    unsigned char *s8, int s8_w,
		    unsigned char *dest, int dest_w)
{
  switch (num_preds) {
  case 1:
    Block_Copy_8_X_4(s1, s1_w, dest, dest_w);
    break;
  case 2:
    Block_Avg2_8_X_4(s1, s1_w, s2, s2_w, dest, dest_w);
    break;
  case 3:
    Block_Avg3_8_X_4(s1, s1_w, s2, s2_w, s3, s3_w, dest, dest_w);
    break;
  case 4:
    Block_Avg4_8_X_4(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, dest, dest_w);
    break;
  case 5:
    Block_Avg5_8_X_4(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, dest, dest_w);
    break;
  case 6:
    Block_Avg6_8_X_4(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, dest, dest_w);
    break;
  case 8:
    Block_Avg8_8_X_4(s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w, s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w, dest, dest_w);
    break;
  }
}

int
Mpeg2ScPFrameToByte(ScImage *sc_image,
		    Mpeg2MBInfoArray *mbi_array,
		    ByteImage *prev,
		    ByteImage *dest_image,
		    int lum_flag,
		    int non_420_flag)
{
  int mb_width, mb_height;
  int mby, mbx;
  Mpeg2MBInfo *mbi;
  ScBlock *scb;
  short temp[64];
  unsigned char *prev_ptr;
  int dest_w, prev_w;
  int mb_addr;
  unsigned char *dest;
  short v0_h, v0_v;
  short dual_top_h, dual_top_v, dual_bot_h, dual_bot_v;
  int num_preds,  num_dual_preds;
  unsigned char *s1, *s2, *s3, *s4, *s5, *s6, *s7, *s8;
  int s1_w, s2_w, s3_w, s4_w, s5_w, s6_w, s7_w, s8_w;
  int ix, iy;
  int mb_h_size, mb_v_size;
  int sc_width, sc_height;
  int field_dct_flag;
  
  mb_width = Mpeg2MBInfoArrayGetWidth(mbi_array);
  mb_height = Mpeg2MBInfoArrayGetHeight(mbi_array);
  
  sc_width = sc_image->width;
  sc_height = sc_image->height;

  /* Using mb width and height and sc width and height, 
   * infer chrominance formatting and set mb_h_size and
   * mb_v_size as appropriate.
   */
  if (mb_width == sc_width) {
    mb_h_size = 1;
  } else if (mb_width*2 == sc_width) {
    mb_h_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }
  if (mb_height == sc_height) {
    mb_v_size = 1;
  } else if (mb_height*2 == sc_height) {
    mb_v_size = 2;
  } else {
    /*Should never get here.*/
    assert(0);
  }

  dest_w = dest_image->parentWidth;
  prev_w = prev->parentWidth;

  for (mby = 0; mby < mb_height; mby++) {
    for (mbx = 0; mbx < mb_width; mbx++) {
      /* Get macroblock info */

      mb_addr = mby * mb_width + mbx;
      mbi = Mpeg2MBInfoArrayGetMBInfoEntry(mbi_array, mb_addr);
      
      /* Get pointer to appropriate block in ScImage */
      
      scb = &(sc_image->firstBlock[sc_image->parentWidth*(mby*mb_v_size)
				  +(mbx*mb_h_size)]);

      /* Get pointer to appropriate dest byte */
      dest = &(dest_image->firstByte[((dest_w*
				       (8*mb_v_size*mby))+
				      (8*mb_h_size*mbx))]);

      if (!scb->intracoded) {

	/* Get pointer to corresponding point in prediction fields */
	prev_ptr = &(prev->firstByte[((prev_w*
					   (8*mb_v_size*mby))+
					  (8*mb_h_size*mbx))]);
	
	/* Calculate all of the required prediction pointers and how many
	 * there are (could be up to 8!!!)
	 */
	
	if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FRAME_MOTION_TYPE_FIELD) {
	  /* Field prediction in a frame. 
	   * Could be 1, 2, or 4, prediction pointers
	   * depending on half-pixel granularity. 
	   * Separate predicitions for top and bottom fields.
	   */
	  
	  /* Do top first */
	  v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	  v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);

	  if (mb_h_size == 1) {
	    v0_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    v0_v >>= 1;
	  }

	  DeterminePredictionSources(v0_h, v0_v, &num_preds,
				     &s1, &s2, &s3, &s4,
				     prev_ptr,
				     prev_w*2);
	  s1_w = s2_w = s3_w = s4_w = prev_w*2;
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w*2);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				   dest, dest_w*2);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w*2);
	  }

	  /* Do bottom */
	  v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 0);
	  v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 0);

	  if (mb_h_size == 1) {
	    v0_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    v0_v >>= 1;
	  }

	  DeterminePredictionSources(v0_h, v0_v, &num_preds,
				     &s1, &s2, &s3, &s4,
				     prev_ptr+prev_w,
				     prev_w*2);
	  s1_w = s2_w = s3_w = s4_w = prev_w*2;
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest+dest_w, dest_w*2);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest+dest_w, dest_w*2);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				 s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				 NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				 dest+dest_w, dest_w*2);
	  }
	} else if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FRAME_MOTION_TYPE_FRAME) {
	  /* Simple frame motion. */
	  /* 1, 2, or 4 prediction sources. Forward only since this is 
	   * a P frame 
	   */

	  v0_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	  v0_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);

	  if (mb_h_size == 1) {
	    v0_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    v0_v >>= 1;
	  }
	  
	  DeterminePredictionSources(v0_h, v0_v, &num_preds, 
				     &s1, &s2, &s3, &s4,
				     prev_ptr,
				     prev_w);
	  s1_w = s2_w = s3_w = s4_w = prev_w;

	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w);
	    } else {
	      Do_8_X_16_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				   dest, dest_w);
	    }
	  } else {
	    Do_16_X_16_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				  dest, dest_w);
	  }
	} else if (Mpeg2MBInfoGetMotionType(mbi) == MPEG2_FRAME_MOTION_TYPE_DUAL_PRIME) {
	  /* Dual vectors have already been calculated and are stored
	   * in mb info in the following way:
	   *
	   * For the prediction of the top field:
	   *     Vector for top ref. field in 0,0
	   *     Vector for bot ref. field in 1,0
	   * For the prediction of the bot field:
	   *     Vector for top ref. field in 0,1
	   *     Vector for bot ref. field in 1,1
	   */

	  /* Do top first. */

	  dual_top_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 0);
	  dual_top_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 0);
	  dual_bot_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 0);
	  dual_bot_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 0);
	  
	  if (mb_h_size == 1) {
	    dual_top_h >>= 1;
	    dual_bot_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    dual_top_v >>= 1;
	    dual_bot_v >>= 1;
	  }
	  
	  DeterminePredictionSources(dual_top_h, dual_top_v, &num_preds, 
				     &s1, &s2, &s3, &s4,
				     prev_ptr,
				     prev_w*2);
	  s1_w = s2_w = s3_w = s4_w = prev_w*2;
	  DeterminePredictionSources(dual_bot_h, dual_bot_v, &num_dual_preds,
				     &s5, &s6, &s7, &s8,
				     prev_ptr,
				     prev_w*2);
	  s5_w = s6_w = s7_w = s8_w = prev_w*2;
	  
	  if (num_preds == 1) {
	    if (num_dual_preds == 2) {
	      s3 = s1;
	      s1 = s5;
	      s2 = s6;
	      s3_w = s1_w;
	      s1_w = s5_w;
	      s2_w = s6_w;
	    } else if (num_dual_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1;
	      t1_w = s1_w;
	      s1 = s5;
	      s2 = s6;
	      s3 = s7;
	      s4 = s8;
	      s5 = t1;
	      s1_w = s5_w;
	      s2_w = s6_w;
	      s3_w = s7_w;
	      s4_w = s8_w;
	      s5_w = t1_w;
	    } else {
	      s2 = s5;
	      s2_w = s5_w;
	    }
	  } else if (num_preds == 2) {
	    if (num_dual_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;

	      t1 = s1;
	      t2 = s2;
	      t1_w = s1_w;
	      t2_w = s2_w;

	      s1 = s5;
	      s2 = s6;
	      s3 = s7;
	      s4 = s8;
	      s5 = t1;
	      s6 = t2;
	      s1_w = s5_w;
	      s2_w = s6_w;
	      s3_w = s7_w;
	      s4_w = s8_w;
	      s5_w = t1_w;
	      s6_w = t2_w;
	    } else {
	      s3 = s5;
	      s4 = s6;
	      s5 = s7;
	      s6 = s8;
	      s3_w = s5_w;
	      s4_w = s6_w;
	      s5_w = s7_w;
	      s6_w = s8_w;
	    }
	  }
	  num_preds += num_dual_preds;
	  
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w*2);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				   dest, dest_w*2);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest, dest_w*2);
	  }

	  /* Do bottom */

	  dual_top_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 0, 1);
	  dual_top_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 0, 1);
	  dual_bot_h = Mpeg2MBInfoGetMVectorHorizComp(mbi, 1, 1);
	  dual_bot_v = Mpeg2MBInfoGetMVectorVertComp(mbi, 1, 1);
	  
	  if (mb_h_size == 1) {
	    dual_top_h >>= 1;
	    dual_bot_h >>= 1;
	  }
	  if (mb_v_size == 1) {
	    dual_top_v >>= 1;
	    dual_bot_v >>= 1;
	  }
	  
	  DeterminePredictionSources(dual_top_h, dual_top_v, &num_preds, 
				     &s1, &s2, &s3, &s4,
				     prev_ptr+prev_w,
				     prev_w*2);
	  s1_w = s2_w = s3_w = s4_w = prev_w*2;
	  DeterminePredictionSources(dual_bot_h, dual_bot_v, &num_dual_preds,
				     &s5, &s6, &s7, &s8,
				     prev_ptr+prev_w,
				     prev_w*2);
	  s5_w = s6_w = s7_w = s8_w = prev_w*2;
	  

	  if (num_preds == 1) {
	    if (num_dual_preds == 2) {
	      s3 = s1;
	      s1 = s5;
	      s2 = s6;
	      s3_w = s1_w;
	      s1_w = s5_w;
	      s2_w = s6_w;
	    } else if (num_dual_preds == 4) {
	      unsigned char *t1;
	      int t1_w;

	      t1 = s1;
	      t1_w = s1_w;
	      s1 = s5;
	      s2 = s6;
	      s3 = s7;
	      s4 = s8;
	      s5 = t1;
	      s1_w = s5_w;
	      s2_w = s6_w;
	      s3_w = s7_w;
	      s4_w = s8_w;
	      s5_w = t1_w;
	    } else {
	      s2 = s5;
	      s2_w = s5_w;
	    }
	  } else if (num_preds == 2) {
	    if (num_dual_preds == 4) {
	      unsigned char *t1, *t2;
	      int t1_w, t2_w;

	      t1 = s1;
	      t2 = s2;
	      t1_w = s1_w;
	      t2_w = s2_w;

	      s1 = s5;
	      s2 = s6;
	      s3 = s7;
	      s4 = s8;
	      s5 = t1;
	      s6 = t2;
	      s1_w = s5_w;
	      s2_w = s6_w;
	      s3_w = s7_w;
	      s4_w = s8_w;
	      s5_w = t1_w;
	      s6_w = t2_w;
	    } else {
	      s3 = s5;
	      s4 = s6;
	      s5 = s7;
	      s6 = s8;
	      s3_w = s5_w;
	      s4_w = s6_w;
	      s5_w = s7_w;
	      s6_w = s8_w;
	    }
	  }
	  num_preds += num_dual_preds;

	  
	  if (mb_h_size == 1) {
	    if (mb_v_size == 1) {
	      Do_8_X_4_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest+dest_w, dest_w*2);
	    } else {
	      Do_8_X_8_Prediction(num_preds,
				   s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				   s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				   dest+dest_w, dest_w*2);
	    }
	  } else {
	    Do_16_X_8_Prediction(num_preds,
				  s1, s1_w, s2, s2_w, s3, s3_w, s4, s4_w,
				  s5, s5_w, s6, s6_w, s7, s7_w, s8, s8_w,
				  dest+dest_w, dest_w*2);
	  }
	}
      }

      if (Mpeg2MBInfoGetDCTType(mbi) && (lum_flag || non_420_flag)) {
	field_dct_flag = 1;
      } else {
	field_dct_flag = 0;
      }

      if (!scb->skipMB) {
	for (iy=0; iy<mb_v_size; iy++) {
	  if (iy == 1) {
	    scb += sc_image->parentWidth;
	    if (field_dct_flag) {
	      dest += dest_image->parentWidth;
	    } else {
	      dest += dest_image->parentWidth*8;
	    }
	  }
	  for (ix=0; ix<mb_h_size; ix++) {
	    /* Adjust sc block pointer and dest pointers for
	     * which block in macroblock we are doing.
	     * This is a bit tricky due to different chrominance
	     * formats.
	     */
	    if (ix == 1) {
	      scb += 1;
	      dest += 8;
	    }
	    if (!scb->skipBlock) {
	      if (scb->intracoded) {
		/* No prediction, just DCT */
		DO_IDCT(scb, temp);
		if (field_dct_flag) {
		  BlockShortToByteCopy(temp, dest, dest_w*2);
		} else {
		  BlockShortToByteCopy(temp, dest, dest_w);
		}
	      } else {
		/* Both prediction and DCT */
		DO_IDCT(scb, temp);
		if (field_dct_flag) {
		  BlockShortToByteAdd(temp, dest, dest_w*2);
		} else {
		  BlockShortToByteAdd(temp, dest, dest_w);
		}
	      }
	    }
	  }
	  if (ix == 2) {
	    scb -= 1;
	    dest -= 8;
	  }
	}
      }
    }
  }
  return 1;
}  
	  


